package cn.wise.sc.pay.common.core.model;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.common.core.config.CommonUrlProperties;
import cn.wise.sc.pay.common.core.util.FastDFSUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 公共设施api
 * @author: qh
 * @create: 2020-06-24 16:01
 **/
@Api(tags = "公共基础设施")
@CrossOrigin
@RestController
@RequestMapping("/common/base")
@EnableConfigurationProperties(CommonUrlProperties.class)
public class CommonController {

    final
    FastDFSUtils fastDFSUtils;
    final
    CommonUrlProperties commonUrlProperties;


    public CommonController(CommonUrlProperties commonUrlProperties, FastDFSUtils fastDFSUtils) {
        this.commonUrlProperties = commonUrlProperties;
        this.fastDFSUtils = fastDFSUtils;
    }

    @ApiOperation("获取基础路径")
    @GetMapping("/url")
    public BaseResponse<Map<String, String>> getBaseUrls() {

        Map<String, String> rts = MapUtil.newConcurrentHashMap(4);

        assert commonUrlProperties != null;

        rts.put("baseUrl", commonUrlProperties.getBaseUrl());
        rts.put("webViewUrl", commonUrlProperties.getWebViewUrl());
        rts.put("imgUrl", commonUrlProperties.getImageUrl());
        rts.put("rabbitUrl", commonUrlProperties.getRabbitUrl());

        return BaseResponse.ok(rts);
    }

    @PostMapping("/img/upload")
    @ApiOperation("上传图片")
    public BaseResponse upload(MultipartFile file) {

        Map<String, String> map = new HashMap<>();
        String path = "";
        if (file != null && StringUtils.isNotEmpty(file.getOriginalFilename())) {
            try {
                path = fastDFSUtils.uploadFile(file).getPath();
                map.put("host", fastDFSUtils.fullUrl(path));
            } catch (IOException e) {
                return BaseResponse.error("图片上传失败", map);
            }
        }
        map.put("uri", path);
        return BaseResponse.ok(map);
    }

    @ApiOperation("拼接图片host  参数body:{ relativeUrls:\\[]}")
    @PutMapping("/img/fullUrl")
    public BaseResponse getImgFullUrl(@RequestBody List<String> relativeUrls) {
        String imgBaseUrl = commonUrlProperties.getImageUrl();
        List<String> rts = new ArrayList<>(relativeUrls.size());
        if (StrUtil.isEmpty(imgBaseUrl)) {
            imgBaseUrl = "https://oxo.chfatech.com/";

        }
        for (String relativeUrl : relativeUrls) {
            rts.add(imgBaseUrl + relativeUrl);
        }
        return BaseResponse.ok(rts);
    }

}
