package cn.wise.sc.pay.common.core.util;

import cn.wise.sc.pay.common.core.eum.PictureTypeEnum;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.WriterException;
import com.google.zxing.client.j2se.MatrixToImageWriter;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.QRCodeWriter;
import org.apache.commons.lang3.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.FileSystems;
import java.nio.file.Path;

/**
 * @description: 二维码工具类
 * @author: qh
 * @create: 2019-11-25 10:47
 **/
public class QRCodeUtils {

    /**
     * 生成二维码  以图片的形式保存
     *
     * @param text     内容
     * @param width
     * @param height
     * @param filePath 路径
     * @throws WriterException
     * @throws IOException
     */
    public static void generateQRCodeImage(String text, int width, int height, String filePath) throws WriterException, IOException {

        QRCodeWriter qrCodeWriter = new QRCodeWriter();

        BitMatrix bitMatrix = qrCodeWriter.encode(text, BarcodeFormat.QR_CODE, width, height);

        Path path = FileSystems.getDefault().getPath(filePath);

        MatrixToImageWriter.writeToPath(bitMatrix, PictureTypeEnum.PNG.getType(), path);

    }

    /**
     * 生成二维码  以字节流的形式
     *
     * @param text
     * @param width
     * @param height
     * @return
     * @throws WriterException
     * @throws IOException
     */
    public static byte[] getQRCodeImage(String text, int width, int height) throws WriterException, IOException {

        QRCodeWriter qrCodeWriter = new QRCodeWriter();

        BitMatrix bitMatrix = qrCodeWriter.encode(text, BarcodeFormat.QR_CODE, width, height);

        ByteArrayOutputStream pngOutputStream = new ByteArrayOutputStream();

        MatrixToImageWriter.writeToStream(bitMatrix, PictureTypeEnum.PNG.getType(), pngOutputStream);

        byte[] pngData = pngOutputStream.toByteArray();

        return pngData;
    }


    /**
     *
     *
     * @param args
     */
    public static void main(String[] args) {
        try {
            generateQRCodeImage("This is my first QR Code", 350, 350, "/");
            byte[] QRCodebt = getQRCodeImage("This is my first QR Code", 350, 350);
            System.out.println(StringUtils.toEncodedString(QRCodebt, Charset.defaultCharset()));
        } catch (WriterException e) {
            System.out.println("Could not generate QR Code, WriterException :: " + e.getMessage());
        } catch (IOException e) {
            System.out.println("Could not generate QR Code, IOException :: " + e.getMessage());
        }
    }
}
