package cn.wise.sc.pay.common.security2.config;


import com.alibaba.druid.pool.DruidDataSource;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.core.JdbcTemplate;

import javax.sql.DataSource;

/**
 * @ClassNameJdbcConfig
 * @Description 数据配置
 * 由于本项目使用多租户系统 token生成策略会不用,可能使用多张表不同字段
 * 按需注入Bean对象
 * @Author qh
 * @Date2020/3/5 16:27
 * @Version V1.0
 **/
@Configuration
@ComponentScan("cn.wise.sc.pay.common.security2")
public class JdbcConfig {

    /**
     * 默认的数据库连接池
     * 用户验证走oxo_main_data中的用户表
     * @return
     */
    @Bean(name = "dataSourceToken")
    @ConditionalOnMissingBean(DataSource.class)
    public DataSource createDataSource() {
        DruidDataSource dataSource = new DruidDataSource();
        dataSource.setDriverClassName("com.mysql.cj.jdbc.Driver");
        dataSource.setUrl("jdbc:mysql://127.0.0.1:3306/oxo_main_data?useUnicode=true&characterEncoding=UTF-8&useSSL=false");
        dataSource.setUsername("root");
        dataSource.setPassword("admin!@#123");
        return dataSource;
    }

    /**
     * OpenApi的token验证
     * 使用的是设备绑定表里面的设备号登录  oxo_merchant库device表 deviceNo登录
     * @return
     */
    public static DataSource createDataSourceDevice(){
        DruidDataSource dataSource = new DruidDataSource();
        dataSource.setDriverClassName("com.mysql.cj.jdbc.Driver");
        dataSource.setUrl("jdbc:mysql://127.0.0.1:3306/oxo_merchant?useUnicode=true&characterEncoding=UTF-8&useSSL=false");
        dataSource.setUsername("root");
        dataSource.setPassword("admin!@#123");
        return dataSource;
    }

    /**
     * jdbcTemplate 注册
     * @param dataSourceToken
     * @return
     */
    @Bean(name = "jdbcTemplate")
    public JdbcTemplate createJdbcTemplate(DataSource dataSourceToken) {
        return new JdbcTemplate(dataSourceToken);
    }
}