package cn.wise.sc.pay.common.security2.config;

import cn.wise.sc.pay.common.security2.core.JwtAuthenticationEntryPoint;
import cn.wise.sc.pay.common.security2.core.JwtRequestFilter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

/**
 * @ClassNameWebSecurityConfig
 * @Description Security全局配置
 * @Author qh
 * @Date2020/3/5 15:21
 * @Version V1.0
 **/
@Configuration
@EnableWebSecurity
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    private final JwtAuthenticationEntryPoint jwtAuthenticationEntryPoint;
    private final UserDetailsService jwtUserDetailsService;
    private final JwtRequestFilter jwtRequestFilter;

    public WebSecurityConfig(JwtAuthenticationEntryPoint jwtAuthenticationEntryPoint, UserDetailsService jwtUserDetailsService, JwtRequestFilter jwtRequestFilter) {
        this.jwtAuthenticationEntryPoint = jwtAuthenticationEntryPoint;
        this.jwtUserDetailsService = jwtUserDetailsService;
        this.jwtRequestFilter = jwtRequestFilter;
    }

    @Autowired
    public void configureGlobal(AuthenticationManagerBuilder auth) throws Exception {
        // configure AuthenticationManager so that it knows from where to load
        // user for matching credentials
        // Use BCryptPasswordEncoder
        auth.userDetailsService(jwtUserDetailsService).passwordEncoder(passwordEncoder());
    }

    @Bean
    public PasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }


    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    @Override
    protected void configure(HttpSecurity httpSecurity) throws Exception {

        httpSecurity.csrf().disable()
//                .formLogin()
//                .loginProcessingUrl("/login")
//                .and()
                .exceptionHandling().authenticationEntryPoint(jwtAuthenticationEntryPoint).and().sessionManagement()
                .sessionCreationPolicy(SessionCreationPolicy.STATELESS) //关闭会话管理
                .and()
                //基于springSecurity过滤器的token实现
//                .addFilter(new JwtAuthenticationFilter(super.authenticationManager()))
//                .addFilter(new JwtTokenVerifyFilter(super.authenticationManager()))
                // dont authenticate this particular request
                .authorizeRequests()
                .antMatchers(
                        "/**/user/token",
                        "/**/login",
                        "/**/common/security/**",
                        "/**/common/base/**",
                        "/**/WX/getPhone",
                        "/**/byOpenId/**",
                        "/**/role/host/**",
                        "/**/swagger-ui.html",
                        "/swagger-resources/**",
                        "/v2/api-docs",
                        "/doc.html",
                        "/**/eshop/goods/conditionGoodsPage",
                        "/**/eshop/goods/detail/**",
                        "/**/eshop/type/typeTree",
                        "/**/eshop/brand/brands",
                        "/**/eshop/region/regionList/**",
                        "/**/register/**",
                        "/**/login/**",
                        "/**/wxRegister",
                        "/**/binding/phone",
                        "/**/wxOrAli/Register/**",
                        "/**/association/article/getArticleVerify",
                        "/**/association/article/selectById",
                        "/**/association/banner/list",
                        "/**/association/banner/rwList",
                        "/**/association/comment/commentPage",
                        "/**/association/banner/selectList",
                        "/**/association/topic/list",
                        "/**/merchant/banner/bannerImages",
                        "/**/merchant/menu/getPageMenus",
                        "/**/comment/commentPage",
                        "/**/goodsRecord/exchangeRecord",
                        "/**/goodsRecord/latestRecords",
                        "/**/pointSource/list/condition",
                        "/**/points/exchangeRecord",
                        "/**/points/list/condition",
                        "/**/points/latestRecords/**",
                        "/**/exhibition/advertising/list",
                        "/**/exhibition/comment/commentPage",
                        "/**/exhibition/list",
                        "/**/exhibition/commentPage",
                        "/**/app/cashier/shop/**",
                        "/**/app/exhibition",
                        "/**/app/eshop/goods/goodsPage/**",
                        "/**/app/merchant/transportation/**",
                        "/**/app/exhibition/detail/**",
                        "/**/association/article/viewNumUp/**",
                        "/**/excel/orderInfo/**",
                        "/webjars/**")
                .permitAll()
                .anyRequest()
                .authenticated()
                .and()
                .cors()
                .and()
                .addFilterBefore(jwtRequestFilter, UsernamePasswordAuthenticationFilter.class);

        // disable page caching
        httpSecurity
                .headers()
                .frameOptions().sameOrigin()  // required to set for H2 else H2 Console will be blank.
                .cacheControl();
    }

}