package cn.wise.sc.pay.common.security2.userDetail;

import cn.wise.sc.pay.common.security2.SysUser;
import cn.wise.sc.pay.common.security2.service.SysUserJdbcService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * JwtUserDetailsService
 * 实现UserDetailsService,重写loadUserByUsername方法
 */
@Slf4j
@Component
@ConditionalOnMissingBean(name = "userDetailsService")
public class JwtUserDetailsService implements UserDetailsService {

    final
    SysUserJdbcService sysUserJdbcService;

    @Value("${spring.datasource.dtn.userbac.prefix}")
    private String prefix;

    public JwtUserDetailsService(SysUserJdbcService sysUserJdbcService) {
        this.sysUserJdbcService = sysUserJdbcService;
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {

        //获取数据库用户信息
        List<SysUser> list = sysUserJdbcService.getSysUserByUsername(prefix + "sys_user", username);
        //todo 权限获取
        if (list.size() == 0) {
            log.warn("未找到当前用户:{} \n ====== cn.wise.sc.pay.common.security2.userDetail.JwtUserDetailsService.loadUserByUsername()", username);
            throw new UsernameNotFoundException("未找到当前用户!");
        }
        UserDetails userDetails = list.get(0);

        return userDetails;
    }
}
