package cn.wise.sc.pay.common.security2.util;


import cn.wise.sc.pay.common.security2.SysUser;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.util.Assert;

import java.util.Date;

/**
 * @ClassNameCustomJwtUtils
 * @Description 自定义JWT工具类
 * @Author qh
 * @Date2020/3/5 11:07
 * @Version V1.0
 **/
public class JwtUtils {

    /**
     * 小程序端 关键标识
     */
    public static final String APP = "APP";

    /**
     * 管理端  关键标识
     */
    public static final String MANAGE = "MANAGE";

    /**
     * 收银端 关键标识
     */
    public static final String CASHIER = "CASHIER";
    /**
     * 请求头
     */
    public static final String TOKEN_HEADER = "Authorization";
    /**
     * token前缀
     */
    public static final String TOKEN_PREFIX = "Bearer ";
    /**
     * 有效时间
     */
    public static final long EXPIRATION = 1000 * 24 * 60 * 60;
    /**
     * 密钥
     */
    public static final String SECRET_KEY = "JXU2MjExJXU1REYyJXU3OUJCJXU4MDRDJXVGRjBDJXU4QkY3JXU1MkZGJXU2MjUzJXU2Mjcw";

    /**
     * 生成Token
     *
     * @param user 用户信息
     * @return token
     */
    public static String generateUsersToken(SysUser user) {
        Assert.hasText(user.getUsername(), "用户名为空");
        Assert.hasText(user.getPassword(), "密码为空");

        String token = Jwts.builder()
                .setSubject("OXO")
                .claim("username", user.getUsername())
                .claim("password", user.getPassword())
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRATION))
                .signWith(SignatureAlgorithm.HS256, SECRET_KEY).compact();

        return token;
    }

    /**
     * 生成Token
     *
     * @param user 用户信息
     * @return token
     */
    public static String generateUsersToken(Authentication user) {
        Assert.hasText(user.getPrincipal().toString(), "用户名为空");
        Assert.hasText(user.getPrincipal().toString(), "密码为空");

        String token = Jwts.builder()
                .setSubject("OXO")
                .claim("username", user.getPrincipal().toString())
                .claim("password", user.getPrincipal().toString())
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRATION))
                .signWith(SignatureAlgorithm.HS256, SECRET_KEY).compact();

        return token;
    }

    /**
     * 生成Token
     *
     * @param user 用户信息
     * @return token
     */
    public static String generateUsersToken(UserDetails user) {
        Assert.hasText(user.getUsername(), "用户名为空");
        Assert.hasText(user.getPassword(), "密码为空");

        String token = Jwts.builder()
                .setSubject("OXO")
                .claim("username", user.getUsername())
                .claim("password", user.getPassword())
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRATION))
                .signWith(SignatureAlgorithm.HS256, SECRET_KEY).compact();

        return token;
    }

    /**
     * 解析token
     *
     * @param token
     * @return SysUser
     */
    public static SysUser decodeToken(String token) {
        Assert.hasText(token, "token为空");

        Claims claims = Jwts.parser().setSigningKey(SECRET_KEY).parseClaimsJws(token).getBody();
        SysUser sysUser = SysUser.builder()
                .username(claims.get("username").toString())
                .password(claims.get("password").toString())
                .build();
        return sysUser;
    }

    /**
     * 判断token是否已过期
     *
     * @param token
     * @return true已过期
     */
    public static boolean isExpirationToken(String token) {
        Assert.hasText(token, "token为空");
        Claims claims = Jwts.parser().setSigningKey(SECRET_KEY).parseClaimsJws(token).getBody();
        Long expirationDate = claims.getExpiration().getTime();
        if (expirationDate > System.currentTimeMillis()) {
            return false;
        }
        return true;
    }
}

