package cn.wise.sc.pay.business.comment.controller;


import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.business.comment.service.ICommentService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */

public class CommonCommentController {

    @Autowired
    ICommentService commentService;

    @PostMapping("/addComment")
    @ApiOperation("评论")
    @ApiImplicitParam(name = "comment", value = "评论实体", paramType = "body", dataType = "Comment")
    public BaseResponse addComment(@RequestBody Comment comment) {
        return commentService.insertComment(comment);
    }

    @GetMapping("/commentPage")
    @ApiOperation("根据评论主体 评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "targetId", value = "评论主体id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse getCommentsByTargetId(@RequestParam("targetId") Long targetId,
                                              PageQuery pageQuery) {
        return commentService.selectByTargetId(targetId, pageQuery);
    }

    @DeleteMapping("/deleteComment")
    @ApiOperation("根据ids删除评论 多个用英文逗号隔开")
    @ApiImplicitParam(name = "idStr", value = "id 多个用逗号分割", paramType = "path", dataType = "String")
    public BaseResponse deleteComment(@RequestParam("idStr") String idStr) {
        if (StringUtils.isNotBlank(idStr)) {
            String[] idStrArr = idStr.split(",");
            List<String> ids = new ArrayList<>(Arrays.asList(idStrArr));
            return commentService.deleteByIds(ids);
        }
        return BaseResponse.error("删除失败", Comment.class);
    }

    @PutMapping("/updatePraiseNum")
    @ApiOperation("评论点赞")
    @ApiImplicitParam(name = "id", value = "评论id", paramType = "path", dataType = "Long")
    public BaseResponse updateCommentPraise(@RequestParam("id") Long id) {
        return commentService.updatePraiseNum(id);
    }

    @GetMapping("/selectComment")
    @ApiOperation("评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse getCommentList(PageQuery pageQuery) {
        return commentService.getCommentList(pageQuery);
    }

    /**
     * 根据条件获取  评论条数
     *
     * @param pageWrapperQuery 条件
     * @return 条数
     */
    public Integer getCommentCount(QueryWrapper pageWrapperQuery) {

        return commentService.getCommentCount(pageWrapperQuery);
    }

}

