package cn.wise.sc.pay.business.goods.service.impl;

import cn.wise.sc.pay.business.goods.model.SpcDetailResponse;
import cn.wise.sc.pay.business.goods.model.bean.SpcDetail;
import cn.wise.sc.pay.business.goods.mapper.SpcDetailMapper;
import cn.wise.sc.pay.business.goods.model.vo.SpcDetailVO;
import cn.wise.sc.pay.business.goods.service.ISpcDetailService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 商品规格详情 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-12-03
 */
@Service
public class SpcDetailServiceImpl extends ServiceImpl<SpcDetailMapper, SpcDetail> implements ISpcDetailService {

    /**
     * 判断商品id和卖家id是否有效
     *
     * @param id
     * @param sellerId
     * @return
     */
    private boolean checkIdAndSeller(long id, long sellerId) {
        return id != 0 && sellerId != 0;
    }

    @Override
    public BaseResponse<Boolean> subStore(long spcDetailId, Integer num) {

        if (spcDetailId == 0 || num <= 0) {
            return BaseResponse.ok(false);
        }

        SpcDetail spcDetail = this.getById(spcDetailId);

        //库存不足
        if (spcDetail.getStore() <= num) {
            return BaseResponse.ok(false);
        }
        //如果库存为-1表示无库存限制
        if (spcDetail.getStore() == -1) {
            return BaseResponse.ok(true);
        }
        UpdateWrapper uw = new UpdateWrapper();
        uw.eq("version", spcDetail.getVersion());
        uw.eq("id", spcDetail.getId());

        uw.setSql("store = store - " + num);
        try{

            boolean isSuccess = this.update(uw);
            return BaseResponse.ok(true);
        }catch (Exception ex){
            return BaseResponse.ok(false);
        }
    }

    @Override
    public SpcDetailResponse delSpcDetail(long detailId, int sellerId) {
        if (!checkIdAndSeller(detailId, sellerId)) {
            return (SpcDetailResponse) BaseResponse.error("参数错误",new SpcDetailVO());
        }
        boolean isRemove = this.removeById(detailId);

        return isRemove ?SpcDetailResponse.ok("删除成功") : SpcDetailResponse.error("删除失败");

    }
}
