

package cn.wise.sc.pay.laas.userbac.business.controller;

import cn.wise.sc.pay.laas.userbac.business.service.MenuService;
import cn.wise.sc.pay.laas.userbac.business.service.RoleMenuService;
import cn.wise.sc.pay.laas.userbac.business.service.RoleService;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysRole;
import cn.wise.sc.pay.laas.userbac.domain.vo.MenuVO;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;


@AllArgsConstructor
public class RoleController {

	private final RoleService roleService;
	private final RoleMenuService roleMenuService;
	private final MenuService menuService;

	/**
	 * 通过ID查询角色信息
	 *
	 * @param id ID
	 * @return 角色信息
	 */
	@GetMapping("/{id}")
	public R getById(@PathVariable Integer id) {
		return R.ok(roleService.getById(id));
	}


	@GetMapping("/userRole/{userId}")
	public R getUserRoleById(@PathVariable Integer userId) {
		return R.ok(roleService.listRolesByUserId(userId));
	}

	/**
	 * 添加角色
	 *
	 * @param sysRole 角色信息
	 * @return success、false
	 */
	//@SysLog("添加角色")
	@PostMapping
	public R save(@Valid @RequestBody SysRole sysRole) {
		return R.ok(roleService.save(sysRole));
	}

	/**
	 * 修改角色
	 *
	 * @param sysRole 角色信息
	 * @return success/false
	 */
	//@SysLog("修改角色")
	@PutMapping
	public R update(@Valid @RequestBody SysRole sysRole) {
		return R.ok(roleService.updateById(sysRole));
	}

	/**
	 * 删除角色
	 *
	 * @param id
	 * @return
	 */
	//@SysLog("删除角色")
	@DeleteMapping("/{id}")
	//@PreAuthorize("@pms.hasPermission('sys_role_del')")
	public R removeById(@PathVariable Integer id) {
		return R.ok(roleService.removeRoleById(id));
	}

	/**
	 * 获取角色列表
	 *
	 * @return 角色列表
	 */
	@GetMapping("/list")
	public R listRoles() {
		return R.ok(roleService.list(Wrappers.emptyWrapper()));
	}

	/**
	 * 分页查询角色信息
	 *
	 * @param page 分页对象
	 * @return 分页对象
	 */
	@GetMapping("/page")
	public R getRolePage(Page page) {
		return R.ok(roleService.page(page, Wrappers.emptyWrapper()));
	}

	/**
	 * 更新角色菜单
	 *
	 * @param roleId  角色ID
	 * @param menuIds 菜单ID拼成的字符串，每个id之间根据逗号分隔
	 * @return success、false
	 */
	//@SysLog("更新角色菜单")
	@PutMapping("/menu")
	//@PreAuthorize("@pms.hasPermission('sys_role_perm')")
	public R saveRoleMenus(Integer roleId, @RequestParam(value = "menuIds", required = false) String menuIds) {
		SysRole sysRole = roleService.getById(roleId);
		return R.ok(roleMenuService.saveRoleMenus(sysRole.getRoleCode(), roleId, menuIds));
	}

	@GetMapping("/getMenus")
	public R getRoleMenus(Integer roleId){
		Set<MenuVO> menus = new HashSet<>();
		if(roleId != null){
			List<MenuVO> menuList = menuService.getMenuByRoleId(roleId)
					.stream()
					.filter(menuVO -> ("0".equals(menuVO.getDelFlag())))
					.collect(Collectors.toList());
			menus.addAll(menuList);
			return R.ok(menus);
		}
		return R.failed("参数错误");
	}
}
