package cn.wise.sc.pay.laas.userbac.business.controller;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserDTO;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

// @RestController
// @RequestMapping("/user")
@AllArgsConstructor
@NoArgsConstructor
public class UserController {

    private UserService userService;

    //    public void authenticate(String username, String password) throws Exception {
    //        try {
    //            authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(username,
    // password));
    //        } catch (DisabledException e) {
    //            throw new Exception("USER_DISABLED", e);
    //        } catch (BadCredentialsException e) {
    //            throw new Exception("INVALID_CREDENTIALS", e);
    //        }
    //    }

    /**
     * 获取当前用户全部信息
     *
     * @return 用户信息
     */
    @GetMapping(value = {"/info"})
    public R info() {
        //        String username = SecurityUtils.getUser().getUsername();
        String username = "";
        SysUser user =
                userService.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));
        if (user == null) {
            return R.failed("获取当前用户信息失败");
        }
        return R.ok(userService.getUserInfo(user));
    }

    /**
     * 获取指定用户全部信息
     *
     * @return 用户信息
     */
    // @Inner
    @GetMapping("/info/{username}")
    public R info(@PathVariable String username) {
        SysUser user =
                userService.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));
        if (user == null) {
            return R.failed(String.format("用户信息为空 %s", username));
        }
        return R.ok(userService.getUserInfo(user));
    }

    /**
     * 通过ID查询用户信息-
     *
     * @param id ID
     * @return 用户信息
     */
    @GetMapping("/{id}")
    // @PreAuthorize("@pms.hasPermission('sys_user_get')")
    public R user(@PathVariable Integer id) {
        return R.ok(userService.getUserVoById(id));
    }

    /**
     * 根据用户名查询用户信息
     *
     * @param username 用户名
     * @return
     */
    @GetMapping("/details/{username}")
    public R user(@PathVariable String username) {
        SysUser condition = new SysUser();
        condition.setUsername(username);
        return R.ok(userService.getOne(new QueryWrapper<>(condition)));
    }

    /**
     * 删除用户信息
     *
     * @param id ID
     * @return R
     */
    // @SysLog("删除用户信息")
    @DeleteMapping("/{id}")
    // @PreAuthorize("@pms.hasPermission('sys_user_del')")
    public R userDel(@PathVariable Integer id) {
        SysUser sysUser = userService.getById(id);
        if ("ADMIN".equals(sysUser.getLevel())) {
            return R.failed("不能删除商户初始管理员");
        }
        return R.ok(userService.removeUserById(sysUser));
    }

    /**
     * 删除用户信息
     *
     * @param id ID
     * @return R
     */
    // @SysLog("删除用户信息")
    @DeleteMapping("app/{id}")
    // @PreAuthorize("@pms.hasPermission('sys_user_del')")
    public R appUserDel(@PathVariable Integer id) {
        SysUser sysUser = userService.getAppUserById(id);
        if ("ADMIN".equals(sysUser.getLevel())) {
            return R.failed("不能删除商户初始管理员");
        }
        return R.ok(userService.removeAppUserById(sysUser));
    }

    /**
     * 添加用户
     *
     * @param userDto 用户信息
     * @return success/false
     */
    // @SysLog("添加用户")
    @PostMapping
    // @PreAuthorize("@pms.hasPermission('sys_user_add')")
    public R user(@RequestBody UserDTO userDto) {
        return R.ok(userService.saveUser(userDto));
    }

    /**
     * 更新用户信息
     *
     * @param userDto 用户信息
     * @return R
     */
    // @SysLog("更新用户信息")
    @PutMapping
    // @PreAuthorize("@pms.hasPermission('sys_user_edit')")
    public R updateUser(@Valid @RequestBody UserDTO userDto) {
        return R.ok(userService.updateUser(userDto));
    }

    /**
     * 分页查询用户
     *
     * @param page    参数集
     * @param userDTO 查询参数列表
     * @return 用户集合
     */
    @GetMapping("/page")
    public R getUserPage(Page page, UserDTO userDTO) {
        return R.ok(userService.getUserWithRolePage(page, userDTO));
    }

    /**
     * 修改个人信息
     *
     * @param userDto userDto
     * @return success/false
     */
    // @SysLog("修改个人信息")
    @PutMapping("/edit")
    public R updateUserInfo(@Valid @RequestBody UserDTO userDto) {
        return R.ok(userService.updateUser(userDto));
    }

    /**
     * 修改个人信息
     *
     * @param userDto userDto
     * @return success/false
     */
    // @SysLog("修改个人信息")
    @PutMapping("/app/edit")
    public R updateAppUserInfo(@Valid @RequestBody UserDTO userDto) {
        return R.ok(userService.updateAppUser(userDto));
    }

    /**
     * @param username 用户名称
     * @return 上级部门用户列表
     */
    @GetMapping("/ancestor/{username}")
    public R listAncestorUsers(@PathVariable String username) {
        return R.ok(userService.listAncestorUsersByUsername(username));
    }

    /**
     * 管理端会员列表 分页查询
     *
     * @param username
     * @param phone
     * @param level
     * @param pageQuery
     * @return
     */
    @GetMapping("/list")
    public R getUserList(String username, String phone, String level, PageQuery pageQuery) {
        return userService.getUserList(username, phone, level, pageQuery);
    }

    /**
     * 获取所有用户集合@return
     */
    public List<SysUser> getUserList() {
        return userService.getUserList();
    }

    /**
     * 管理端获取app会员列表 分页查询
     *
     * @param username
     * @param phone
     * @param level
     * @param pageQuery
     * @return
     */
    @GetMapping("/app/list")
    public R getAppUserList(String username, String phone, String level, PageQuery pageQuery) {
        return userService.getAppUserList(username, phone, level, pageQuery);
    }

    @PutMapping("/lock")
    public R userLock(Integer userId) {
        return R.ok(userService.userLock(userId));
    }

    @PutMapping("app/lock")
    public R userAppLock(Integer userId) {
        return R.ok(userService.appUserLock(userId));
    }

    /**
     * Test 接口
     *
     * @param userName
     * @return
     */
    @GetMapping("/getByName")
    public R getUserByName(String userName) {
        return R.ok(userService.getUserByName(userName));
    }

    /**
     *
     * @param sysUser
     * @return
     */
    protected BaseResponse<Integer> saveVirtualAppUser(SysUser sysUser) {
      return userService.saveVirtualAppUser(sysUser);
    }
}
