package cn.wise.sc.pay.laas.userbac.business.mapper;

import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import cn.wise.sc.pay.laas.userbac.domain.vo.UserVO;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/** 用户表 Mapper 接口 */
@Mapper
public interface SysUserMapper extends BaseMapper<SysUser> {
  /**
   * 通过用户名查询用户信息（含有角色信息）
   *
   * @param username 用户名
   * @return userVo
   */
  UserVO getUserVoByUsername(
      @Param("username") String username,
      @Param("prefix") String prefix,
      @Param("suffix") String suffix);

  /**
   * 分页查询用户信息（含角色）
   *
   * @param page 分页
   * @param
   * @return list
   */
  IPage<List<UserVO>> getUserVosPageTest(
      Page page,
      @Param("username") String username,
      @Param("phone") String phone,
      @Param("prefix") String prefix,
      @Param("suffix") String suffix);

  /**
   * 通过ID查询用户信息
   *
   * @param id 用户ID
   * @return userVo
   */
  UserVO getUserVoById(
      @Param("id") Integer id, @Param("prefix") String prefix, @Param("suffix") String suffix);

  /**
   * 通过ID查询用户信息
   *
   * @param id 用户ID
   * @return userVo
   */
  SysUser getAppUserById(
      @Param("id") Integer id, @Param("prefix") String prefix, @Param("suffix") String suffix);

  /** 通过ID查询用户信息 */
  void updateAppUser(SysUser userDTO);

  /**
   * 通过用户id获取appd端用户 manage端专用
   *
   * @param userId
   * @return
   */
  SysUser getappUserById(@Param("userId") Integer userId);

  void removeAppById(@Param("userId") Integer userId);

  /**
   * 获取所有用户集合
   *
   * @return list
   */
  List<SysUser> list();

  /**
   * 添加app虚拟用户
   * @param sysUser
   * @return
   */
  Integer saveVirtualAppUser(SysUser sysUser);
}
