package cn.wise.sc.pay.pass.business.app.controller;

import cn.hutool.captcha.CaptchaUtil;
import cn.hutool.captcha.LineCaptcha;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.map.MapUtil;
import cn.wise.sc.pay.api.points.service.IMemberShipService;
import cn.wise.sc.pay.api.points.service.IUserPointsMShipExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.util.DecryptUtil;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.query.UserExreaQuery;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import cn.wise.sc.pay.laas.userbac.business.controller.UserController;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserDTO;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import cn.wise.sc.pay.laas.userbac.domain.vo.UserVO;
import cn.wise.sc.pay.pass.business.app.config.CommonUrlProperties;
import cn.wise.sc.pay.pass.business.app.config.FastDfsUtil;
import cn.wise.sc.pay.pass.business.app.model.WXInfo;
import cn.wise.sc.pay.pass.business.app.model.WxAuthPhone;
import cn.wise.sc.pay.pass.business.app.model.query.OpenIdRegisterQuery;
import cn.wise.sc.pay.pass.business.app.model.vo.SysUserInfoVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.AlgorithmParameters;
import java.security.Security;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author neo.shu
 * @since 2019/12/9 18:27
 */
@CrossOrigin
@RestController
@RequestMapping("/app/user")
@Api(value = "app/user", tags = "app端用户管理")
@EnableConfigurationProperties(CommonUrlProperties.class)
public class UserControler extends UserController {

    @Autowired
    IUserPointsMShipExtraService iUserPointsMShipExtraService;
    @Autowired
    IMemberShipService memberShipService;
    @Autowired
    UserService userService;
    @Autowired
    FastDfsUtil fastDFSUtils;
    @Autowired
    IUserPointsMShipExtraService userPointsMShipExtraService;
    @Autowired
    CommonUrlProperties commonUrlProperties;

    private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();
    public UserControler(UserService userService) {
        super(userService);
    }

    @ApiOperation("app端账号注册")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "password", value = "用户密码", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "code", value = "验证码", dataType = "String", paramType = "query")
    })
    @PostMapping("/register")
    public BaseResponse registerSysUser(String userName, String phone, String code) {
        UserDTO sysUser = new UserDTO();
        sysUser.setUsername(userName);
        sysUser.setPhone(phone);
        sysUser.setPassword("123456");
        List<Integer> roleList = new ArrayList<>();
        roleList.add(1);
        sysUser.setRole(roleList);

        Object existUser = super.user(userName).getData();
        if (existUser != null) {
            return BaseResponse.error("用户已存在", SysUser.class);
        }
        super.user(sysUser);

        SysUser user = (SysUser) super.user(userName).getData();
        if (user == null) {
            return BaseResponse.error("注册失败", SysUser.class.getName());
        }
        UserExreaQuery userExreaQuery = new UserExreaQuery();
        userExreaQuery.setUserId(Integer.toUnsignedLong(user.getUserId()));
        userExreaQuery.setMembershipId(1L);
        userExreaQuery.setPoints(0d);
        userExreaQuery.setExperience(0d);

        BaseResponse baseResponse = iUserPointsMShipExtraService.addNewUserExtra(userExreaQuery);
        if (baseResponse.getCode() == 400) {
            super.userDel(user.getUserId());

            return BaseResponse.error("注册失败", SysUser.class.getName());
        }

        return baseResponse;
    }

    @ApiOperation(value = "会员基本信息修改")
    @ApiImplicitParams(
            @ApiImplicitParam(name = "userDTO", value = "参数对象", dataType = "UserDTO", paramType = "body")
    )
    @PutMapping("/edit/userInfo")
    @Transactional
    public BaseResponse editUserInfo(@Valid @RequestBody UserDTO userDTO) {
        BaseResponse editUserExtra = null;

        R r = super.updateUserInfo(userDTO);

        if (userDTO.getMemberShipId() != null) {
            UserExreaQuery userExreaQuery = UserExreaQuery.builder().userId(Integer.toUnsignedLong(userDTO.getUserId()))
                    .membershipId(Integer.toUnsignedLong(userDTO.getMemberShipId()))
                    .build();
            editUserExtra = iUserPointsMShipExtraService.editUserExtra(userExreaQuery);
            if (r.ok() && editUserExtra != null && editUserExtra.getCode() == 200) {
                return BaseResponse.ok(r.getData());
            } else {
                return BaseResponse.error("更新失败", this.getClass().getName());
            }
        }
        if (r.ok()) {
            return BaseResponse.ok(r.getData());
        }
        return BaseResponse.error("更新失败", this.getClass().getName());
    }

    @Deprecated
    @PostMapping("wxRegister")
    @ApiIgnore          //不用  用下面个
    @ApiOperation(value = "微信授权注册")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "wxOpenid", value = "微信Openid", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "nickname", value = "昵称", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "phone", value = "手机号码", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "multiAvatar", value = "头像地址字符串", dataType = "String", paramType = "query")
            }
    )
    public BaseResponse WXRegister(String wxOpenid, String nickname, String multiAvatar, String phone) {


        if (StringUtils.isEmpty(wxOpenid)) {
            return BaseResponse.error("wxOpenid 不能为空", this.getClass().getName());
        }
        SysUser sysUser = new SysUser();
        if (StringUtils.isNotEmpty(multiAvatar)) {
            sysUser.setAvatar(multiAvatar);
        }
        sysUser.setWxOpenid(wxOpenid);
        if (StringUtils.isNotEmpty(nickname)) {
            sysUser.setUsername(nickname);
        }
        if (StringUtils.isNotEmpty(phone)) {
            sysUser.setPhone(phone);
        }
        QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
        sysUserQueryWrapper.eq("wx_openid", wxOpenid);
        SysUser one = null;
        one = userService.getOne(sysUserQueryWrapper);
        if (one != null) {
            return BaseResponse.error("用户已存在", this.getClass().getName());
        }

        //添加用户
        boolean save = userService.save(sysUser);
        if (save == false) {
            return BaseResponse.error("注册失败", this.getClass().getName());
        }
        one = userService.getOne(sysUserQueryWrapper);
        //给用户关联一个新的   积分、经验、会员等级信息
        UserExreaQuery userExreaQuery = new UserExreaQuery();
        userExreaQuery.setUserId(Integer.toUnsignedLong(one.getUserId()));
        //获取最低级会话等级
        BaseResponse<Membership> primaryMemberShip = memberShipService.getPrimaryMemberShip();
        Membership membership = primaryMemberShip.getData();
        if (primaryMemberShip.getCode() == 400 || membership == null) {
            return BaseResponse.error("绑定会员失败，注册失败", SysUser.class.getName());
        }
        userExreaQuery.setMembershipId(membership.getId());
        userExreaQuery.setPoints(0d);
        userExreaQuery.setExperience(0d);
        BaseResponse baseResponse = iUserPointsMShipExtraService.addNewUserExtra(userExreaQuery);
        if (baseResponse.getCode() == 400) {
            super.userDel(one.getUserId());

            return BaseResponse.error("注册失败", SysUser.class.getName());
        }
        return BaseResponse.ok(one);
    }

//=======================================================================================

    @PostMapping("/wxOrAli/Register")
    @ApiOperation(value = "微信和阿里授权方式注册")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "OpenIdRegisterQuery", value = "微信或者阿里注册参数对象", dataType = "OpenIdRegisterQuery", paramType = "body")
            }
    )
    public BaseResponse WxOrAliRegister(@Valid @RequestBody OpenIdRegisterQuery openIdRegisterQuery) {
        long start = System.currentTimeMillis();
        System.out.println(start);
        //条件构造
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        //判断是否存在用户
        SysUser sysUser = this.verifyOpenIdUser(queryWrapper, openIdRegisterQuery.getOpenId(), openIdRegisterQuery.getType());
        if (sysUser != null) {
            return BaseResponse.error("你已经注册", this.getClass().getName());
        }
        //添加用户注册信息
        SysUser sysUserBuild = SysUser.builder()
                .username(openIdRegisterQuery.getUsername())
                .avatar(openIdRegisterQuery.getMultiAvatar())
                .filed1(openIdRegisterQuery.getSex())
                .filed2(openIdRegisterQuery.getCity())
                .build();
        this.openIdByType(sysUserBuild, openIdRegisterQuery.getType(), openIdRegisterQuery.getOpenId());

        boolean save = userService.save(sysUserBuild);
        if (!save) {
            return BaseResponse.error("注册失败", SysUser.class.getName());
        }

        //获取最低级会话等级
        BaseResponse<Membership> primaryMemberShip = memberShipService.getPrimaryMemberShip();
        Membership membership = primaryMemberShip.getData();
        if (primaryMemberShip.getCode() == 400 || membership == null) {
            super.userDel(sysUserBuild.getUserId());
            return BaseResponse.error("绑定会员失败，注册失败", SysUser.class.getName());
        }
        //新增成功时 给用户关联一个新的   积分、经验、会员等级信息
        UserExreaQuery userExreaQuery = new UserExreaQuery();
        userExreaQuery.setUserId(Integer.toUnsignedLong(sysUserBuild.getUserId()));
        userExreaQuery.setMembershipId(membership.getId());
        userExreaQuery.setPoints(0d);
        userExreaQuery.setExperience(0d);
        BaseResponse baseResponse = iUserPointsMShipExtraService.addNewUserExtra(userExreaQuery);
        //添加积分信息失败时，删除注册信息
        if (baseResponse.getCode() == 400) {
            super.userDel(sysUserBuild.getUserId());
            return BaseResponse.error("注册失败", this.getClass().getName());
        }
        long end = System.currentTimeMillis();
        System.out.println(end);
        System.out.println(end - start);
        return BaseResponse.ok(sysUserBuild, this.getClass());
    }


    @PutMapping("/binding/phone")
    @ApiOperation(value = "根据openId绑定用户手机号")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "openId", value = "openId", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "phoneNum", value = "手机号", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "type", value = "openId所属类型", dataType = "Integer", paramType = "query")
            }
    )
    public BaseResponse bindPhoneByOpenId(@NotBlank(message = "openId不能为空") String openId,
                                          @NotBlank(message = "电话不能为空") String phoneNum,
                                          @NotNull(message = "openId所属类型不能为空") Integer type) {
        //条件构造
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        SysUser sysUser = this.verifyOpenIdUser(queryWrapper, openId, type);
        if (sysUser == null) {
            return BaseResponse.error("绑定手机号的用户不存在", this.getClass().getName());
        }
        sysUser.setPhone(phoneNum);
        boolean update = userService.updateById(sysUser);
        return update ? BaseResponse.ok(sysUser, this.getClass()) : BaseResponse.error("绑定手机号失败", this.getClass().getName());
    }


    /**
     * 根据openid查询用户记录
     *
     * @param queryWrapper
     * @param openId
     * @param type
     * @return
     */
    public SysUser verifyOpenIdUser(QueryWrapper<SysUser> queryWrapper, String openId, Integer type) {
        //WxAli
        if (type == 1) {
            queryWrapper.eq("ali_openid", openId);
        } else {
            queryWrapper.eq("wx_openid", openId);
        }
        SysUser sysUser = userService.getOne(queryWrapper);

        return sysUser;
    }

    /**
     * 根据OpenID所属类型分别设置openID值
     *
     * @param sysUser
     * @param type
     * @param openId
     * @return
     */
    public void openIdByType(SysUser sysUser, Integer type, String openId) {
        if (type == 0) {
            sysUser.setWxOpenid(openId);
        }
        //阿里openId
        if (type == 1) {
            sysUser.setAliOpenid(openId);
        }
    }


    @GetMapping("/userInfo/byOpenId")
    @ApiOperation("根据openID查询用户")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "openId", value = "open_id", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "flag", value = "openId类别标记，0是WXOpenid,1是AliOpenId", dataType = "Integer", paramType = "query")
            }
    )
    public BaseResponse getUserByOpenId(String openId, Integer flag) {
        if (StringUtils.isEmpty(openId)) {
            return BaseResponse.error("openId不能为空", this.getClass().getName());
        }
        if (flag == null || flag > 1 || flag < 0) {
            return BaseResponse.error("flag参数错误", this.getClass().getName());
        }
        QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
        if (flag == 0) {
            //WXOpenId
            sysUserQueryWrapper.eq("wx_openid", openId);
        }
        if (flag == 1) {
            //AliOpenId
            sysUserQueryWrapper.eq("ali_openid", openId);
        }
        SysUser one = userService.getOne(sysUserQueryWrapper);

        if (one == null) {
            return BaseResponse.error("用户不存在", this.getClass().getName());
        }

        //获取动态路径的地址
        Map<String, String> urlInfo = MapUtil.newConcurrentHashMap(4);
        assert commonUrlProperties != null;
        urlInfo.put("baseUrl", commonUrlProperties.getBaseUrl());
        urlInfo.put("webViewUrl", commonUrlProperties.getWebViewUrl());
        urlInfo.put("imgUrl", commonUrlProperties.getImageUrl());
        urlInfo.put("rabbitUrl", commonUrlProperties.getRabbitUrl());
        //返回结果
        Map<String, Object> rts = new HashMap<>(2);
        rts.put("userInfo", one);
        rts.put("urlInfo", urlInfo);

        return BaseResponse.ok(this.getClass().getName(), rts);
    }

    @ApiOperation(value = "查询用户用户详情信息", notes = "包括积分等信息")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "userId", value = "用户id", dataType = "long", paramType = "query")
            }
    )
    @GetMapping("/userDetailsInfo")
    public BaseResponse userDetailsInfo(Long userId) {
        if (userId == null) {
            return BaseResponse.error("请传入参数", this.getClass().getName());
        }

        //用户信息
        SysUser sysUser = userService.getById(userId);
        if (sysUser == null) {
            return BaseResponse.error("用户不存在", this.getClass().getName());
        }
        //当前用户积分相关信息
        int flag = 0;
        BaseResponse<UserExtraVOApp> userExtra = iUserPointsMShipExtraService.getUserExtra(userId, flag);
        UserExtraVOApp data = userExtra.getData();
        if (userExtra.getCode() != 200 || data == null) {
            return BaseResponse.error("获取信息失败", this.getClass().getName());
        }
        SysUserInfoVO build = SysUserInfoVO.builder()
                .userId(sysUser.getUserId())
                .memberShipName(data.getMemberShipName()) //会员名字
                .wxOpenid(sysUser.getWxOpenid())
                .username(sysUser.getUsername())
                .phone(sysUser.getPhone())
                .experience(data.getExperience())  // 经验
                .points(data.getPoints())  // 积分
                .memberInterests(data.getMemberInterests())  //权益
                .nextPoints(data.getNextPoints())
                .filed2(data.getBackPhoto())   //会员背景图
                .membershipId(data.getMembershipId()) //会员等级id
                .upExperience(data.getUpExperience())  //上限积分
                .downExperience(data.getDownExperience()) //下限积分
                .discount(data.getDiscount())
                .nextDiscount(data.getNextDiscount())
                .build();
        return BaseResponse.ok(build);
    }

    @PostMapping("/code")
    @ResponseBody
    public void getCode(HttpServletResponse response) {
        //定义图形验证码的长和宽
        LineCaptcha lineCaptcha = CaptchaUtil.createLineCaptcha(200, 100);

        try {
            lineCaptcha.write(response.getOutputStream());
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                response.getOutputStream().close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @PostMapping("/login")
    public R loginSysUser(@RequestParam("username") String username,
                          @RequestParam("password") String password) {
        Object res = user(username).getData();
        if (res != null) {
            String pw = ((SysUser) res).getPassword();
            if (ENCODER.matches(password, pw)) {
                return R.ok(res);
            }
        }
        return R.failed("登录失败");
    }

    @ApiOperation("app端会员用户条件分页列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名搜索", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "phone", value = "用户手机号搜索", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "level", value = "会员等级搜索（当前未实现）", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageNo", value = "当前页", dataType = "Integer", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", dataType = "Integer", paramType = "query")
    })
    @GetMapping("/userPage")
    public R getUserPage(String username, String phone, String level, PageQuery pageQuery) {
        Page page = (Page) getUserList(username, phone, level, pageQuery).getData();
        List<UserVO> users = page.getRecords();
        List<Long> userId = users.stream()
                .map(arg -> Integer.toUnsignedLong(arg.getUserId()))
                .collect(Collectors.toList());
        List<LinkedHashMap> result = userPointsMShipExtraService.getAllUserExtra(userId).getList();
        List<UserExtra> userExtras = JSON.parseArray(JSON.toJSONString(result), UserExtra.class);
        List<SysUserInfoVO> res = new ArrayList<>();
        users.stream().forEach(arg -> {
            UserExtra user = userExtras.stream().filter(parm ->
                    Integer.toUnsignedLong(arg.getUserId()) == parm.getUserId()
            ).findFirst().orElse(null);
            SysUserInfoVO sysUserInfoVO = new SysUserInfoVO();
            BeanUtil.copyProperties(user, sysUserInfoVO);
            BeanUtil.copyProperties(arg, sysUserInfoVO);
            res.add(sysUserInfoVO);
        });
        Page pageResult = new Page(page.getCurrent(), page.getSize(), page.getTotal());
        pageResult.setRecords(res);
        return R.ok(pageResult);
    }

    @GetMapping("/getMemberShipUser")
    @ApiOperation(value = "查询会员列表", notes = "根据关键字分页查询")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "keyWord", value = "关键字,用户名", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "phone", value = "手机号", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "memberShipId", value = "会员等级id", dataType = "int", paramType = "query"),
                    @ApiImplicitParam(name = "pageQuery", value = "分页参数", dataType = "PageQuery", paramType = "query")
            }
    )
    public BaseResponse getMemberShipUser(String keyWord, String phone, Integer memberShipId, PageQuery pageQuery) {
        //分页对象
        Page<SysUser> userPage = new Page<>();
        userPage.setCurrent(pageQuery.getPageNo());
        userPage.setSize(pageQuery.getPageSize());
        //条件
        QueryWrapper<SysUser> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("del_flag", 0);
        userQueryWrapper.orderByDesc("create_time");
        if (StringUtils.isNotBlank(keyWord)) {
            userQueryWrapper.like("username", keyWord);
        }
        if (StringUtils.isNotBlank(phone)) {
            userQueryWrapper.like("phone", phone);
        }
        if (memberShipId != null) {
            BaseResponse<List<UserExtra>> byMemberId = iUserPointsMShipExtraService.getByMemberId(Integer.toUnsignedLong(memberShipId));
            List<UserExtra> userData = byMemberId.getData();
            if (byMemberId.getCode() != 200 || userData == null || userData.size() < 1) {
                return BaseResponse.ok(userPage.setRecords(null));
            }
            List<Long> userIds = userData.stream().map(s -> s.getUserId()).collect(Collectors.toList());
            userQueryWrapper.in("user_id", userIds);
        }
        //分页记录
        IPage<SysUser> page = userService.page(userPage, userQueryWrapper);
        //用户信息
        List<SysUser> users = page.getRecords();
        if (users == null || users.size() < 1) {
            return BaseResponse.ok(page);
        }
        List<Long> userIds = users.stream().map(u -> Integer.toUnsignedLong(u.getUserId())).collect(Collectors.toList());

        BaseResponse<List<UserExtra>> listByUserId = iUserPointsMShipExtraService.getListByUserId(userIds);
        //UserExtra信息
        List<UserExtra> userExtraData = listByUserId.getData();

        List<Long> memberShipIds = userExtraData.stream().map(ue -> ue.getMembershipId()).collect(Collectors.toList());
        BaseResponse<List<Membership>> memberShipResponse = memberShipService.listByIds(memberShipIds);
        //Membership信息
        List<Membership> memberships = memberShipResponse.getData();
        //返回信息
        List<SysUserInfoVO> userInfoVOS = new ArrayList<>();
        users.stream().forEach(u -> {
            SysUserInfoVO userInfoVO = new SysUserInfoVO();
            userInfoVO.setUserId(u.getUserId());
            userInfoVO.setUsername(u.getUsername());
            userInfoVO.setPhone(u.getPhone());
            userInfoVO.setLockFlag(u.getLockFlag());
            if (userExtraData != null && userExtraData.size() > 0) {
                userExtraData.stream().forEach(ue -> {

                    if (Integer.toUnsignedLong(u.getUserId()) == ue.getUserId()) {
                        userInfoVO.setExperience(ue.getExperience());
                        userInfoVO.setPoints(ue.getPoints());
                    }
                    if (memberships != null && memberships.size() > 0) {
                        memberships.stream().forEach(m -> {
                            if (ue.getMembershipId().equals(m.getId()) && m.getDelFlag() == 0) {
                                userInfoVO.setMemberShipName(m.getName());
                            }
                        });
                    }
                });

            }
            userInfoVOS.add(userInfoVO);
        });
        Page<SysUserInfoVO> userInfoVOPage = new Page<>();
        userInfoVOPage.setRecords(userInfoVOS);
        userInfoVOPage.setTotal(page.getTotal());
        userInfoVOPage.setSize(page.getSize());
        userInfoVOPage.setCurrent(page.getCurrent());
        userInfoVOPage.setPages(page.getPages());
        return BaseResponse.ok(userInfoVOPage);
    }


    @ApiOperation("app端用户导出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名搜索", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "phone", value = "用户手机号搜索", dataType = "String", paramType = "query")
    })
    @GetMapping("/exportUser")
    public void exportUserList(String username, String phone, HttpServletResponse response) {
        try {
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");

            String fileName = URLEncoder.encode("小程序用户列表" + System.currentTimeMillis(), "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            List<SysUser> list = (List<SysUser>) getUserList(username, phone, null, null).getData();
            List<Long> userId = list.stream()
                    .map(arg -> Integer.toUnsignedLong(arg.getUserId()))
                    .collect(Collectors.toList());
            List<LinkedHashMap> result = userPointsMShipExtraService.getAllUserExtra(userId).getList();
            List<SysUserInfoVO> res = new ArrayList<>();
            list.stream().forEach(arg -> {
                LinkedHashMap user = result.stream().filter(parm ->
                        arg.getUserId() == parm.get("userId")
                ).findFirst().orElse(null);
                SysUserInfoVO sysUserInfoVO = new SysUserInfoVO();
                if (user != null) {
                    sysUserInfoVO.setUserId((Integer) user.get("userId"));
                    sysUserInfoVO.setMembershipId((Long) user.get("membershipId"));
                    sysUserInfoVO.setExperience((double) user.get("experience"));
                    sysUserInfoVO.setPoints((double) user.get("points"));
                    sysUserInfoVO.setMemberShipName((String) user.get("memberShipName"));
                }
                if ("0".equals(arg.getLockFlag())) {
                    arg.setLockFlag("正常");
                } else {
                    arg.setLockFlag("锁定");
                }
                BeanUtil.copyProperties(arg, sysUserInfoVO);
                res.add(sysUserInfoVO);
            });
            EasyExcel.write(response.getOutputStream(), SysUserInfoVO.class).sheet("模板").doWrite(res);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @PostMapping("WX/getPhone")
    @ApiOperation("微信获取手机号")
    public BaseResponse registeByWXInfo(@RequestBody WxAuthPhone wxAuthPhone) throws Exception {

        String encryptedData, sessionKey, iv, openId;
        encryptedData = wxAuthPhone.getEncryData();
        sessionKey = wxAuthPhone.getSessionKey();
        iv = wxAuthPhone.getIv();
        DecryptUtil d = new DecryptUtil();
        byte[] result = d.decrypt(encryptedData, sessionKey, iv);
        String rst = "";
        try {
            rst = new String(result, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            return BaseResponse.error("微信授权未通过", SysUser.class.getName());
        }
        WXInfo wxInfo = JSON.parseObject(rst, WXInfo.class);
        return BaseResponse.ok(wxInfo);
    }

    @PutMapping("/extra")
    @ApiOperation("修改会员 等级 积分 经验值的接口")
    public BaseResponse editUserExtraInfo(UserExreaQuery userExreaQuery) {
        return iUserPointsMShipExtraService.editUserExtra(userExreaQuery);
    }

    private static String decryptData(String encryptedData, String sessionKey, String iv) throws Exception {

        byte[] dataByte = org.apache.commons.codec.binary.Base64.decodeBase64(encryptedData);
        // 加密秘钥
        byte[] keyByte = org.apache.commons.codec.binary.Base64.decodeBase64(sessionKey);
        // 偏移量
        byte[] ivByte = Base64.decodeBase64(iv);
        try {
            // 如果密钥不足16位，那么就补足.  这个if 中的内容很重要
            int base = 16;
            if (keyByte.length % base != 0) {
                int groups = keyByte.length / base + (keyByte.length % base != 0 ? 1 : 0);
                byte[] temp = new byte[groups * base];
                Arrays.fill(temp, (byte) 0);
                System.arraycopy(keyByte, 0, temp, 0, keyByte.length);
                keyByte = temp;
            }
            // 初始化
            // 初始化
            Security.addProvider(new BouncyCastleProvider());
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding");
            SecretKeySpec spec = new SecretKeySpec(keyByte, "AES");
            AlgorithmParameters parameters = AlgorithmParameters.getInstance("AES");
            parameters.init(new IvParameterSpec(ivByte));
            cipher.init(Cipher.DECRYPT_MODE, spec, parameters);// 初始化
            byte[] resultByte = cipher.doFinal(dataByte);
            if (null != resultByte && resultByte.length > 0) {
                String result = new String(resultByte, "UTF-8");
                return result;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}
