package cn.wise.sc.pay.pass.business.app.controller.cashier;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.api.cashier.service.ICashierOrdersService;
import cn.wise.sc.pay.api.cashier.service.IGoodsService;
import cn.wise.sc.pay.api.cashier.service.OrderDetailQuery;
import cn.wise.sc.pay.api.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.goods.model.vo.GoodsVO;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.pass.business.app.config.FastDfsUtil;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 在线点单  商铺
 * @author: qh
 * @create: 2020-06-08 17:15
 **/
@RestController
@Api(description = "在线单点商铺")
@RequestMapping("/app/cashier/shop")
@CrossOrigin
@Validated
public class AppShopController {

    @Autowired
    IMerchantService iMerchantService;
    @Autowired
    IGoodsService iGoodsService;
    @Autowired
    FastDfsUtil fastDfsUtil;
    @Autowired
    ICashierOrdersService iCashierOrdersService;
    @Autowired
    IOrdersService ordersService;

    @Autowired
    IMerchantProfitService merchantProfitService;

    @PostMapping("/list")
    @ApiOperation("获取正在营业和未关闭的店铺")
    public BaseResponse<Merchant> list(PageQuery pageQuery) {
        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        HashMap<String, Object> eq = new HashMap<>(2);
        eq.put("del_status", 0);
        eq.put("manage_status", 0);
        pageWrapperQuery.setEq(eq);
        pageWrapperQuery.setPageQuery(pageQuery);
        return iMerchantService.getStoreInfoPage(pageWrapperQuery);
    }

    @GetMapping("/{id}")
    @ApiOperation("获取商铺信息")
    public BaseResponse getStore(@PathVariable("id") Long id) {
        return iMerchantService.getStoreInfo(id);
    }


    @GetMapping("/goods/{shopId}")
    @ApiOperation("获取商铺下的商品列表")
    public BaseResponse<List<GoodsVO>> getGoodsByShopId(@PathVariable("shopId") Long shopId, Long tableId, String type) {
        BaseResponse<List<HashMap>> goodsBaseResponse = iGoodsService.list(shopId.intValue(), "", type, 1);
        List<HashMap> list = goodsBaseResponse.getData();
        List<GoodsVO> rts = new ArrayList<>();
        for (HashMap map : list) {
            GoodsVO goodsVO = BeanUtil.mapToBean(map, GoodsVO.class, true);
            goodsVO.setCover(fastDfsUtil.fullUrl(goodsVO.getCover()));
            rts.add(goodsVO);
        }
        return BaseResponse.ok(rts);
    }


    @PostMapping("/newOrder")
    @ApiOperation("新增一个订单")
    @ApiImplicitParam(name = "orderDetailQuery",value = "订单请求",paramType = "body",dataType = "OrderDetailQuery")
    public BaseResponse newOrder(@RequestBody OrderDetailQuery orderDetailQuery){
        orderDetailQuery.setStatus(OrderStatus.WAIT_PAY.name());

        if (orderDetailQuery.getPayPrice() == 0){
            BaseResponse response = iCashierOrdersService.newOrder(orderDetailQuery);
            if (response.getCode() == 200){
                Orders order = (Orders) response.getData();
                orderDetailQuery.setOrderNo(order.getOrderNo());
                orderDetailQuery.setPayStatus(PayStatus.PAY_YES.name());
                orderDetailQuery.setStatus(OrderStatus.COMPLETE.name());
                orderDetailQuery.setPayment("赠送");
                return iCashierOrdersService.editEntryOrder(orderDetailQuery);
            }else {
                return BaseResponse.ok("赠送超时！请重试","");
            }
        }else {
            orderDetailQuery.setPaas("APP");
            return iCashierOrdersService.newOrder(orderDetailQuery);
        }
    }

    public static void main(String[] args) {
        OrderDetailQuery orderDetailQuery = new OrderDetailQuery();
        orderDetailQuery.setOrderNo("123456789");
        orderDetailQuery.setCreateTime(1576746683827L);
        orderDetailQuery.setPayment("支付宝");
        orderDetailQuery.setPayPrice(80.00);
        orderDetailQuery.setUserId(1234L);
        orderDetailQuery.setUserPhone("13123456789");
        orderDetailQuery.setRemark("测试");
        orderDetailQuery.setPaas("app");
        System.out.println(JSON.toJSONString(orderDetailQuery));
    }


    @DeleteMapping("{orderNo}")
    @ApiOperation("取消订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse delOrder(@PathVariable("orderNo") String orderNo) {
        return iCashierOrdersService.delOrder(orderNo);
    }

    @GetMapping("listApp/user/{userId}/{status}")
    @ApiOperation("查询用户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId",value = "用户id",paramType = "path",dataType = "Integer"),
            @ApiImplicitParam(name = "status",value = "订单状态",paramType = "path",dataType = "Integer")
    })
    public BaseResponse listUsers(@PathVariable("userId") Integer userId , @PathVariable("status") Integer status){
        BaseResponse responseEntity;
        responseEntity = iCashierOrdersService.listUsers(userId,status);
        return responseEntity;
    }

    @GetMapping("pageApp/user/{userId}/{status}")
    @ApiOperation("查询用户订单/分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId",value = "用户id",paramType = "path",dataType = "String"),
            @ApiImplicitParam(name = "status",value = "订单状态",paramType = "path",dataType = "Integer")
    })
    public BaseResponse pageAppUser(@PathVariable("userId") String userId,@PathVariable("status") Integer status,PageQuery pageQuery){
        BaseResponse responseEntity;
        responseEntity = iCashierOrdersService.getOrdersPage(userId,status,pageQuery);
        return responseEntity;
    }

    @PutMapping("/afterApp/success/{orderNo}")
    @ApiOperation("支付成功后的回调")
    public BaseResponse afterSuccess(@PathVariable("orderNo")String orderNo){
        BaseResponse response = iCashierOrdersService.afterSuccess(orderNo);
        if (response.code == 200){
            Map map = new HashMap();
            map.put("order_no",orderNo);
            OrderResponse oneOrderResponse = ordersService.getOneOrder(JSON.toJSONString(map));
            Orders orders = oneOrderResponse.getData();

            //佣金推送
            if (orders != null && oneOrderResponse.getCode() == 200){
                if (OrderStatus.WAIT_COMMENT.name().equals(orders.getOrderStatus())){
                    merchantProfitService.addMoneyData(orders.getSellerId(),
                            orders.getId(),orders.getPaymentMethodName(),orders.getPayPrice(),
                            orders.getCreateTime());
                }
            }
        }
        return response;
    }

    @PutMapping("/afterApp/fail/{orderNo}")
    @ApiOperation("支付失败后的回调")
    public BaseResponse afterFail(@PathVariable("orderNo") String orderNo) {
        return iCashierOrdersService.afterFail(orderNo);
    }
}
