package cn.wise.sc.pay.pass.business.app.controller.merchant;

import cn.wise.sc.pay.api.merchant.service.IMerchantCommentService;
import cn.wise.sc.pay.api.points.service.IPointDetailService;
import cn.wise.sc.pay.api.points.service.IPointsSourceV1Service;
import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * @description: 在线点单评论
 * @author: qh
 * @create: 2020-06-15 15:21
 **/
@CrossOrigin
@RestController
@Api(tags = "在线点评")
@RequestMapping("/app/merchant/comment")
public class MerchantCommentController {

    @Autowired
    IMerchantCommentService iMerchantCommentService;
    @Autowired
    IPointsSourceV1Service pointsSourceV1Service;
    @Autowired
    IPointDetailService iPointDetailService;

    /**
     * //     * 添加评论
     * //     * @param comment
     * //     * @return
     * //
     */
    @PostMapping("/addComment")
    public BaseResponse addComment(@RequestBody Comment comment) {

        BaseResponse response = iMerchantCommentService.addComment(comment);

        //增长积分
        Double score = pointsSourceV1Service.getScore(Long.valueOf(comment.getContent().length()), "发布文章");
        PointDetailQuery pointDetailQuery = new PointDetailQuery();
        pointDetailQuery.setUserId(comment.getCriticId());
        pointDetailQuery.setInOrOut(1);
        pointDetailQuery.setPathWay("发布文章");
        pointDetailQuery.setScore(score);
        iPointDetailService.add(pointDetailQuery);

        return response;
    }

    /**
     * 删除评论，多个评论id用英文逗号 ','隔开
     *
     * @param idStr
     * @return
     */
    @DeleteMapping("/deleteComment")
    public BaseResponse deleteComment(@RequestBody String idStr) {
        return iMerchantCommentService.deleteComment(idStr);
    }

    /**
     * 更新评论点赞数
     *
     * @param id
     * @return
     */
    @PutMapping("/updatePraiseNum/{id}")
    public BaseResponse updateCommentPraise(@PathVariable("id") Long id) {
        return iMerchantCommentService.updateCommentPraise(id);
    }

    /**
     * 评论列表分页查询
     *
     * @param pageQuery
     * @return
     */
    @GetMapping("/selectComment")
    public BaseResponse getCommentList(@SpringQueryMap PageQuery pageQuery) {
        return iMerchantCommentService.getCommentList(pageQuery);
    }

    /**
     * 根据评论目标id分页查询
     *
     * @param targetId
     * @param pageQuery
     * @return
     */
    @GetMapping("/commentPage")
    public BaseResponse<Page<Comment>> getCommentsByTargetId(@RequestParam("targetId") Long targetId,
                                                             @SpringQueryMap PageQuery pageQuery) {
        return iMerchantCommentService.getCommentsByTargetId(targetId, pageQuery);
    }

}
