package cn.wise.sc.pay.pass.business.app.controller.orders;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionOrderService;
import cn.wise.sc.pay.api.order.service.IOrderCommentReplyService;
import cn.wise.sc.pay.api.order.service.IOrderCommentService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.order.service.ISafeguardService;
import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.business.comment.bean.Reply;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.mq.client.dto.TransactionMessage;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.OrdersDBColumn;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.domain.order.query.SafeguardDetailQuery;
import cn.wise.sc.pay.pass.business.app.config.FastDfsUtil;
import cn.wise.sc.pay.pass.business.app.config.Producer;
import cn.wise.sc.pay.pass.business.app.model.vo.OrdersVO;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 消费端 订单服务
 * @author: qh
 * @create: 2019-12-26 11:12
 **/
@CrossOrigin
@RestController
@Api(description = "订单服务")
@RequestMapping("/app/orders")
public class OrdersController {

    final
    IOrdersService iOrdersService;
    final
    IExhibitionOrderService iExhibitionOrderService;
    final
    ISafeguardService iSafeguardService;
    final
    FastDfsUtil fastDFSUtils;
    @Autowired
    IOrderCommentService iOrderCommentService;
    @Autowired
    IOrderCommentReplyService iOrderCommentReplyService;

    public OrdersController(IOrdersService iOrdersService, IExhibitionOrderService iExhibitionOrderService, ISafeguardService iSafeguardService, FastDfsUtil fastDFSUtils) {
        this.iOrdersService = iOrdersService;
        this.iExhibitionOrderService = iExhibitionOrderService;
        this.iSafeguardService = iSafeguardService;
        this.fastDFSUtils = fastDFSUtils;
    }

    /**
     * 获取所有 当前用户所有订单
     *
     * @param userId
     * @return
     */
    @PostMapping("/list/{userId}/{status}")
    @ApiOperation("获取用户特定状态下所有订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "string"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]", paramType = "path", dataType = "OrderStatus")
    })
    public BaseResponse order(@PathVariable("userId") String userId, @PathVariable("status") int status) {

        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        BaseResponse baseResponse = iOrdersService.getOrdersByUserId(userId, orderStatus);
        List<Orders> list = baseResponse.getList();
        List<OrdersVO> ordersVOS = list.stream().map(arg -> new OrdersVO(arg, fastDFSUtils)).collect(Collectors.toList());
        baseResponse.setList(ordersVOS);
        return baseResponse;
    }

    @PutMapping("/pay/cancel")
    @ApiOperation("用户取消支付后续,将订单回退到待支付")
    public BaseResponse payCancel(String orderNo) {
        if (StrUtil.isEmpty(orderNo)){
            return BaseResponse.error("订单号为空",OrdersController.class);
        }
        OrderDetailQuery orderDetailQuery = new OrderDetailQuery();
        orderDetailQuery.setOrderNo(orderNo);
        orderDetailQuery.setStatus(OrderStatus.WAIT_PAY.name());
        return iOrdersService.editOrder(orderDetailQuery);
    }
    /**
     * 获取所有 当前用户所有订单  分页
     *
     * @param userId
     * @return
     */
    @GetMapping("/page/{userId}/{status}")
    @ApiOperation("获取用户特定状态下所有订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "string"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]", paramType = "path", dataType = "OrderStatus")
    })
    public BaseResponse order(@PathVariable("userId") String userId, @PathVariable("status") Integer status, PageQuery pageQuery) {

        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        pageWrapperQuery.setPageQuery(pageQuery);
        Map<String, Object> eq = new HashMap(2);
        if (status != null) {
            OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
            if (!OrderStatus.ALL.equals(orderStatus)) {
                eq.put("order_status", orderStatus.name());
            }
        }
        if (StrUtil.isNotEmpty(userId)) {
            eq.put("user_id", userId);
        }
        pageWrapperQuery.setEq(eq);
        BaseResponse response = iOrdersService.page(pageWrapperQuery);
        List<Orders> list = response.getPage().getRecords();
        List<OrdersVO> ordersVOS = list.stream().map(arg -> new OrdersVO(arg, fastDFSUtils)).collect(Collectors.toList());
        response.getPage().setRecords(ordersVOS);
        return response;
    }

    @PostMapping
    @ApiOperation("预下单")
    @ApiImplicitParam(name = "orderQuery", value = "下单", paramType = "body", dataType = "OrderQuery")
    public BaseResponse preOrder(@RequestBody OrderDetailQuery orderQuery) {

        return iExhibitionOrderService.preOrder(orderQuery);
    }

    @GetMapping("/status/{orderNo}")
    @ApiOperation("获取展览订单状态")
    @ApiImplicitParam(name = "orderNo", value = "预下单号", paramType = "path", dataType = "String")
    public BaseResponse getOrderStatus(@PathVariable("orderNo") String orderNo) {
        return iExhibitionOrderService.getOrderStatus(orderNo);
    }

    @GetMapping("/preOderNo")
    @ApiOperation("获取唯一订单号(下单前)")
    public BaseResponse getPreOrderNo() {

        return iOrdersService.getOrderNo();
    }

    @PostMapping("/prePay")
    @ApiOperation("预支付订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse prePayOrder(String orderNo) {

        BaseResponse responseEntity = iExhibitionOrderService.prePayOrder(orderNo);
        return responseEntity;
    }

    @PostMapping("/pay/success")
    @ApiOperation("支付成功回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderSuccess(String orderNo) {

        return iExhibitionOrderService.payOrderSuccess(orderNo);
    }


    @PostMapping("/pay/fail")
    @ApiOperation("支付失败回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderFail(String orderNo) {

        return iExhibitionOrderService.payOrderFail(orderNo);
    }

    @PostMapping("/safeguard")
    @ApiOperation("用户申请退款")
    public BaseResponse create(@RequestBody SafeguardDetailQuery safeguardDetailQuery) {
        return iSafeguardService.create(safeguardDetailQuery);
    }

    @ApiOperation("取消订单")
    @PutMapping("/cancel")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse cancelOrder(@Valid
                                    @NotEmpty(message = "订单号不能为空！")
                                    @RequestParam("orderNo") String orderNo) {
        Map<String, Object> condition = new HashMap<>(1);
        condition.put("pay_status", PayStatus.PAY_ON.name());
        BaseResponse response = iOrdersService.changeOrderStatus(orderNo, OrderStatus.CANCELLED, JSON.toJSONString(condition));
        if (response.getCode() == 400) {
            return BaseResponse.error("无效订单或已支付请申请退款！", Orders.class);
        } else {
            return response;
        }
    }

    @ApiOperation("确认完成订单")
    @PutMapping("/complete")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse complete(@Valid
                                 @NotEmpty(message = "订单号不能为空！")
                                 @RequestParam("orderNo") String orderNo,
                                 @NotEmpty @RequestParam("userId") String userId) {
        Map<String, Object> condition = new HashMap<>(1);
        condition.put(OrdersDBColumn.PAY_STATUS, PayStatus.PAY_YES.name());
        condition.put("order_status", OrderStatus.WAIT_COMMENT.name());
        condition.put("user_id", userId);
        BaseResponse response = iOrdersService.changeOrderStatus(orderNo, OrderStatus.COMPLETE, JSON.toJSONString(condition));
        if (response.getCode() == 400) {
            return BaseResponse.error("无效订单或已支付请申请退款！", Orders.class);
        } else {
            return response;
        }
    }

    @ApiOperation("确认收货")
    @PutMapping("/receive")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse receive(@Valid
                                @NotEmpty(message = "订单号不能为空！")
                                @RequestParam("orderNo") String orderNo,
                                @NotEmpty @RequestParam("userId") String userId) {
        Map<String, Object> condition = new HashMap<>(1);
        condition.put(OrdersDBColumn.PAY_STATUS, PayStatus.PAY_YES.name());
        condition.put("order_status", OrderStatus.WAIT_RECEIVE.name());
        condition.put("user_id", userId);
        BaseResponse response = iOrdersService.changeOrderStatus(orderNo, OrderStatus.WAIT_COMMENT, JSON.toJSONString(condition));
        if (response.getCode() == 400) {
            return BaseResponse.error("无效订单或已支付请申请退款！", Orders.class);
        } else {
            return response;
        }
    }


    @PostMapping("/addComment")
    @ApiOperation("评论")
    @ApiImplicitParam(name = "comment", value = "评论实体", paramType = "body", dataType = "Comment")
    public BaseResponse addComment(@RequestBody Comment comment) {
        return iOrderCommentService.addComment(comment);
    }

    @GetMapping("/commentPage")
    @ApiOperation("根据评论主体 评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "targetId", value = "评论主体id", paramType = "query", dataType = "Long"),
            @ApiImplicitParam(name = "pageQuery", value = "分页查询对象", paramType = "query", dataType = "PageQuery")
    })
    public BaseResponse<Page<Comment>> getCommentsByTargetId(Long targetId, PageQuery pageQuery) {
        return iOrderCommentService.getCommentsByTargetId(targetId,pageQuery);
    }

    @PostMapping("/addReply")
    @ApiOperation("评论回复")
    @ApiImplicitParam(name = "reply", value = "评论回复实体", paramType = "body", dataType = "Reply")
    public BaseResponse inserReply(@RequestBody Reply reply) {
        return iOrderCommentReplyService.inserReply(reply);
    }

    @Autowired
    Producer producer;

    public OrderResponse mqpost(@RequestBody TransactionMessage msg) throws IOException {
        /*TransactionMessage msg = new TransactionMessage();
        msg.setMessage(message); //发送的信息
        msg.setRouteKey(SN); //sn号
        msg.setQueue(SN);    //sn号
        msg.setSendSystem(system);//发消息系统
        */
        msg.setCreateTime(LocalDateTime.now());
        producer.createQueue(msg.getQueue());
        System.out.println(msg.getQueue());
        System.out.println(msg.getRouteKey());
        producer.send(msg.getRouteKey(), JSON.toJSONString(msg));
        //String msgs = (String) rabbitTemplate.receiveAndConvert("test1",2000l);
        //String transactionMessage = (String) rabbitTemplate.receiveAndConvert("test1");

        return OrderResponse.ok("发送成功");
    }


}
