package cn.wise.sc.pay.pass.business.app.controller.orders;

import cn.hutool.core.map.MapUtil;
import cn.wise.sc.pay.api.order.service.ISafeguardService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.order.query.SafeguardDetailQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;

/**
 * @description: 维权订单  app
 * @author: qh
 * @create: 2020-03-31 15:30
 **/
@Api(tags = "维权订单(app)")
@CrossOrigin
@RestController
@RequestMapping("/app/safeguard")
public class SafeguardController {

    final
    ISafeguardService iSafeguardService;

    public SafeguardController(ISafeguardService iSafeguardService) {
        this.iSafeguardService = iSafeguardService;
    }

    @PostMapping
    @ApiOperation("用户申请退款")
    public BaseResponse create(@RequestBody SafeguardDetailQuery safeguardDetailQuery) {
        return iSafeguardService.create(safeguardDetailQuery);
    }

    @PutMapping("/repetition")
    @ApiOperation("再次申请退款")
    public BaseResponse edit(@RequestBody SafeguardDetailQuery safeguardDetailQuery) {

        if (safeguardDetailQuery.getStatus() == 0) {
            safeguardDetailQuery.setOrderNo(null);
            return iSafeguardService.edit(safeguardDetailQuery);
        } else {
            return BaseResponse.error("申请失败!", this.getClass().getName());
        }
    }

    @GetMapping("/page")
    @ApiOperation("获取用户退款记录")
    public BaseResponse getRefundPageByUserId(String userId, PageQuery pageQuery) {
        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        pageWrapperQuery.setPageQuery(pageQuery);
        HashMap<String, Object> eq = MapUtil.newHashMap(1);
        eq.put("user_id", userId);
        pageWrapperQuery.setEq(eq);

        return iSafeguardService.page(pageWrapperQuery);
    }

    @PutMapping
    @ApiOperation("关联物流号")
    public BaseResponse refundConnectLogisticsNo(String refundNo, String LogisticsNo) {
        return iSafeguardService.connectLogisticsNo(refundNo, LogisticsNo);
    }
}