package cn.wise.sc.pay.pass.business.app.model.query;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import java.io.Serializable;

/**
 * 微信或者阿里注册参数对象
 */
@Builder
@Data
@NoArgsConstructor
@AllArgsConstructor
@ApiModel
public class OpenIdRegisterQuery implements Serializable {
    /**
     * OpenId
     */
    @ApiModelProperty("Openid")
    @NotBlank(message = "Openid不能为空 ")
    private String openId;
    /**
     * 昵称
     */
    @ApiModelProperty("昵称")
    private String username;
    /**
     * 头像地址字符串
     */
    @ApiModelProperty("头像地址字符串")
    private String multiAvatar;
    /**
     * Openid的类型
     */
    @ApiModelProperty("Openid的类型;0代表wxOpenId,1代表AliOpenId")
    @Max(value = 1, message = "Openid的类型不能大于1")
    @Min(value = 0, message = "Openid的类型不能小于0")
    private Integer type;

    /**
     * 性别
     */
    @NotEmpty(message = "性别不能为空")
    @ApiModelProperty(value = "性别",required = true)
    private String sex;

    /**
     * 城市
     */
    @NotEmpty(message = "所在城市不能为空!")
    @ApiModelProperty(value = "城市",required = true)
    private String city;
}
