package cn.wise.sc.pay.pass.business.app.model.vo;

import cn.wise.sc.pay.business.goods.model.bean.Goods;
import cn.wise.sc.pay.business.goods.model.vo.GoodsVO;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopGoods;
import cn.wise.sc.pay.domain.exhibition.po.Commodity;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.pass.business.app.config.FastDfsUtil;
import com.alibaba.fastjson.JSON;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: app端订单VO
 * @author: qh
 * @create: 2019-12-26 13:58
 **/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class OrdersVO {

    /**
     * 主键
     */
    private Long id;

    /**
     * 订单号
     */
    private String orderNo;

    /**
     * 商品json
     */
    private List goods;

    /**
     * 订单状态
     */
    private String orderStatus;

    /**
     * 支付状态
     */
    private String payStatus;

    /**
     * 用户id
     */
    private Long userId;

    /**
     * 用户电话
     */
    private String userPhone;

    /**
     * 订单总金额
     */
    private BigDecimal totalPrice;

    /**
     * 实际需要支付金额
     */
    private BigDecimal payPrice;

    /**
     * 支付方式
     */
    private String paymentMethodName;

    /**
     * 所属平台
     */
    private String paas;

    /**
     * 备注
     */
    private String remark;

    /**
     * 创建时间
     */
    private Long createTime;

    /**
     * 最后更新时间
     */
    private Long updateTime;

    /**
     * 预留字段
     */
    private String feild1;

    /**
     * 预留字段
     */
    private String feild2;

    /**
     * 预留字段
     */
    private String feild3;

    /**
     * 预留字段
     */
    private String feild4;

    /**
     * 预留字段
     */
    private String feild5;

    /**
     * 商家id
     */
    private Long sellerId;

    /**
     * 物流
     */
    private String logistics;

    /**
     * 店铺id
     */
    private Long shopId;

    private String address;

    private boolean isRefund;

    public OrdersVO(Orders orders) {

        if ("EXHIBITION".equals(orders.getPaas())) {
            this.goods = JSON.parseArray(orders.getGoods(), Commodity.class);
            this.paas = "展览";
        }
        if ("ESHOP".equals(orders.getPaas())) {
            this.goods = JSON.parseArray(orders.getGoods(), EshopGoods.class);
            this.paas = "网上商城";
        }
        if ("cashier".equals(orders.getPaas())) {
            this.goods = JSON.parseArray(orders.getGoods(), Goods.class);
            this.paas = "收银系统";
        }

        this.id = orders.getId();
        this.createTime = orders.getCreateTime();
        this.orderNo = orders.getOrderNo();
        this.orderStatus = getStatus(orders.getOrderStatus());
        this.paymentMethodName = orders.getPaymentMethodName();
        this.payPrice = orders.getPayPrice();
        this.payStatus = getPayStatus(orders.getPayStatus());
        this.remark = orders.getRemark();
        this.sellerId = orders.getSellerId();
        this.shopId = orders.getShopId();
        this.totalPrice = orders.getTotalPrice();
        this.userId = orders.getUserId();
        this.userPhone = orders.getUserPhone();
        this.address = orders.getReceivePlace();
        this.logistics = orders.getFeild2();
        this.isRefund = OrderStatus.REFUNDING.name().equals(orders.getCashier());
    }

    public OrdersVO(Orders orders, FastDfsUtil fastDfsUtil) {

        if ("EXHIBITION".equals(orders.getPaas())) {
            this.goods = JSON.parseArray(orders.getGoods(), Commodity.class);
            this.paas = "展览";
            List<Commodity> list = new ArrayList<>();
            for (Object good : goods) {
                Commodity commodity = (Commodity) good;
                commodity.setCover(fastDfsUtil.fullUrl(commodity.getCover()));
                list.add(commodity);
            }
            this.goods = list;
        }
        if ("ESHOP".equals(orders.getPaas())) {
            this.goods = JSON.parseArray(orders.getGoods(), EshopGoods.class);
            this.paas = "网上商城";
            List<EshopGoods> list = new ArrayList<>();
            for (Object good : goods) {
                EshopGoods commodity = (EshopGoods) good;
                commodity.setOriginal(fastDfsUtil.fullUrl(commodity.getOriginal()));
                list.add(commodity);
            }
            this.goods = list;
        }
        if ("cashier".equals(orders.getPaas())) {
            this.goods = JSON.parseArray(orders.getGoods(), CashierGoodVo.class);
            this.paas = "收银系统";
            List<CashierGoodVo> list = new ArrayList<>();
            for (Object good : goods) {
                CashierGoodVo commodity = (CashierGoodVo) good;
                commodity.setOriginal(fastDfsUtil.fullUrl(commodity.getOriginal()));
                list.add(commodity);
            }
            this.goods = list;
        }

        this.id = orders.getId();
        this.createTime = orders.getCreateTime();
        this.orderNo = orders.getOrderNo();
        this.orderStatus = getStatus(orders.getOrderStatus());
        this.paymentMethodName = orders.getPaymentMethodName();
        this.payPrice = orders.getPayPrice();
        this.payStatus = getPayStatus(orders.getPayStatus());
        this.remark = orders.getRemark();
        this.sellerId = orders.getSellerId();
        this.shopId = orders.getShopId();
        this.totalPrice = orders.getTotalPrice();
        this.userId = orders.getUserId();
        this.userPhone = orders.getUserPhone();
        this.address = orders.getReceivePlace();
        this.isRefund = OrderStatus.REFUNDING.name().equals(orders.getCashier());

        if (OrderStatus.REFUNDING.name().equals(orders.getCashier())) {
            this.orderStatus = "申请退款中";
        }
    }

    /**
     * 数据库支付状态转中文
     *
     * @param dbStatus
     * @return
     */
    private String getStatus(String dbStatus) {
        if (OrderStatus.WAIT_PAY.name().equals(dbStatus)) {
            return "待支付";
        }
        if (OrderStatus.ENTRY_ORDER.name().equals(dbStatus)) {
            return "挂起";
        }
        if (OrderStatus.CANCELLED.name().equals(dbStatus)) {
            return "已取消";
        }
        if (OrderStatus.COMPLETE.name().equals(dbStatus)) {
            return "已完成";
        }
        if (OrderStatus.WAIT_COMMENT.name().equals(dbStatus)) {
            return "待评价";
        }
        if (OrderStatus.WAIT_DELIVERING.name().equals(dbStatus)) {
            return "待发货";
        }
        if (OrderStatus.WAIT_RECEIVE.name().equals(dbStatus)) {
            return "待收货";
        }
        if (OrderStatus.REFUND.name().equals(dbStatus)){
            return "已退款";
        }
        return "无效订单";
    }

    private String getPayStatus(String payStatus) {
        if (PayStatus.PAY_YES.name().equals(payStatus)) {
            return "已支付";
        } else {
            return "未支付";
        }
    }

}
