package cn.wise.sc.pay.pass.business.cashiers.controller;

import cn.hutool.core.map.MapUtil;
import cn.wise.sc.pay.api.merchant.service.IDeviceUserService;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.laas.userbac.business.controller.RoleController;
import cn.wise.sc.pay.laas.userbac.business.controller.UserController;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserInfo;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysRole;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import cn.wise.sc.pay.pass.business.cashiers.config.CommonUrlProperties;
import cn.wise.sc.pay.pass.business.cashiers.model.CheckPasswordQuery;
import cn.wise.sc.pay.pass.business.cashiers.model.Employee;
import com.baomidou.mybatisplus.extension.api.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @author neo.shu
 * @since 2019/12/9 18:27
 */
@CrossOrigin
@RestController
@RequestMapping("/cashiers/user")
@EnableConfigurationProperties(CommonUrlProperties.class)
@Api(value = "cashiers/user", tags = "收银端用户管理")
public class UserControler extends UserController {

    private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();

    public UserControler(UserService userService) {
        super(userService);
    }

    @Autowired
    IMerchantService merchantService;
    @Autowired
    IDeviceUserService deviceUserService;
    @Autowired
    RoleController roleController;
    @Autowired
    CommonUrlProperties commonUrlProperties;
    @Autowired
    IMerchantService iMerchantService;

    @ApiOperation("收银端注册账号")
    @PostMapping("/register")
    @ApiResponses({@ApiResponse(code = 200, message = "成功", response = String.class)})
    public R<String> registerEmployee(@RequestBody Employee employee) {
        boolean res = (boolean) user(employee).getData();
        if (res) {
            Integer userId = ((SysUser) user(employee.getUsername()).getData()).getUserId();
            List<String> deviceList = employee.getDeviceCodes();
            StringBuffer deviceCode = new StringBuffer();
            deviceList.stream().forEach(arg -> {
                deviceCode.append(arg + ",");
            });
            boolean deFlag = (boolean) deviceUserService.addDeviceUser(
                    deviceCode.toString().substring(0, deviceCode.length() - 1), userId.longValue()).getData();
            if (deFlag) {
                return R.ok("添加店员成功");
            } else {
                userDel(userId);
                return R.failed("店员添加失败");
            }
        }
        return R.failed("添加店员失败");
    }

    @ApiOperation("收银端账号密码登录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "username", value = "用户名", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "password", value = "用户密码", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "deviceCode", value = "当前设备号", dataType = "String", paramType = "query")
    })
    @PostMapping("/login")
    public R loginCashier(@RequestParam("deviceCode") String deviceCode,
                          @RequestParam("username") String username, @RequestParam("password") String password) {

        UserInfo user = (UserInfo) info(username).getData();
        if (user == null) {
            return R.failed("用户不存在");
        }
        String pw = user.getSysUser().getPassword();
        boolean flag = ENCODER.matches(password, pw);
        if (!flag) {
            return R.failed("密码错误");
        }
        Map map = new HashMap();
        List<SysRole> list = (List<SysRole>) roleController.getUserRoleById(user.getSysUser().getUserId()).getData();
        map.put("role", list);
        map.put("user", user);

        //获取动态路径的地址
        Map<String, String> urlInfo = MapUtil.newConcurrentHashMap(4);
        assert commonUrlProperties != null;
        urlInfo.put("baseUrl", commonUrlProperties.getBaseUrl());
        urlInfo.put("webViewUrl", commonUrlProperties.getWebViewUrl());
        urlInfo.put("imgUrl", commonUrlProperties.getImageUrl());
        urlInfo.put("rabbitUrl", commonUrlProperties.getRabbitUrl());
        map.put("url", urlInfo);

        Integer userId = user.getSysUser().getUserId();
        Long merchId = user.getSysUser().getMerchId();
        BaseResponse<Merchant> merchantInfo = merchantService.getMerchantInfo(merchId);
        if (merchantInfo.getCode() == 200) {
            Merchant data = merchantInfo.getData();
            if (data.getManageStatus() == 1) {
                return R.failed("当前商户已关闭");
            }
        }
        if ("ADMIN".equals(user.getSysUser().getLevel())) {
            List<LinkedHashMap> results = deviceUserService.getMerchDevices(merchId).getList();
            if (results == null || results.size() == 0) {
                return R.failed("当前设备与商户不匹配");
            }
            boolean f = false;
            for (LinkedHashMap linkMap : results) {
                if (linkMap.get("deviceCode").equals(deviceCode)) {
                    f = true;
                }
            }
            if (f) {
                map.put("device", results);
                return R.ok(map);
            } else {
                return R.failed("当前设备未绑定到商户");
            }
        } else {
            LinkedHashMap result = (LinkedHashMap) deviceUserService.getDeviceUser(deviceCode, userId).getData();
            if (result != null && result.get("id") != null) {
                List<LinkedHashMap> list1 = new ArrayList();
                list1.add(result);
                map.put("device", list1);
                return R.ok(map);
            } else {
                return R.failed("当前设备未绑定商户，无法登录");
            }
        }
    }


    @PostMapping("/check/pwd")
    @ApiOperation("验证敏感操作密码")
    public BaseResponse<Boolean> checkPwd(@RequestBody CheckPasswordQuery checkPasswordQuery) {
       return iMerchantService.checkPwd(checkPasswordQuery.getSeller(),checkPasswordQuery.getPwd());
    }

}
