package cn.wise.sc.pay.pass.business.cashiers.controller.merchant;

import cn.wise.sc.pay.api.merchant.service.IMerchDiscountService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchDiscount;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 店铺折扣
 * @author: qh
 * @create: 2020-09-07 13:36
 **/
@RestController
@CrossOrigin
@RequestMapping("/merchant/discount")
@Slf4j
public class MerchDiscountController {

    final
    IMerchDiscountService iMerchDiscountService;

    public MerchDiscountController(IMerchDiscountService iMerchDiscountService) {
        this.iMerchDiscountService = iMerchDiscountService;
    }

    @GetMapping("/{seller}")
    @ApiOperation("根据商户id获取折扣配置")
    @ApiImplicitParam(name = "seller", value = "商户id")
    BaseResponse<List<MerchDiscount>> getMerchDiscountBySeller(@PathVariable("seller") Long seller){
        return iMerchDiscountService.getMerchDiscountBySeller(seller);
    }

//    @PostMapping("/updateMerchDiscount")
//    @ApiOperation("更新商户折扣信息")
//    BaseResponse<Boolean> updateMerchDiscount(@RequestBody MerchDiscount merchDiscount){
//        return iMerchDiscountService.updateMerchDiscount(merchDiscount);
//    }

    @PostMapping("/addOrUpdateMerchDiscount")
    @ApiOperation("新增一个折扣信息")
    BaseResponse<Boolean> addNewMerchDiscount(@RequestBody MerchDiscount merchDiscount){
        return iMerchDiscountService.addNewMerchDiscount(merchDiscount);
    }

    public static void main(String[] args) {
        MerchDiscount merchDiscount = new MerchDiscount();
        merchDiscount.setId(1L);
        merchDiscount.setSellerId(1L);
        List<MerchDiscount.DiscountSetting> list = new ArrayList<>();
        MerchDiscount.DiscountSetting a = new MerchDiscount.DiscountSetting();
        a.setDiscount(200D);
        a.setId("1");
        MerchDiscount.DiscountSetting b = new MerchDiscount.DiscountSetting();
        b.setDiscount(200D);
       b.setId("1");
       list.add(a);
       list.add(b);
       String str = JSON.toJSONString(list);
       merchDiscount.setDiscountStr(str);
        System.out.println(JSON.toJSONString(merchDiscount));
    }
    @PostMapping("/delete/{id}")
    @ApiOperation("删除一个折扣信息")
    BaseResponse<Boolean> deleteMerchDiscount(@PathVariable("id") Long id){
        return iMerchDiscountService.deleteMerchDiscount(id);
    }

    @PostMapping("/deleteBySeller/{seller}")
    @ApiOperation("按照商户id删除折扣信息")
    BaseResponse<Boolean> deleteMerchDiscountBySeller(@PathVariable("seller") String seller){
        return iMerchDiscountService.deleteMerchDiscountBySeller(seller);
    }
}
