package cn.wise.sc.pay.pass.business.manager.controller;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.laas.userbac.business.controller.RoleController;
import cn.wise.sc.pay.laas.userbac.business.service.MenuService;
import cn.wise.sc.pay.laas.userbac.business.service.RoleMenuService;
import cn.wise.sc.pay.laas.userbac.business.service.RoleService;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysRole;
import cn.wise.sc.pay.laas.userbac.domain.vo.RoleVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author neo.shu
 * @since 2019/12/9 16:10
 */
@CrossOrigin
@RestController
@RequestMapping("manager/role")
@Api(value = "manager/role", tags = "管理端角色管理")
public class UserRoleController extends RoleController {

    @Autowired
    RoleService roleService;

    public UserRoleController(RoleService roleService, RoleMenuService roleMenuService, MenuService menuService) {
        super(roleService, roleMenuService, menuService);
    }

    /**
     * 添加或修改用户角色
     *
     * @param roleVO
     * @return
     */
    @ApiOperation("添加或更新角色")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleId", value = "角色id", dataType = "Integer", paramType = "path", required = false),
            @ApiImplicitParam(name = "roleName", value = "角色名称", dataType = "String", paramType = "path", required = true),
            @ApiImplicitParam(name = "roleCode", value = "角色标识", dataType = "String", paramType = "path", required = true),
            @ApiImplicitParam(name = "roleDesc", value = "角色描述", dataType = "String", paramType = "path", required = true),
            @ApiImplicitParam(name = "menus", value = "角色权限id用逗号拼接的字符串", dataType = "String", paramType = "path")
    })
    @PostMapping("addorUpdateRole")
    public BaseResponse addOrUpdateRole(RoleVO roleVO) {
        SysRole sysRole = new SysRole();
        BeanUtils.copyProperties(roleVO, sysRole);
        if (roleVO.getRoleId() != null) {
            update(sysRole);
            saveRoleMenus(sysRole.getRoleId(), roleVO.getMenus());
            return BaseResponse.ok("角色更新成功");
        } else {
            long code = save(sysRole).getCode();
            if (code == 0) {
                QueryWrapper wrapper = new QueryWrapper();
                wrapper.eq("role_name", sysRole.getRoleName());
                wrapper.eq("role_desc", sysRole.getRoleDesc());
                wrapper.eq("role_code", sysRole.getRoleCode());
                wrapper.orderByDesc("role_id");
                wrapper.last("limit 1");
                SysRole one = roleService.getOne(wrapper);
                if (roleVO.getMenus() != null) {
                    saveRoleMenus(one.getRoleId(), roleVO.getMenus());
                }
            }
            return BaseResponse.ok("角色添加成功");
        }
    }

    @GetMapping("/v1/page")
    @ApiOperation("获取角色数据分页(走这个,那个分页传不进去)")
    public R page(PageQuery pageQuery) {
        Page page = new Page();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());
        return super.getRolePage(page);
    }


    @Autowired
    private Environment environment;

    @ApiOperation("获取图片地址host")
    @GetMapping("/host")
    public BaseResponse<String> getPictureHost() {
        return BaseResponse.ok(environment.getProperty("fdfs.imgaddress"));
    }
}

