package cn.wise.sc.pay.pass.business.manager.controller.exhibition;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.exhibition.query.ExhibitionQuery;
import cn.wise.sc.pay.domain.exhibition.vo.TicketVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 展览管理端控制器
 * @author: qh
 * @create: 2019-12-30 09:57
 **/
@CrossOrigin
@RestController
@Api(description = "管理端展览接口")
@RequestMapping("/manage/exhibition")
public class ExhibitionController {

    @Autowired
    IExhibitionsService iExhibitionService;

    @PutMapping("/store/tickets")
    @ApiOperation("获取票据库存")
    @ApiImplicitParam(name = "ticketVOS", value = "票据集合", paramType = "body", dataType = "TicketVO")
    public BaseResponse updateTicketStore(@RequestBody List<TicketVO> ticketVOS){
        return iExhibitionService.updateTicketStore(ticketVOS);
    }

    @GetMapping("/store/exhibition/{id}")
    @ApiOperation("获取展览的票据信息")
    @ApiImplicitParam(name = "id", value = "展览id", paramType = "path", dataType = "Long")
    public BaseResponse getExhibitionStore(@PathVariable("id") Long id){
        return iExhibitionService.getExhibitionStore(id);
    }


    @PostMapping("/new")
    @ApiImplicitParam(name = "exhibitionQuery", value = "展览查询类", paramType = "body", dataType = "ExhibitionQuery")
    @ApiOperation("添加一个新的展览")
    public BaseResponse addExhibition(@RequestBody ExhibitionQuery exhibitionQuery) {

        return iExhibitionService.addExhibition(exhibitionQuery);
    }

    @DeleteMapping("{id}")
    @ApiImplicitParam(name = "id", value = "id", paramType = "ath", dataType = "int")
    @ApiOperation("删除展览")
    public BaseResponse delExhibition(@PathVariable("id") Integer id) {
        return iExhibitionService.delExhibition(id);
    }

    @PutMapping
    @ApiImplicitParam(name = "exhibitionQuery", value = "展览查询类", paramType = "body", dataType = "ExhibitionQuery")
    @ApiOperation("编辑展览")
    public BaseResponse editExhibition(@RequestBody ExhibitionQuery exhibitionQuery) {
        return iExhibitionService.editExhibition(exhibitionQuery);
    }

    @PostMapping("/upload")
    @ApiOperation("上传图片")
    public BaseResponse upload(MultipartFile file) {

        return iExhibitionService.upload(file);
    }

    @PutMapping("/{id}/{type}")
    @ApiOperation("上新/推荐")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "展览id", paramType = "path", dataType = "long"),
            @ApiImplicitParam(name = "type", value = "展示类型 0:上新(默认) 1:推荐", paramType = "path", dataType = "int")
    })
    public BaseResponse editShowType(@PathVariable("id") long id, @PathVariable("type") int type) {

        return iExhibitionService.editShowType(id, type);
    }

    @GetMapping("/list")
    @ApiOperation("获取所有(条件)展览,不分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型(字符串)", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "showType", value = "展示类型 0:上新(默认) 1:推荐", paramType = "query", dataType = "string")
    })
    public BaseResponse getExhibitionList(String type, Integer showType, TimeBucket timeBucket) {

        return iExhibitionService.getExhibitionList(type, showType, timeBucket);
    }

    @GetMapping("/advertising/list")
    @ApiOperation("获取所有(条件)展览广告位,不分页")
    public BaseResponse getAdvertising() {
        return iExhibitionService.getAdvertising();
    }

    @GetMapping("/page")
    @ApiOperation("获取所有(条件)展览,分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型(字符串)", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "title", value = "标题", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "option", value = "属性", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "状态", paramType = "query", dataType = "int")
    })
    public BaseResponse getExhibitionPage(PageQuery pageQuery, String title, String type, Integer option, Integer status) {

        PageWrapperQuery page = PageWrapperQueryUtil.create();
        page.setPageQuery(pageQuery);
        Map like = new HashMap();
        if (StrUtil.isNotEmpty(title)) {
            like.put("title", title);
        }
        if (StrUtil.isNotEmpty(type)) {
            like.put("type", type);
        }
        page.setLike(like);
        Map eq = new HashMap();

        if (option != null) {
            eq.put("show_type", option.intValue());
        }
        if (status != null) {
            eq.put("status", status);
        }
        page.setEq(eq);

        return iExhibitionService.getExhibitionPage(page);
    }

    @GetMapping("/{id}")
    @ApiOperation("获取展览详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "long")
    })
    public BaseResponse getExhibitionDetail(@PathVariable("id") long id) {

        return iExhibitionService.getExhibitionDetail(id);
    }

    @PutMapping("/QRCode")
    @ApiOperation("核销入场二维码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "enQCodeJSON", value = "enQCodeJSON", paramType = "query", dataType = "String")
    })
    public BaseResponse entranceCheck(@RequestParam("enQCodeJSON") String enQCodeJSON) {

        return iExhibitionService.entranceCheck(enQCodeJSON);
    }

    @GetMapping("/QRCode")
    @ApiOperation("获取入场二维码")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse getQRCode(@RequestParam("orderNo") String orderNo) {
        return iExhibitionService.getQRCode(orderNo);
    }

}
