package cn.wise.sc.pay.pass.business.manager.controller.merchant;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.order.service.IBillsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.common.core.util.DateTimeUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

@RestController
@CrossOrigin
@RequestMapping("/manage/merchant/bills")
@Slf4j
@Api(description = "交易记录")
public class BillController {
    final
    IBillsService iBillsService;

    public BillController(IBillsService iBillsService) {
        this.iBillsService = iBillsService;
    }

    /**
     * 更具条件获取 分页
     *
     * @return
     */
    @PostMapping("/page")
    @ApiOperation("更具条件获取 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "开始时间", name = "start", paramType = "query", dataType = "long"),
            @ApiImplicitParam(value = "结束时间", name = "end", paramType = "query", dataType = "long"),
            @ApiImplicitParam(value = "店铺id", name = "shopId", paramType = "query", dataType = "long"),
            @ApiImplicitParam(value = "交易类型", name = "tradeType", paramType = "query", dataType = "String"),
            @ApiImplicitParam(value = "交易状态", name = "tradeStatus", paramType = "query", dataType = "String")
    })
    public BaseResponse getBillsPage(long start, long end, long shopId, String tradeType, String tradeStatus, PageQuery pageQuery) {

        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        pageWrapperQuery.setPageQuery(pageQuery);
        if (DateTimeUtils.check(start, end)) {
            Map<String, Object> ge = MapUtil.newConcurrentHashMap(1);
            Map<String, Object> lt = MapUtil.newConcurrentHashMap(1);
            ge.put("transaction_time", start);
            lt.put("transaction_time", end);
            pageWrapperQuery.setGe(ge);
            pageWrapperQuery.setLt(lt);
        }
        Map<String, Object> eq = MapUtil.newConcurrentHashMap(3);
        if (shopId != 0) {
            eq.put("ship_no", shopId);
            pageWrapperQuery.setEq(eq);
        }
        if (StrUtil.isNotEmpty(tradeType)) {
            eq.put("trade_type", tradeType);
            pageWrapperQuery.setEq(eq);
        }
        if (StrUtil.isNotEmpty(tradeStatus)) {
            eq.put("trade_status", tradeStatus);
            pageWrapperQuery.setEq(eq);
        }
        return iBillsService.getBillsPage(pageWrapperQuery);
    }
}
