package cn.wise.sc.pay.pass.business.manager.controller.merchant;

import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.merchant.query.MerchantQuery;
import cn.wise.sc.pay.domain.merchant.vo.StoreVo;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author lqk
 * @since 2019/12/17
 */
@RestController
@CrossOrigin
@RequestMapping("/manage/merchant/")
@Slf4j
@Validated
public class MerchantController {

    final
    IMerchantService merchantService;

    public MerchantController(IMerchantService merchantService) {
        this.merchantService = merchantService;
    }

    @GetMapping("getMerchantPage")
    public ResponseEntity<Page<Merchant>> getMerchantPage(Integer status, String merchantName,
                                                          PageQuery pageQuery, TimeBucket timeBucket) {
        Long sTime = 0L;
        Long eTime = 19999999999999L;
        if(timeBucket.getStartTime() != null && timeBucket.getEndTime() != null){
            sTime = timeBucket.getStartTime();
            eTime = timeBucket.getEndTime();
        }
        BaseResponse baseResponse = merchantService.getMerchantPage(status, merchantName,
                pageQuery.getPageNo(),pageQuery.getPageSize(),sTime,eTime);
        return ResponseEntity.ok((Page<Merchant>)baseResponse.getData());
    }
    @PutMapping("updateMerchant")
    public ResponseEntity updatemerchant(@RequestBody Merchant merchant) {
        return ResponseEntity.ok(merchantService.updatemerchant(merchant).getData());
    }
    @PostMapping("addMerchant")
    public ResponseEntity addMerchant(@RequestBody MerchantQuery merchantQuery) {
        BaseResponse response = merchantService.addMerchant(merchantQuery);
        return ResponseEntity.ok(response.getData());
    }

    @DeleteMapping("deleteMerchant")
    public ResponseEntity deleteMerchant(Long merchantId) {
        return ResponseEntity.ok(merchantService.deleteMerchant(merchantId).getData());
    }
    @GetMapping("getMerchantInfo")
    public ResponseEntity<Merchant> getMerchantInfo(Long id) {
        return ResponseEntity.ok(merchantService.getMerchantInfo(id).getData());
    }
    //根据商户id取消商家白名单
    @ApiOperation(value = "根据id取消白名单")
    @ApiImplicitParam(name = "id",value = "商户id",dataType = "long",paramType = "path")
    @PutMapping("cancelWhiteList/{id}")
    public ResponseEntity cancelWhiteList(@PathVariable Long id){

        BaseResponse responseEntity = merchantService.cancelWhiteList(id);
        return ResponseEntity.ok(responseEntity.getData());
    }

    //根据商户id添加商家白名单
    @ApiOperation(value = "根据id添加白名单")
    @ApiImplicitParam(name = "id",value = "商户id",dataType = "long",paramType = "path")
    @PutMapping("addWhiteList/{id}")
    public ResponseEntity addWhiteList(@PathVariable Long id){
        BaseResponse responseEntity = merchantService.addWhiteList(id);
        return ResponseEntity.ok(responseEntity.getData());
    }
    @GetMapping("like")
    public ResponseEntity getMerchList(@RequestParam("merchName") String merchName) {
        return ResponseEntity.ok(merchantService.getMerchList(merchName));
    }
    @GetMapping("/userStaff")
    public BaseResponse getMerchantStaff(Long merchId, Integer roleId, String username, PageQuery page) {
        return merchantService.getMerchantStaff(merchId, roleId, username, page.getPageNo(),page.getPageSize());
    }

    @PostMapping("getMerchNames")
    public BaseResponse getMerchNameByIds(@RequestBody List<Long> merchIds) {
        return merchantService.getMerchNameByIds(merchIds);
    }

    @GetMapping("getMerchOrders")
    public ResponseEntity getMerchantOrders(String merchAppid, TimeBucket timeBucket, String status) {
        return ResponseEntity.ok(merchantService.getMerchantOrders(merchAppid, timeBucket.getStartTime(),timeBucket.getEndTime(), status).getData());
    }

    @GetMapping("merchStaff")
    public BaseResponse getMerchStaff(Long merchId, String username, PageQuery pageQuery){
      return merchantService.getMerchStaff(merchId, username, pageQuery.getPageNo(),pageQuery.getPageSize());
    }

    @GetMapping("merchantExport")
    public void exportMerchantList(@RequestParam("status") Integer status, HttpServletResponse response){
       merchantService.exportMerchantList(status, response);
    }

    @ApiOperation(value = "根据商户名称寻找商户id")
    @ApiImplicitParam(name = "merchantName",value = "商户名称",dataType = "String",paramType = "query")
    @GetMapping("name")
    public ResponseEntity<List<Long>> getMerchantIdByName(String merchantName){
        BaseResponse<List<Long>> merchantIdByName = merchantService.getMerchantIdByName(merchantName);
        return ResponseEntity.ok(merchantIdByName.getData());
    }

    @GetMapping("/{id}")
    @ApiOperation("获取商铺信息")
    public BaseResponse getStore(@PathVariable("id") Long id) {
        return merchantService.getStoreInfo(id);
    }

    @PutMapping("/storeInfo")
    @ApiOperation("更新商铺信息")
    public BaseResponse updateStore(@RequestBody StoreVo storeVo) {
        return merchantService.updateStore(storeVo);
    }
}
