package cn.wise.sc.pay.pass.business.manager.controller.merchant;

import cn.wise.sc.pay.api.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.api.merchant.service.IProfitAllocationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;

/**
 * 商户店铺控制器
 */
@RestController
@CrossOrigin
@RequestMapping("/manage/merchant/money")
@Slf4j
public class MerchantProfitController {

    private final IMerchantProfitService merchantProfitService;

    public MerchantProfitController(IMerchantProfitService merchantProfitService) {
        this.merchantProfitService = merchantProfitService;
    }

    @GetMapping("/")
    @ApiOperation(value = "根据店铺id查询相关分润额")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "merchantId",value = "商户id",paramType = "query",dataType = "Long"),
            @ApiImplicitParam(name = "strategy",value = "今天 1，本月 2，本季度 3，本年 4，自定义时间 5",paramType = "query",dataType = "Integer"),
            @ApiImplicitParam(name = "startTime",value = "自定义其实时间",paramType = "query",dataType = "Long"),
            @ApiImplicitParam(name = "endTime",value = "自定义结束时间",paramType = "query",dataType = "Long")
    })
    public BaseResponse getMoneyByStrategy(@RequestParam("merchantId") Long merchantId,
                                           @RequestParam("strategy") Integer strategy,
                                           @RequestParam("startTime") Long startTime,
                                           @RequestParam("endTime") Long endTime){
        return merchantProfitService.getMoneyByStrategy(merchantId,strategy,startTime,endTime);
    }


    @PostMapping("/addProfit")
    @ApiOperation(value = "交易成功分润额")
    public BaseResponse addMoneyData(@RequestParam("merchantId") Long merchantId,
                                     @RequestParam("orderId")Long orderId,
                                     @RequestParam("payment")String payment,
                                     @RequestParam("money")BigDecimal money,
                                     @RequestParam("createTime")Long createTime){
        return merchantProfitService.addMoneyData(merchantId,orderId,payment,money,createTime);
    }

}
