package cn.wise.sc.pay.api.association.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.association.bean.Blacklist;
import cn.wise.sc.pay.domain.association.query.BlacklistQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

@FeignClient(
    value = "saas-v1-association",
    path = "/association/blacklist",
    fallback = BlacklistServiceImpl.class)
public interface IBlacklistService {

  /**
   * 获取社群黑名单
   *
   * @param userName 用户名称
   * @param pageQuery 分页实体
   * @return
   */
  @GetMapping("/list")
  BaseResponse<Page<Blacklist>> list(
      @RequestParam(value = "userName", required = false) String userName,
      @SpringQueryMap PageQuery pageQuery);

  /**
   * 新增黑名单成员
   *
   * @param blacklistQuery 黑名单查询类
   * @return
   */
  @PostMapping("/addBlacklist")
  BaseResponse addBlacklist(@RequestBody BlacklistQuery blacklistQuery);

  /**
   * 移除黑名单成员
   *
   * @param id 黑名单id
   * @return
   */
  @DeleteMapping("/deleteBlacklist")
  BaseResponse deleteBannerById(@RequestParam("id") Long id);
}
