package cn.wise.sc.pay.api.association.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.association.bean.Concern;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@FeignClient(value = "saas-v1-association", path = "/association/concern/", fallback = ConcernServiceImpl.class)
public interface IConcernService {

    /**
     * 关注对方
     *
     * @param concern
     * @return
     */
    @PostMapping("addConcern")
    ResponseEntity addConcern(@RequestBody Concern concern);

    /**
     * 取消关注
     *
     * @param meId 我的id
     * @param heId 被关注人id
     * @return
     */
    @DeleteMapping("deleteConcern")
    ResponseEntity deleteConcern(@RequestParam("meId") Long meId, @RequestParam("heId") Long heId);

    /**
     * 我的关注/粉丝列表
     *
     * @param meId 我的id
     * @param type 类型
     * @return
     */
    @GetMapping("myConcerns")
    ResponseEntity<List<Concern>> getConcernList(@RequestParam("meId") Long meId, @RequestParam("type") Integer type);

    /**
     * 首页数据
     *
     * @param userId 我的id
     * @return
     */
    @GetMapping("mainData")
    ResponseEntity<Map<String, Integer>> mainData(@RequestParam("userId") Long userId);

    /**
     * 是否关注
     *
     * @param meId 我的id
     * @param heId 被关注人id
     * @return
     */
    @GetMapping("judgeConcern")
    BaseResponse judgeConcern(@RequestParam("meId") Long meId, @RequestParam("heId") Long heId);
}
