package cn.wise.sc.pay.business.association.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.query.ArticleQuery;
import cn.wise.sc.pay.domain.association.vo.ArticleVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.http.ResponseEntity;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
public interface IArticleService extends IService<Article> {

    /**
     * 新增社群文章
     *
     * @return
     */
    BaseResponse addArticle(ArticleQuery articleQuery);

    /**
     * 查询未审核文章列表
     *
     * @param timeBucket
     * @param pageQuery
     * @return
     */
    BaseResponse<Page<Article>> articlesNotVerify(String authorName, TimeBucket timeBucket, PageQuery pageQuery);

    /**
     * 更改文章审核状态 可批量操作
     *
     * @param ids
     * @param status
     * @return
     */
    BaseResponse updateArticleStatus(List<Long> ids, byte status);

    /**
     * 通过审核的文章列表
     *
     * @param type
     * @param name
     * @param title
     * @param pageQuery
     * @return
     */
    BaseResponse<Page<Article>> articlesVerify(Integer type, String order, String name, String title, PageQuery pageQuery);

    /**
     * 删除文章
     *
     * @param idList
     * @return
     */
    BaseResponse deleteArticlesById(List<Long> idList);

    /**
     * 上传图片或文件
     *
     * @param file
     * @return
     */
    BaseResponse uploadPic(MultipartFile file);

    /**
     * 根据id获取图文详情
     * @param id
     * @return
     */
    BaseResponse selectArticleInfo(Long id);

    /**
     * 更新评论数 +1
     * @param targetId
     * @return
     */
    boolean updateArticleComNum(Long targetId);

    /**
     * 根据用户id获取用户收藏的文章
     * @param userId
     * @param pageQuery
     * @return
     */
    BaseResponse<Page<Article>> getCollectionArticle(Long userId, PageQuery pageQuery);

    /**
     * 图文/文章 浏览量 +1
     * @param id
     * @return
     */
    BaseResponse updateArticleViewNum(Long id);
    /**
     * 用户发布文章列表
     * @param userId
     * @param pageQuery
     * @return
     */
    BaseResponse<Page<Article>> getUserPublishArticle(Long userId,PageQuery pageQuery);

    /**
     * 用户点赞的文章列表
     * @param userId
     * @param pageQuery
     * @return
     */
    BaseResponse<Page<Article>> getUserPraiseArticle(Long userId, PageQuery pageQuery);

    /**
     * 我文章的总点赞数
     * @param userId
     * @return
     */
    Integer getMyPraiseSum(Long userId);

    /**
     * 获取上一篇或下一篇图文
     * @param articleId
     * @param pon
     * @return
     */
    BaseResponse preOrNextArticle(Long articleId, Integer pon, Integer status);
}
