package cn.wise.sc.pay.business.association.service.impl;

import cn.wise.sc.pay.business.association.mapper.TopicMapper;
import cn.wise.sc.pay.business.association.service.IArticleTopicService;
import cn.wise.sc.pay.business.association.service.ITopicService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.association.bean.Topic;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.sf.jsqlparser.statement.select.Top;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-25
 */
@Service
public class TopicServiceImpl extends ServiceImpl<TopicMapper, Topic> implements ITopicService {

    @Autowired
    IArticleTopicService articleTopicService;

    @Override
    public BaseResponse<List<Topic>> selectList() {
        QueryWrapper<Topic> wrapper = new QueryWrapper();
        wrapper.orderByAsc("order_num");
        List<Topic> result = this.list(wrapper);
        return BaseResponse.ok(result);
    }

    @Override
    public BaseResponse<Page<Topic>> pageTopic(String name, PageQuery pageQuery) {
        QueryWrapper<Topic> wrapper = new QueryWrapper();
        Page<Topic> page = new Page<>();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());
        wrapper.orderByAsc("order_num");
        if (StringUtils.isNotEmpty(name)) {
            wrapper.like("name", name);
        }
        page = (Page<Topic>) this.page(page, wrapper);
        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse insertTopic(Topic topic) {
        QueryWrapper<Topic> wrapper = new QueryWrapper<>();
        wrapper.eq("name",topic.getName());
        List<Topic> list = this.list(wrapper);
        if(list != null && list.size() > 0){
            return BaseResponse.error("话题已存在",Topic.class);
        }
        wrapper = new QueryWrapper<>();
        wrapper.last("limit 1");
        wrapper.orderByDesc("order_num");
        Topic last = this.getOne(wrapper);
        int order = 1;
        if(last != null)
        {
            order = last.getOrderNum() + 1;
        }
        Topic insertE = Topic.builder()
                .name(topic.getName())
                .orderNum(order)
                .createTime(System.currentTimeMillis())
                .updateTime(System.currentTimeMillis())
                .build();
        boolean result = this.save(insertE);
        return result ? BaseResponse.ok("添加成功") : BaseResponse.error("添加失败", Topic.class);
    }

    @Override
    public BaseResponse updateTopic(Topic topic) {

        QueryWrapper<Topic> wrapper = new QueryWrapper<>();
        wrapper.eq("name",topic.getName());
        List<Topic> list = this.list(wrapper);
        if(list != null && list.size() > 0){
            return BaseResponse.error("话题已存在",Topic.class);
        }

        boolean result = this.updateById(topic);
        return result ? BaseResponse.ok("更新成功") : BaseResponse.error("更新失败",Topic.class);
    }

    @Override
    public BaseResponse updateTopicOrder(Long tId, Long oId) {
        Topic topic1 = this.getById(tId);
        Topic topic2 = this.getById(oId);
        if(topic1 == null || topic2 == null)
        {
            return BaseResponse.error("参数错误",Topic.class);
        }
        int order = topic1.getOrderNum();
        topic1.setOrderNum(topic2.getOrderNum());
        topic2.setOrderNum(order);
        this.updateById(topic1);
        boolean result = this.updateById(topic2);
        return BaseResponse.ok("更新成功");
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteTopicById(Long id) {
        boolean result = this.removeById(id);
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("topic_id", id);
        articleTopicService.remove(wrapper);
        return result ? BaseResponse.ok("删除成功") : BaseResponse.error("删除失败",Topic.class);
    }
}
