package cn.wise.sc.pay.business.cashier.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.business.cashier.util.GoodsSpecExcelUtils;
import cn.wise.sc.pay.business.cashier.util.SpinnerWriteHandler;
import cn.wise.sc.pay.business.goods.model.GoodsResponse;
import cn.wise.sc.pay.business.goods.model.bean.Goods;
import cn.wise.sc.pay.business.goods.model.query.GoodsQuery;
import cn.wise.sc.pay.business.goods.model.query.GoodsScanQuery;
import cn.wise.sc.pay.business.goods.model.query.GoodsTypeQuery;
import cn.wise.sc.pay.business.goods.model.vo.GoodsExcel;
import cn.wise.sc.pay.business.goods.model.vo.GoodsTypeVO;
import cn.wise.sc.pay.business.goods.model.vo.GoodsVO;
import cn.wise.sc.pay.business.goods.service.IGoodsService;
import cn.wise.sc.pay.business.goods.service.IGoodsTypeService;
import cn.wise.sc.pay.business.goods.service.ISpcDetailService;
import cn.wise.sc.pay.business.goods.service.ISpecificationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import com.alibaba.excel.EasyExcelFactory;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.env.Environment;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 商品控制器
 * @author: qh
 * @create: 2019-12-04 11:46
 **/
@Api(description = "商品")
@RequestMapping("/goods")
@RestController
@CrossOrigin
public class GoodsController {
    private final Environment environment;
    final
    IGoodsService iGoodsService;
    final
    ISpecificationService iSpcService;
    final
    ISpcDetailService iSpcDetailService;
    final
    IGoodsTypeService iTypeService;
    final
    IMerchantService iMerchantService;

    private final FastFileStorageClient fastFileStorageClient;

    public GoodsController(Environment environment, IGoodsService iGoodsService, ISpecificationService iSpcService, ISpcDetailService iSpcDetailService, IGoodsTypeService iTypeService, IMerchantService iMerchantService, FastFileStorageClient fastFileStorageClient) {
        this.environment = environment;
        this.iGoodsService = iGoodsService;
        this.iSpcService = iSpcService;
        this.iSpcDetailService = iSpcDetailService;
        this.iTypeService = iTypeService;
        this.iMerchantService = iMerchantService;
        this.fastFileStorageClient = fastFileStorageClient;
    }

    @GetMapping("/list/{sellerId}")
    @ApiOperation("所有商品集合(管理端)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "key", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "状态 0下架 1上架", paramType = "query", dataType = "int")
    })
    public BaseResponse list(@PathVariable("sellerId") int sellerId, String key, String type, int status) {
        return iGoodsService.list(sellerId, key, type, status);
    }

    @GetMapping("/page/v2/seller/{seller}")
    @ApiOperation("查询商品订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "key", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "状态 0下架 1上架", paramType = "query", dataType = "int")
    })
    public BaseResponse pageSeller(PageQuery pageQuery, @PathVariable("seller") int seller,
                                   String key, String type, int status) {

        Page page = new Page(pageQuery.getPageNo(),pageQuery.getPageSize());
        QueryWrapper qw = new QueryWrapper();

        if (StrUtil.isNotEmpty(type)) {
            qw.like("type", type);
        }
        if (seller != 0) {
            qw.eq("seller_id", seller);
        }
        if (StrUtil.isNotEmpty(key)) {
            try {
                int id = Integer.valueOf(key);
                qw.eq("id", id);
            } catch (Exception ex) {
                qw.like("name", key);
            }
        }
        if (status != -1) {
            qw.eq("status", status);
        }
        iGoodsService.page(page, qw);
        return  BaseResponse.ok(page);
    }

    @GetMapping("/list/user")
    @ApiOperation("按照类型获取list")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "状态 0下架 1上架 -1:全部", paramType = "query", dataType = "int")
    })
    public BaseResponse listUser(String key, String type, int status) {
        return iGoodsService.list(0, "", type, 1);
    }

    @GetMapping("/spc/{goodsId}")
    @ApiOperation("获取商品规格")
    @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    public BaseResponse getGoodsSpc(@PathVariable("goodsId") int goodsId) {
        return iSpcService.getGoodsSpc(goodsId);
    }

    @GetMapping("/{goodsId}")
    @ApiOperation("获取商品")
    @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    public BaseResponse getGoods(@PathVariable("goodsId") int goodsId) {
        return (iGoodsService.getGoods(goodsId));
    }

    @PostMapping
    @ApiOperation("添加一个商品")
    @ApiImplicitParam(name = "goodsQuery", value = "商品", paramType = "body", dataType = "GoodsQuery")
    public BaseResponse newGoods(@RequestBody GoodsQuery goodsQuery) {
        return (iGoodsService.newGoods(goodsQuery));
    }

    @PutMapping
    @ApiOperation("编辑商品")
    @ApiImplicitParam(name = "goodsQuery", value = "商品", paramType = "body", dataType = "GoodsQuery")
    public BaseResponse edit(@RequestBody GoodsQuery goodsQuery) {
        return iGoodsService.editGoods(goodsQuery);
    }

    @DeleteMapping("/{goodsId}/{sellerId}")
    @ApiOperation("删除商品")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    })
    public BaseResponse delete(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.delGoods(goodsId, sellerId);
    }

    @DeleteMapping("/spc/{spcId}/{sellerId}")
    @ApiOperation("删除商品一级规格")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "spcId", value = "一级规格id", paramType = "path", dataType = "int")
    })
    public BaseResponse deleteSpc(@PathVariable("spcId") int spcId, @PathVariable("sellerId") int sellerId) {
        return iSpcService.delSpecification(spcId, sellerId);
    }

    @DeleteMapping("/spc/detail/{detailId}/{sellerId}")
    @ApiOperation("删除商品二级规格")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "detailId", value = "二级规格id", paramType = "path", dataType = "int")
    })
    public BaseResponse deleteSpcDetal(@PathVariable("detailId") int detailId, @PathVariable("sellerId") int sellerId) {
        return iSpcDetailService.delSpcDetail(detailId, sellerId);
    }

    @PutMapping("/status/1/{goodsId}/{sellerId}")
    @ApiOperation("上架")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int")
    })
    public BaseResponse shelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.shelveGoods(goodsId, sellerId);
    }

    @PutMapping("/status/0/{goodsId}/{sellerId}")
    @ApiOperation("下架")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int"),
    })
    public BaseResponse unShelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId) {
        return iGoodsService.unshelveGoods(goodsId, sellerId);
    }

    @PutMapping("/manage/status/0/{goodsId}/{sellerId}")
    @ApiOperation("下架(非主观下架,带原因)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "goodsId", value = "商品id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "cause", value = "原因", paramType = "query", dataType = "string")
    })
    public GoodsResponse unShelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId, String cause) {
        return iGoodsService.unshelveGoods(goodsId, sellerId, cause);
    }

    @PostMapping("/list/waitAudit")
    @ApiOperation("获取待审核的商品集合")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "key", value = "商品名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "shopId", value = "商铺名", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "goodsType", value = "商品类型", paramType = "query", dataType = "string"),
    })
    public BaseResponse getWaitingAuditList(@RequestBody PageQuery pageQuery, String key, Long shopId, String goodsType) {
        BaseResponse baseResponse = iGoodsService.getWaitingAuditPage(pageQuery, key, shopId, goodsType);
        return goodShopIdToShopName(baseResponse);
    }

    @PutMapping("/audit")
    @ApiOperation("审核商品")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsId", value = "商户id", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "状态  1:通过   3:不通过", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "cause", value = "原因", paramType = "query", dataType = "string"),
    })
    public BaseResponse auditGoods(long goodsId, int status, String cause) {
        return iGoodsService.auditGoods(goodsId, cause, status);
    }

    /**
     * @param pageQuery
     * @param key
     * @param goodsType
     * @return
     */
    @PostMapping("/list")
    @ApiOperation("获取所有的商品集合")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "key", value = "商品名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "shopId", value = "商铺id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "goodsType", value = "商品类型", paramType = "query", dataType = "string"),
    })
    public BaseResponse list(@RequestBody PageQuery pageQuery, String key, long shopId, String goodsType) {
        BaseResponse baseResponse = iGoodsService.getPage(pageQuery, key, shopId, goodsType);
        return goodShopIdToShopName(baseResponse);
    }

    @PostMapping("/type")
    @ApiOperation("新增一个类型")
    @ApiImplicitParam(name = "typeQuery", value = "商品类型", paramType = "body", dataType = "GoodsTypeQuery")
    public BaseResponse newGoodsType(@RequestBody GoodsTypeQuery typeQuery) {
        return iTypeService.newGoodsType(typeQuery);
    }

    @GetMapping("/type/list/{sellerId}")
    @ApiOperation("获取类型集合")
    @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int")
    public BaseResponse typeList(@PathVariable("sellerId") int sellerId, short status) {
        return iTypeService.getList(sellerId, status);
    }

    @GetMapping("/app/type/list/{sellerId}")
    @ApiOperation("获取类型集合")
    @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int")
    public BaseResponse getAppGoodsType(@PathVariable("sellerId") int sellerId, short status) {
        return iTypeService.getAppList(sellerId, status);
    }

    @PutMapping("/type")
    @ApiOperation("编辑一个类型")
    @ApiImplicitParam(name = "typeQuery", value = "商品类型", paramType = "body", dataType = "GoodsTypeQuery")
    public BaseResponse editType(@RequestBody GoodsTypeQuery typeQuery) {
        return iTypeService.editGoodsType(typeQuery);
    }

    @DeleteMapping("/type/{id}/{sellerId}")
    @ApiOperation("删除一个类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "类型id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int")
    })
    public BaseResponse delType(@PathVariable("id") int id, @PathVariable("sellerId") int sellerId) {
        return iTypeService.delGoodsType(id, sellerId);
    }

    @DeleteMapping("/typeAndUnShelveGoods/{id}/{sellerId}")
    @ApiOperation("删除一个类型以及下架该类别下的所有商品")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "类型id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "path", dataType = "int")
    })
    public BaseResponse delTypeAndUnShelveGoods(@PathVariable("id") int id, @PathVariable("sellerId") int sellerId) {
        return iTypeService.delTypeAndUnShelveGoods(id, sellerId);
    }

    /**
     * 将商品所属店铺名展示出来
     *
     * @param baseResponse
     * @return
     */
    private BaseResponse goodShopIdToShopName(BaseResponse baseResponse) {

        if (baseResponse.getCode() != 400) {
            List<GoodsVO> list = baseResponse.getPage().getRecords();
            if (list.size() == 0) {
                return BaseResponse.okMsg("没有相关数据", Goods.class);
            }
            List<Long> shopIds = list.stream().map(arg -> arg.getSellerId()).collect(Collectors.toList());
            BaseResponse merchNameByIds = iMerchantService.getMerchNameByIds(shopIds);
            List<HashMap> list1 = merchNameByIds.getList();

            List<Merchant> merchantList = new ArrayList<>();
            for (HashMap map : list1) {
                Merchant merchant = BeanUtil.mapToBean(map, Merchant.class, true);
                merchantList.add(merchant);
            }
            List<GoodsVO> goodsVOS = new ArrayList<>();
            if (merchNameByIds.getCode() != 400) {
                String host = environment.getProperty("fdfs.imgaddress");
                list.stream().forEach(arg -> {
                    Merchant merchant = merchantList.stream().filter(
                            param -> param.getId() == arg.getSellerId().longValue())
                            .findFirst().orElse(null);
                    GoodsVO goodsVO = new GoodsVO();
                    BeanUtil.copyProperties(arg, goodsVO);
                    if (StrUtil.isNotBlank(goodsVO.getCover())) {
                        goodsVO.setCover(host + goodsVO.getCover());
                    } else {
                        goodsVO.setCover("");
                    }
                    if (merchant != null) {
                        goodsVO.setShopName(merchant.getMerchName());
                    } else {
                        goodsVO.setShopName("");
                    }
                    goodsVOS.add(goodsVO);
                });
            }
            Page page = baseResponse.getPage();
            page.setRecords(goodsVOS);
            page.setTotal(baseResponse.getPage().getTotal());
            page.setCurrent(baseResponse.getPage().getCurrent());
            baseResponse.setPage(page);
        }
        return baseResponse;
    }

    @GetMapping("/downloadGoodsExcel")
    public void downloadGoodsExcel(String merchantIdStr, HttpServletRequest request, HttpServletResponse response) {
        if (!StringUtils.isNumeric(merchantIdStr)) {
            throw new RuntimeException("商户id为空");
        }
        int merchantId = Integer.parseInt(merchantIdStr);
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        try {
            String fileName = URLEncoder.encode("商品上新", "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            BaseResponse base = iTypeService.getList(merchantId, (short) 0);
            if (base.getCode() == 200) {
                List<GoodsTypeVO> list = (List<GoodsTypeVO>) base.getData();
                if (list == null || list.size() == 0) {
                    throw new RuntimeException("请在管理端先添加商品类型");
                }
                List<GoodsExcel> excels = iGoodsService.getOneGoodsExcel(list.get(0).getName());
                EasyExcelFactory.write(response.getOutputStream(), GoodsExcel.class)
                        .registerWriteHandler(new SpinnerWriteHandler(list))
                        .sheet("模板").doWrite(excels);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @PostMapping("/uploadGoodsExcel")
    public BaseResponse uploadGoodsExcel(Integer merchantId, MultipartFile file) {
        try {
            Map<Integer, String> map = GoodsSpecExcelUtils.getPictureBytes(file.getInputStream(), fastFileStorageClient);
            for (Map.Entry<Integer, String> entry : map.entrySet()) {
                map.put(entry.getKey(), entry.getValue());
            }
            GoodsSpecExcelUtils.readExcel(file.getInputStream(), merchantId, iGoodsService, map, GoodsExcel.class);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("添加失败");
        }
        return BaseResponse.ok("添加成功");
    }

    @PostMapping("/scan")
    @ApiOperation("扫码录入商品")
    @ApiImplicitParam(value = "扫码商品", name = "goodsScanQuery", paramType = "body", dataTypeClass = GoodsScanQuery.class)
    public BaseResponse<Goods> scanAddGoods(@RequestBody GoodsScanQuery goodsScanQuery) {
        return iGoodsService.scanAddGoods(goodsScanQuery);
    }

    @GetMapping("/scan")
    @ApiOperation("扫码获取商品信息")
    @ApiImplicitParam(name = "code", value = "二维码", paramType = "query", dataTypeClass = String.class)
    public BaseResponse<Goods> getGoodsInfoByCode(String code) {
        return iGoodsService.getGoodsInfoByCode(code);
    }


    @PostMapping("/batch/add")
    @ApiOperation("批量添加")
    public BaseResponse<Boolean> batchAdd(@RequestBody List<Goods> goods) {
        return iGoodsService.batchAdd(goods);
    }

    @PostMapping("/goodIds")
    public List<Goods> getByGoodIds(@RequestBody List<Long> ids){
        return iGoodsService.getByGoodIds(ids);
    }

    @PostMapping("/updateFields")
    @ApiOperation("修改商品价格")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "类型id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "field1", value = "商品价格", paramType = "path", dataType = "String")
    })
    public BaseResponse<String> updateField1(Long id,String field1){
        return iGoodsService.updateField1(id,field1);
    }
}
