package cn.wise.sc.pay.business.cashier.controller;

import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.cashier.service.IPaasOrderService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 入驻商家订购单
 * @author: qh
 * @create: 2019-12-04 15:14
 **/
@RequestMapping("/order")
@RestController
@CrossOrigin
@Api(description = "订单")
public class OrderController {

    final
    IPaasOrderService iOrderService;
    final
    IMerchantService iMerchantService;
    final
    IOrdersService iOrdersService;

    public OrderController(IMerchantService iMerchantService,
                           IPaasOrderService iOrderService,
                           IOrdersService iOrdersService) {
        this.iMerchantService = iMerchantService;
        this.iOrderService = iOrderService;
        this.iOrdersService = iOrdersService;
    }

    @PostMapping("/new")
    @ApiOperation("新增一个订单")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderQuery")
    public BaseResponse newOrder(@RequestBody OrderDetailQuery orderQuery) {

        orderQuery.setStatus(OrderStatus.WAIT_PAY.name());
        return iOrderService.createOrder(orderQuery);
    }

    @GetMapping("/entry/{orderNo}")
    @ApiOperation("取挂单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse getEntryOrder(@PathVariable("orderNo") String orderNo) {
        return iOrderService.getEntryOrder(orderNo);
    }

    @PostMapping("/new/entry")
    @ApiOperation("挂单")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderQuery")
    public BaseResponse newEntryOrder(@RequestBody OrderDetailQuery orderQuery) {
        orderQuery.setStatus(OrderStatus.ENTRY_ORDER.name());
        return iOrderService.createOrder(orderQuery);
    }

    @PutMapping("/entry")
    @ApiOperation("修改挂单信息")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderQuery")
    public BaseResponse editEntryOrder(@RequestBody OrderDetailQuery orderQuery) {

        orderQuery.setStatus(OrderStatus.ENTRY_ORDER.name());
        return iOrderService.editOrder(orderQuery);
    }

    @GetMapping("/list/user/{userId}/{status}")
    @ApiOperation("查询用户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 确定认支付,2 待评论,3 订单完成,4 申请退款,5 已退款,6 取消订单 ,7 待售后 8 挂单 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse listUsers(@PathVariable("userId") int userId, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("user_id", userId);
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus.name());
        responseEntity = iOrderService.getOrders(condition);
        return responseEntity;
    }

    @GetMapping("/list/seller/{seller}/{status}")
    @ApiOperation("查询商户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 确定认支付,2 待评论,3 订单完成,4 申请退款,5 已退款,6 取消订单 ,7 待售后 8 挂单 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            ),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse listSeller(@PathVariable("seller") int seller, @PathVariable("status") int status,
                                   long start, long end) {

        return iOrderService.getSellerOrders(seller, status, start, end);
    }

    @GetMapping("/page/seller/{seller}/{status}")
    @ApiOperation("查询商户订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 确定认支付,2 待评论,3 订单完成,4 申请退款,5 已退款,6 取消订单 ,7 待售后 8 挂单 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            ),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse pageSeller(@PathVariable("seller") int seller, @PathVariable("status") int status,
                                   long start, long end, PageQuery pageQuery) {

        return iOrderService.getSellerOrdersPage(seller, status, start, end, pageQuery);
    }

    @PutMapping("/{orderNo}")
    @ApiOperation("支付订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse payOrder(@PathVariable("orderNo") String orderNo) {
        return iOrderService.payOrder(orderNo);
    }

    @GetMapping("/orderNo")
    @ApiOperation("获取订单号")
    public BaseResponse getOrderNo() {
        return BaseResponse.ok(iOrderService.getPreOrderNo());
    }

    @DeleteMapping("/{orderNo}")
    @ApiOperation("取消订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse delOrder(@PathVariable("orderNo") String orderNo) {
        return iOrderService.cancelOrder(orderNo);
    }


    @GetMapping("/merchant/orders")
    @ApiOperation(value = "根据商家关键字条件查询分页列表")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "pageNO", value = "当前页", paramType = "query", dataType = "int"),
                    @ApiImplicitParam(name = "PageSize", value = "每页显示数量", paramType = "query", dataType = "int"),
                    @ApiImplicitParam(name = "merchantName", value = "商家名称", paramType = "query", dataType = "String"),
                    @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "long"),
                    @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "long")
            }
    )
    public BaseResponse merchantOrderByKeys(Integer pageNO, Integer PageSize, String merchantName, Long startTime, Long endTime) {

        BaseResponse baseResponse = iOrderService.merchantOrderByKeys(pageNO, PageSize, merchantName, startTime, endTime);
        return baseResponse;
    }

    @GetMapping("/list/condition")
    @ApiOperation(value = "根据商品名称与交易时间筛选订单")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "pageNO", value = "当前页", paramType = "query", dataType = "int"),
                    @ApiImplicitParam(name = "PageSize", value = "每页显示数量", paramType = "query", dataType = "int"),
                    @ApiImplicitParam(name = "goodsName", value = "商品名称", paramType = "query", dataType = "String"),
                    @ApiImplicitParam(name = "startTime", value = "交易开始时间", paramType = "query", dataType = "long"),
                    @ApiImplicitParam(name = "endTime", value = "交易结束时间", paramType = "query", dataType = "long"),
                    @ApiImplicitParam(name = "orderNo", value = "订单编号", paramType = "query", dataType = "String")
            }
    )
    public BaseResponse orderDetailsByKeys(Integer pageNO, Integer PageSize, String goodsName, Long startTime, Long endTime, String orderNo) {
        BaseResponse baseResponse = iOrderService.orderDetailsByKeys(pageNO, PageSize, goodsName, startTime, endTime, orderNo);
        return baseResponse;
    }

    @GetMapping("/orderByNo")
    @ApiOperation(value = "根据订单号查询订单")
    @ApiImplicitParam(name = "orderNo", value = "订单编号", dataType = "String", paramType = "query")
    public BaseResponse orderByNo(String orderNo) {
        BaseResponse baseResponse = iOrderService.orderByNo(orderNo);
        return baseResponse;
    }

    @GetMapping("/v2/orderByNo")
    @ApiOperation(value = "根据订单号查询订单")
    @ApiImplicitParam(name = "orderNo", value = "订单编号", dataType = "String", paramType = "query")
    public BaseResponse orderByNoV2(String orderNo) {
        BaseResponse baseResponse = iOrderService.orderByNoV2(orderNo);
        return baseResponse;
    }
    /**
     * 统计  根据支付类型 统计金额
     */
    @GetMapping("/statistics/payMethod/Money")
    @ApiOperation("统计  根据支付类型 统计金额")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "strategy", value = "统计类型  策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "shopName", value = "商品名", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse statisticsMoneyByPatMethod(int strategy, String shopName, long start, long end) {

        BaseResponse baseResponse = iOrdersService.statisticsMoneyByPatMethod(strategy, shopName, start, end);
        BaseResponse response = iOrdersService.statisticsCountByPatMethod(strategy, shopName, start, end);

        if (baseResponse.getCode() == 200 && response.getCode() == 200) {
            List<Object> data = new ArrayList<>(2);
            data.add(baseResponse.getData());
            data.add(response.getData());
            return BaseResponse.ok(data);
        }
        return BaseResponse.error("统计失败!", null);
    }



    @PutMapping("/afterSuccess")
    @ApiOperation("成功后回调")
    public BaseResponse afterSuccess(String orderNo) {
        return iOrderService.afterSuccess(orderNo);
    }

    @PutMapping("/afterFail")
    @ApiOperation("成功后回调")
    public BaseResponse afterFail(String orderNo) {
        return iOrderService.afterFail(orderNo);
    }

    @GetMapping("pageApp/user/{userId}/{status}")
    @ApiOperation("获取用户订单分页")
    public BaseResponse getOrdersPage(String userId,Integer status,PageQuery pageQuery){
        Map<String ,Object> condition = new HashMap<>();
        condition.put("user_id",userId);

        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status",orderStatus);
        return iOrderService.getOrdersPage(condition,pageQuery);
    }
}
