package cn.wise.sc.pay.business.cashier.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Map;

/**
 * @description: 订单
 * @author: qh
 * @create: 2019-12-04 17:10
 **/
public interface IPaasOrderService {

    /**
     * 支付订单
     *
     * @param orderNo
     * @return BaseResponse
     */
    BaseResponse payOrder(String orderNo);

    /**
     * 创建订单
     *
     * @param orderDetail 订单描述
     * @return BaseResponse
     */
    BaseResponse createOrder(OrderDetailQuery orderDetail);

    /**
     * 改变订单状态
     *
     * @param orderNo     订单号
     * @param orderStatus 要改变的状态
     * @param condition   关键条件 <field,value> 如:卖家id
     * @return BaseResponse
     */
    BaseResponse changeOrderStatus(String orderNo, OrderStatus orderStatus, Map<String, Object> condition);

    /**
     * 根据条件查询
     *
     * @param condition 条件 map集合  <field,value>
     * @return BaseResponse
     */
    BaseResponse getOrders(Map<String, Object> condition);

    /**
     * 根据key删除
     *
     * @param orderNo   订单号儿
     * @param condition 删除条件
     * @return BaseResponse
     */
    BaseResponse delOrders(String orderNo, Map<String, Object> condition);

    /**
     * 获取Orderid
     *
     * @return BaseResponse
     */
    BaseResponse getPreOrderNo();

    /**
     * 编辑订单
     *
     * @param orderQuery 订单
     * @return BaseResponse
     */
    BaseResponse editOrder(OrderDetailQuery orderQuery);

    /**
     * 取消订单
     *
     * @param orderNo 订单号
     * @return BaseResponse
     */
    BaseResponse cancelOrder(String orderNo);

    /**
     * 取单 根据订单
     * @param orderNo
     * @return BaseResponse
     */
    BaseResponse getEntryOrder(String orderNo);

    /**
     * 根据商家
     * @return BaseResponse
     */
    BaseResponse merchantOrderByKeys(Integer pageNO,Integer PageSize, String merchantName, Long startTime,Long endTime);

    /**
     * 根据关键字查询订单详情
     * @param pageNO    订单号
     * @param PageSize 分页大小
     * @param goodsName 商品名
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param orderNo 订单号
     * @return BaseResponse
     */
    BaseResponse orderDetailsByKeys(Integer pageNO,Integer PageSize,String goodsName,Long startTime,Long endTime,String orderNo);

    /**
     * 根据订单号查询订单
     * @param orderNo 订单号
     * @return BaseResponse
     */
    BaseResponse orderByNo(String orderNo);

    /**
     * 根据订单号查询订单
     * @param orderNo 订单号
     * @return BaseResponse
     */
    BaseResponse orderByNoV2(String orderNo);
    /**
     * 查询商户订单
     * @param seller 商户订单
     * @param status 订单状态
     * @param start 开始时间
     * @param end 结束时间
     * @return BaseResponse
     */
    BaseResponse getSellerOrders(int seller, int status, long start, long end);

    /**
     * 查询商户订单
     * @param seller 商户订单
     * @param status 订单状态
     * @param start 开始时间
     * @param end 结束时间
     * @return BaseResponse
     */
    BaseResponse getSellerOrdersPage(int seller, int status, long start, long end, PageQuery pageQuery);

    BaseResponse getOrdersPage(Map<String, Object> condition, PageQuery pageQuery);

    BaseResponse afterSuccess(String orderNo);

    BaseResponse afterFail(String orderNo);
}
