package cn.wise.sc.pay.api.eshopping.service;

import cn.wise.sc.pay.api.eshopping.service.hystrix.LogisticsTemplateServiceImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.query.LogisticsTemplateQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

/**
 * @description: 物流模板服务接口
 * @author: qh
 * @create: 2020-04-02 13:34
 **/
@FeignClient(value = "saas-v1-eshopping", path = "/logisticsTemplate", fallback = LogisticsTemplateServiceImpl.class)
public interface ILogisticsTemplateService {

    /**
     * 获取物流信息
     *
     * @param logisticsNo 物流号
     * @param com         快递平台
     * @return String
     */
    @GetMapping("/logisticsInfo")
    BaseResponse getLogisticsInfo(@RequestParam("logisticsNo") String logisticsNo,
                                  @RequestParam("com") String com);

    /**
     * 新增物流模板记录
     *
     * @param logisticsTemplateQuery
     * @return
     */
    @PostMapping
    BaseResponse add(@RequestBody LogisticsTemplateQuery logisticsTemplateQuery);

    /**
     * 根据id修改物流模板信息
     *
     * @param logisticsTemplateQuery
     * @return
     */
    @PutMapping
    BaseResponse edit(@RequestBody LogisticsTemplateQuery logisticsTemplateQuery);

    /**
     * 根据id获取物流模板信息
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    BaseResponse getInfoById(@NotNull(message = "id值不能为空") @PathVariable("id") Integer id);

    /**
     * 分页查询物流模板记录
     *
     * @param
     * @return
     */
    @GetMapping("/page")
    BaseResponse pageInfo(@RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize);

    /**
     * 根据id删除物流模板信息
     *
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    BaseResponse deleteById(@NotNull(message = "id值不能为空") @PathVariable("id") Integer id);


    /**
     * 根据id删除子物流模板信息
     *
     * @param id
     * @return
     */
    @DeleteMapping("/child/{id}")
    BaseResponse deleteChildById(@NotNull(message = "id值不能为空") @PathVariable("id") Integer id);


}