package cn.wise.sc.pay.business.eshopping.controller;

import cn.wise.sc.pay.business.eshopping.configure.FastDFSUtils;
import cn.wise.sc.pay.business.eshopping.service.*;
import cn.wise.sc.pay.business.eshopping.utils.GoodsSpecExcelUtils;
import cn.wise.sc.pay.business.eshopping.utils.SpinnerWriteHandler;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopGoods;
import cn.wise.sc.pay.domain.eshopping.model.bean.LogisticsParent;
import cn.wise.sc.pay.domain.eshopping.model.bean.LogisticsTemplate;
import cn.wise.sc.pay.domain.eshopping.model.query.CalCulate;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsQuery;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsSpec;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsSpecification;
import cn.wise.sc.pay.domain.eshopping.model.vo.GoodsSpecExcel;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.*;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(tags = "商品")
@RequestMapping("/eshop/goods")
@CrossOrigin
public class EshopGoodsController {

    final
    IEshopGoodsService eshopGoodsService;

    final
    ILogisticsTemplateService templateService;

    final
    ILogisticsParentService parentService;

    final
    IEshopBrandService brandService;

    final
    IEshopTypeService typeService;

    @Autowired
    private FastFileStorageClient fastFileStorageClient;

    @Autowired
    FastDFSUtils fastDFSUtils;

    public EshopGoodsController(IEshopGoodsService eshopGoodsService, ILogisticsTemplateService templateService,
                                ILogisticsParentService parentService, IEshopBrandService brandService,
                                IEshopTypeService typeService) {
        this.eshopGoodsService = eshopGoodsService;
        this.templateService = templateService;
        this.parentService = parentService;
        this.brandService = brandService;
        this.typeService = typeService;
    }

    @PostMapping
    public BaseResponse addGoods(@RequestBody @Valid EshopGoodsQuery eshopGoodsQuery) {
        return eshopGoodsService.addGoods(eshopGoodsQuery);
    }

    @PutMapping
    public BaseResponse updateGoods(@RequestBody @Valid EshopGoodsQuery eshopGoodsQuery) {
        if (eshopGoodsQuery.getGoodsId() == null) {
            return BaseResponse.error("id为必填", EshopGoods.class);
        }
        return eshopGoodsService.updateGoods(eshopGoodsQuery);
    }

    @GetMapping("goodsPage")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品名称", name = "name", paramType = "query", dataType = "String"),
            @ApiImplicitParam(value = "商品状态", name = "marketEnable", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(value = "页码", name = "pageNo", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "每页条数", name = "pageSize", paramType = "query", dataType = "Integer", required = true)
    })
    public BaseResponse goodsPage(@RequestParam(value = "name", required = false) String name,
                                  @RequestParam(value = "marketEnable", required = false) Integer marketEnable,
                                  @RequestParam(value = "pageNo") @DecimalMin("0") Integer pageNo,
                                  @RequestParam(value = "pageSize") Integer pageSize) {
        return eshopGoodsService.goodsPage(name, marketEnable, pageNo, pageSize);
    }

    @DeleteMapping
    @ApiImplicitParam(value = "商品id", name = "id", paramType = "query", dataType = "Integer", required = true)
    public BaseResponse deleteGoods(@RequestParam(value = "id") Integer id) {
        return eshopGoodsService.deleteGoods(id);
    }

    @PutMapping("updateStatus")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品id", name = "goodsId", paramType = "query", dataType = "String", required = true),
            @ApiImplicitParam(value = "上下架（0下架，1上架）", name = "marketEnable", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "下架原因", name = "underMessage", paramType = "query", dataType = "String")
    })
    public BaseResponse updateGoodsStatus(@RequestParam("goodsId") Integer goodsId,
                                          @RequestParam("marketEnable") Integer marketEnable,
                                          @RequestParam(value = "underMessage", required = false) String underMessage) {
        if (goodsId == null || marketEnable == null) {
            return BaseResponse.error("参数错误", EshopGoods.class);
        }
        return eshopGoodsService.updateStatus(goodsId, marketEnable, underMessage);
    }

    @GetMapping("conditionGoodsPage")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品分类", name = "categoryId", paramType = "query", dataType = "int"),
            @ApiImplicitParam(value = "商品价格排序，0升序，1降序", name = "price", paramType = "query", dataType = "int"),
            @ApiImplicitParam(value = "商品发布时间,0顺序，1倒序", name = "createTime", paramType = "query", dataType = "int"),
            @ApiImplicitParam(value = "页码", name = "pageNo", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "每页条数", name = "pageSize", paramType = "query", dataType = "Integer", required = true)
    })
    public BaseResponse conditionGoodsPage(
            @RequestParam(value = "categoryId", required = false) Integer categoryId,
            @RequestParam(value = "price", required = false) Integer price,
            @RequestParam(value = "createTime", required = false) Integer createTime,
            @RequestParam(value = "multiple", required = false) Integer multiple,
            @RequestParam(value = "pageNo") @DecimalMin("0") Integer pageNo,
            @RequestParam(value = "pageSize") Integer pageSize) {
        return eshopGoodsService.appGoodsPage(categoryId, price, createTime, multiple, pageNo, pageSize);
    }

    @GetMapping("/detail/{goodsId}")
    public BaseResponse goodsDetail(@Min(value = 0) @PathVariable("goodsId") Integer goodsId) {
        return eshopGoodsService.getGoodsDetail(goodsId);
    }

    @PostMapping("calculate")
    public BaseResponse calculatePrice(@RequestBody CalCulate calCulate) {
        List<EshopGoodsSpec> goodsList = calCulate.getGoodsList();
        //数据库获取商品及商品规格信息
        if (goodsList == null || goodsList.size() == 0) {
            return BaseResponse.error("当前网络异常，请稍后重试", EshopGoods.class.getName());
        }
        List<EshopGoodsSpecification> eshopGoods = eshopGoodsService.eshopGoodsInfo(goodsList);
        Integer addressId = calCulate.getReceiveAddress();
        //1.提取由买家承担的运费模板id集合
        Set<Integer> templateId = new HashSet<>();
        eshopGoods.stream().forEach(arg -> {
            if (arg.getGoodsTransfeeCharge() == 0) {
                templateId.add(arg.getTemplateId());
            }
        });

        Map<Long, Double> result = new HashMap<>();
        List<Long> noRegionIds = new ArrayList<>();
        //2.如果有的商品由买家承担运费，计算运费结果
        if (templateId.size() > 0) {
            List<LogisticsTemplate> templateList = templateService.getReceiveList(templateId);
            //首 续标志
            Map<Integer, Integer> contine = new HashMap<>();
            templateList.stream().forEach(arg -> {
                contine.put(arg.getId(), 0);
            });

            Map<Integer, LogisticsParent> map = parentService.getParents(templateId);
            for (EshopGoodsSpecification goods : eshopGoods) {
                //确定商品数量
                Long specId = goods.getId();
                double specWeight = goods.getWeight();
                Integer goodsId = goods.getGoodsId();
                int number = 1;
                for (EshopGoodsSpec egs : goodsList) {
                    if (egs.getGoodsId().intValue() == goodsId.intValue() && egs.getSpecId().intValue() == specId.intValue()) {
                        number = egs.getNumber();
                    }
                }
                //判断每件商品是否在配送范围
                boolean templateFlag = false;
                for (LogisticsTemplate template : templateList) {
                    Integer parentId = template.getParentId();
                    Integer id = template.getId();
                    if (goods.getTemplateId().longValue() == parentId.longValue()) {
                        JSONArray regionIds = JSON.parseArray(template.getRegionId());
                        if (regionIds.contains(addressId)) {
                            templateFlag = true;
                            LogisticsParent lp = map.get(parentId);
                            if (lp.getFeeWay() == 0) {
                                //按件计费
                                if (contine.get(id) == 0) {
                                    //首件
                                    result.put(specId, template.getFee() + template.getContinueFee() * (number - 1));
                                    contine.put(id, 1);
                                } else {
                                    result.put(specId, template.getContinueFee() * number);
                                }
                            } else {
                                //按重量计费
                                if (contine.get(id) == 0) {
                                    //首重
                                    if (specWeight <= 1) {
                                        if (number > 1) {
                                            double pri = template.getFee();
                                            pri = pri + template.getContinueFee() * specWeight * (number - 1);
                                            result.put(specId, pri);
                                        } else {
                                            result.put(specId, template.getFee());
                                        }
                                    } else {
                                        double pri = template.getFee();
                                        pri = pri + (specWeight - 1) * template.getContinueFee()
                                                + template.getContinueFee() * (number - 1) * specWeight;
                                        result.put(specId, pri);
                                    }
                                    contine.put(id, 1);
                                } else {
                                    double pri = template.getContinueFee() * specWeight * number;
                                    result.put(specId, pri);
                                }
                            }
                        }
                    }
                }
                if (!templateFlag) {
                    noRegionIds.add(specId);
                }

            }
        } else {
            for (EshopGoodsSpecification goods : eshopGoods) {
                Long id = goods.getId();
                result.put(id, new Double(0));
            }
        }
        //计算总价
        double sum = 0.0;
        for (EshopGoodsSpecification goods : eshopGoods) {
            Double price = result.get(goods.getId());
            if (price != null) {
                sum += price;
            }
        }
        Map<String, Object> map = new HashMap<>();
        if (noRegionIds.size() > 0) {
            map.put("list", noRegionIds);
            map.put("freight", -1);
        } else {
            map.put("list", noRegionIds);
            map.put("freight", sum);
        }
        return BaseResponse.ok(map);
    }

    @GetMapping("/downloadGoodsExcel")
    public void downloadGoodsExcel(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("商品上新", "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
        List<GoodsSpecExcel> excels = eshopGoodsService.getOneGoodsSpec();
        EasyExcelFactory.write(response.getOutputStream(), GoodsSpecExcel.class)
                .registerWriteHandler(new SpinnerWriteHandler(brandService, typeService, parentService))
                .sheet("模板").doWrite(excels);
    }

    @PostMapping("/uploadGoodsExcel")
    public BaseResponse uploadGoodsExcel(MultipartFile file) {
        try {
            Map<Integer, String> map = GoodsSpecExcelUtils.getPictureBytes(file.getInputStream(), fastFileStorageClient);
            for (Map.Entry<Integer, String> entry : map.entrySet()) {
                map.put(entry.getKey(), entry.getValue());
            }


            GoodsSpecExcelUtils.readExcel(file.getInputStream(), eshopGoodsService, map, GoodsSpecExcel.class);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InvalidFormatException e) {
            e.printStackTrace();
        }
        return BaseResponse.ok("添加成功");
    }

    //  tianjia  商品评论数量
    @PutMapping("/incrCommentNum/{id}")
    public BaseResponse incrCommentNumById(@Min(value = 0) @PathVariable("id") Integer id) {
        BaseResponse response = eshopGoodsService.incrCommentNumById(id);
        return response;
    }


    public static String verificationStudentExcelHeadLine(Workbook wb, String[] columnName) throws Exception {
        String result = null;

        try {
            Sheet sheet = wb.getSheetAt(0);
            Row row = sheet.getRow(0);
            if (row != null && row.getLastCellNum() >= columnName.length) {
                int lastCellNum = row.getLastCellNum();
                for (int idx = 0; idx < lastCellNum; idx++) {
                    String value = getCellValue(row.getCell(idx)).toString();
                    if (idx < columnName.length) {
                        if (StringUtils.isBlank(value) || !columnName[idx].equals(value)) {
                            result = "标题行第" + (idx + 1) + "列名称错误！";
                            throw new Exception();
                        }
                    } else {
                        if (idx == columnName.length) {
                            if (StringUtils.isBlank(value)) {
                                result = "标题与导出的表格表头不一致";
                                throw new Exception();
                            }
                        }
                    }
                }
            } else {
                result = "上传文件首行不能为空或与导出学员的表格表头不一致！";
            }
        } catch (Exception ex) {

        }
        return result;
    }

    public static Object getCellValue(Cell cell) {
        if (CellType.STRING.getCode() == cell.getCellType()) {
            return cell.getRichStringCellValue().getString().trim();
        }
        if (CellType.NUMERIC.getCode() == cell.getCellType()) {
            if (DateUtil.isCellDateFormatted(cell)) {
                return cell.getDateCellValue();
            } else {
                cell.setCellType(CellType.STRING);
                return new BigDecimal(cell.getStringCellValue());
            }
        }
        if (CellType.BOOLEAN.getCode() == cell.getCellType())
            return cell.getBooleanCellValue();
        if (CellType.BLANK.getCode() == cell.getCellType())
            return "";
        if (CellType.ERROR.getCode() == cell.getCellType())
            return cell.getErrorCellValue();
        return null;
    }

}
