package cn.wise.sc.pay.business.eshopping.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopStore;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.concurrent.ConcurrentHashMap;

/**
 * <p>
 * 库存Service
 * </p>
 *
 * @author qh
 * @since 2020-03-24
 */
public interface IEshopStoreService extends IService<EshopStore> {

    /**
     * 初始化库存
     *
     * @param goodsId 商品id
     * @param store   库存
     * @param skuId   规格
     * @return BaseResponse
     */
    BaseResponse<EshopStore> newStore(Long goodsId, Integer store, Long skuId);


    /**
     * 根据商品id 添加库存
     *
     * @param goodId   商品id
     * @param quantity 数量
     * @param skuId    规格
     * @return BaseResponse
     */
    BaseResponse<EshopStore> addStore(Long goodId, Integer quantity, Long skuId);

    /**
     * 根据商品id  减少库存
     *
     * @param goodId   商品id
     * @param quantity 数量
     * @param skuId    规格id
     * @return BaseResponse
     */
    BaseResponse<EshopStore> decreaseStore(Long goodId, Integer quantity, Long skuId);

    /**
     * 库存同步数据库
     *
     * @param goodId 商品
     * @param skuId  规格
     * @return BaseResponse
     */
    BaseResponse<EshopStore> saveDB(Long goodId, Long skuId);

    /**
     * 根据商品id集合查找库存
     *
     * @param goodId 商品id
     * @return Map<long, int>
     */
    BaseResponse<ConcurrentHashMap<String, Long>> storesByGoodId(Long goodId);

    /**
     * 根据商品id获取库存
     *
     * @param goodId 商品id
     * @param skuId  skuId
     * @return EshopStore
     */
    EshopStore storeByGoodIdAndSkuId(Long goodId, Long skuId);
}
