package cn.wise.sc.pay.business.eshopping.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.eshopping.model.bean.LogisticsTemplate;
import cn.wise.sc.pay.domain.eshopping.model.vo.LogisticsDetailVo;
import cn.wise.sc.pay.domain.eshopping.query.LogisticsTemplateQuery;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 物流管理模板业务接口层
 */
public interface ILogisticsTemplateService extends IService<LogisticsTemplate> {
    /**
     * 新增物流模板记录
     *
     * @param logisticsTemplateQuery
     * @return
     */
    BaseResponse add(LogisticsTemplateQuery logisticsTemplateQuery);

    /**
     * 修改物流模板信息
     *
     * @param logisticsTemplateQuery
     * @return
     */
    BaseResponse edit(LogisticsTemplateQuery logisticsTemplateQuery);

    /**
     * 根据id获取物流模板信息
     *
     * @param id
     * @return
     */
    BaseResponse<LogisticsDetailVo> getInfoById(Integer id);

    /**
     * 分页查询物流模板记录
     *
     * @param pageQuery
     * @return
     */
    BaseResponse pageInfo(PageQuery pageQuery);

    /**
     * 根据id删除物流模板信息
     *
     * @param id
     * @return
     */
    BaseResponse deleteById(Integer id);

    /**
     * 根据id删除子物流模板信息
     * @param id
     * @return
     */
    BaseResponse deleteChildById(Integer id);

    /**
     * 获取物流信息
     *
     * @param logisticsNo 物流号
     * @param com         物流平台
     * @return String
     */
    BaseResponse<String> getLogisticsInfo(String logisticsNo, String com);

    /**
     * 获取所需运费模板列表
     * @param templateId
     * @return
     */
    List<LogisticsTemplate> getReceiveList(Set<Integer> templateId);
}
