package cn.wise.sc.pay.business.eshopping.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.eshopping.mapper.LogisticsTemplateMapper;
import cn.wise.sc.pay.business.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.business.eshopping.service.ILogisticsParentService;
import cn.wise.sc.pay.business.eshopping.service.ILogisticsTemplateService;
import cn.wise.sc.pay.business.eshopping.utils.SynQueryUtils;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.eshopping.model.bean.LogisticsParent;
import cn.wise.sc.pay.domain.eshopping.model.bean.LogisticsTemplate;
import cn.wise.sc.pay.domain.eshopping.model.vo.LogisticsDetailVo;
import cn.wise.sc.pay.domain.eshopping.query.LogisticsTemplateQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 物流管理模板业务实现类
 */
@Service
@Transactional
public class LogisticsTemplateServiceImpl extends ServiceImpl<LogisticsTemplateMapper, LogisticsTemplate> implements ILogisticsTemplateService {
    @Autowired
    ILogisticsParentService logisticsParentService;
    @Autowired
    IEshopGoodsService iEshopGoodsService;

    @Override
    /**
     * 新增物流模板记录
     */
    public BaseResponse add(LogisticsTemplateQuery logisticsTemplateQuery) {
        //父物流模板
        LogisticsParent logisticsParent = LogisticsTemplateQuery.query2Entity(logisticsTemplateQuery);
        logisticsParent.setCreateTime(System.currentTimeMillis());
        boolean save = logisticsParentService.save(logisticsParent);
        //子物流模板
        List<LogisticsTemplate> logisticsTemplates = LogisticsTemplateQuery.query2List(logisticsTemplateQuery);
        logisticsTemplates.forEach(lt -> {
            lt.setCreateTime(System.currentTimeMillis());
            lt.setParentId(logisticsParent.getId());
        });
        boolean saveBatch = this.saveBatch(logisticsTemplates);


        LogisticsDetailVo logisticsDetailVo = LogisticsDetailVo.builder()
                .logisticsTemplates(logisticsTemplates)
                .build();
        BeanUtils.copyProperties(logisticsParent, logisticsDetailVo);
        return save && saveBatch ? BaseResponse.ok(logisticsDetailVo) : BaseResponse.error("添加失败", LogisticsTemplateServiceImpl.class.getName());
    }

    @Override
    /**
     * 修改物流模板信息
     */
    public BaseResponse edit(LogisticsTemplateQuery logisticsTemplateQuery) {
        if (logisticsTemplateQuery.getId() == null) {
            return BaseResponse.error("请传入模板主键id值", LogisticsTemplateServiceImpl.class.getName());
        }
        LogisticsParent logisticsParent = LogisticsTemplateQuery.query2Entity(logisticsTemplateQuery);
        logisticsParent.setUpdateTime(System.currentTimeMillis());
        boolean update = logisticsParentService.updateById(logisticsParent);
        //
        List<LogisticsTemplate> logisticsTemplates = LogisticsTemplateQuery.query2List(logisticsTemplateQuery);
        //需要新增的list
        List<LogisticsTemplate> saveList = new ArrayList<>();
        //需要修改的list
        List<LogisticsTemplate> updateList = new ArrayList<>();
        //过滤参数信息
        for (LogisticsTemplate logisticsTemplate : logisticsTemplates) {
            if (logisticsTemplate.getId() == null) {
                logisticsTemplate.setParentId(logisticsParent.getId());
                logisticsTemplate.setCreateTime(System.currentTimeMillis());
                logisticsTemplate.setUpdateTime(System.currentTimeMillis());
                saveList.add(logisticsTemplate);
            } else {
                logisticsTemplate.setUpdateTime(System.currentTimeMillis());
                updateList.add(logisticsTemplate);
            }
        }
        boolean batchById = true;
        if (!CollectionUtils.isEmpty(updateList)) {
            boolean updates = this.updateBatchById(updateList);
            batchById = batchById && updates;
        }
        if (!CollectionUtils.isEmpty(saveList)) {
            boolean saveBatch = this.saveBatch(saveList);
            batchById = batchById && saveBatch;
        }
        logisticsTemplates.clear();
        logisticsTemplates.addAll(updateList);
        logisticsTemplates.addAll(saveList);
        //值对象
        LogisticsDetailVo logisticsDetailVo = LogisticsDetailVo.builder()
                .logisticsTemplates(LogisticsDetailVo.logisticsTemplates(logisticsTemplates))
                .build();
        BeanUtils.copyProperties(logisticsParent, logisticsDetailVo);
        return update && batchById ? BaseResponse.ok(logisticsDetailVo) : BaseResponse.error("更新失败", LogisticsTemplateServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据id获取物流模板信息
     */
    public BaseResponse<LogisticsDetailVo> getInfoById(Integer id) {
        if (id == null || id == 0) {
            return BaseResponse.error("请传入主键id值", LogisticsTemplateServiceImpl.class.getName());
        }
        //父
        LogisticsParent logisticsParent = logisticsParentService.getById(id);
        if (logisticsParent == null) {
            return BaseResponse.ok((LogisticsDetailVo) null, LogisticsTemplateServiceImpl.class);
        }
        //条件查询
        QueryWrapper<LogisticsTemplate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", logisticsParent.getId());
        //子
        List<LogisticsTemplate> logisticsTemplates = this.list(queryWrapper);
//值对象
        LogisticsDetailVo logisticsDetailVo = LogisticsDetailVo.builder()
                .logisticsTemplates(LogisticsDetailVo.logisticsTemplates(logisticsTemplates))
                .build();
        BeanUtils.copyProperties(logisticsParent, logisticsDetailVo);
        return BaseResponse.ok(logisticsDetailVo);
    }

    @Override
    /**
     * 分页查询物流模板记录
     */
    public BaseResponse pageInfo(PageQuery pageQuery) {
        //分页
        Page<LogisticsParent> page = new Page<>();
        page.setSize(pageQuery.getPageSize());
        page.setCurrent(pageQuery.getPageNo());
        //条件
        QueryWrapper<LogisticsParent> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("create_time");
        //父模板数据
        IPage<LogisticsParent> pageInfo = logisticsParentService.page(page, queryWrapper);
        List<LogisticsParent> records = pageInfo.getRecords();
        //子模板条件
        Set<Integer> parentIds = records.stream().map(re -> re.getId()).collect(Collectors.toSet());
        QueryWrapper<LogisticsTemplate> templateQueryWrapper = new QueryWrapper<>();
        templateQueryWrapper.in("parent_id", parentIds);
        //子模板数据
        List<LogisticsTemplate> logisticsTemplates = this.list(templateQueryWrapper);
        //重组数据
        ArrayList<LogisticsDetailVo> logisticsDetailVos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(records)) {
            records.stream().forEach(re -> {
                //
                LogisticsDetailVo logisticsDetailVo = new LogisticsDetailVo();
                BeanUtils.copyProperties(re, logisticsDetailVo);
                //子模板数据
                ArrayList<LogisticsTemplate> templateArrayList = new ArrayList<>();

                if (!CollectionUtils.isEmpty(logisticsTemplates)) {
                    logisticsTemplates.stream().forEach(lt -> {
                        if (re.getId().equals(lt.getParentId())) {
                            templateArrayList.add(lt);
                        }
                    });
                    logisticsDetailVo.setLogisticsTemplates(LogisticsDetailVo.logisticsTemplates(templateArrayList));
                }
                logisticsDetailVos.add(logisticsDetailVo);
            });
        }
        //返回分页数据
        IPage<LogisticsDetailVo> pageVo = new Page<>();
        pageVo.setRecords(logisticsDetailVos);
        pageVo.setCurrent(pageInfo.getCurrent());
        pageVo.setPages(pageInfo.getPages());
        pageVo.setSize(pageInfo.getSize());
        pageVo.setTotal(pageInfo.getTotal());
        return BaseResponse.ok(pageVo);
    }

    @Override
    /**
     * 根据id删除物流模板信息
     */
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteById(Integer id) {
        if (id == null || id == 0) {
            return BaseResponse.error("请传入主键id值", LogisticsTemplateServiceImpl.class.getName());
        }
        QueryWrapper<LogisticsTemplate> templateQueryWrapper = new QueryWrapper<>();
        templateQueryWrapper.eq("parent_id", id);

        //判断模板是否已被商品使用
        QueryWrapper qw = new QueryWrapper();
        qw.eq("template_id", id);
        int count = iEshopGoodsService.count(qw);

        if (count > 0) {
            return BaseResponse.error("当前模板正在使用中!", this.getClass().getName());
        }

        this.remove(templateQueryWrapper);
        boolean removeParent = logisticsParentService.removeById(id);
        return removeParent ? BaseResponse.ok() : BaseResponse.error("删除失败", LogisticsTemplateServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据id删除子物流模板信息
     * @param id
     * @return
     */
    public BaseResponse deleteChildById(Integer id) {
        if (id == null || id == 0) {
            return BaseResponse.error("请传入主键id值", LogisticsTemplateServiceImpl.class.getName());
        }

        //找到父级id,判断是否被使用中
        LogisticsTemplate logisticsTemplate = this.getById(id);
        ObjectUtil.isNotNull(logisticsTemplate);
        Integer parentId = logisticsTemplate.getParentId();
        QueryWrapper qw = new QueryWrapper();
        qw.eq("parent_id", parentId);
        int count = iEshopGoodsService.count(qw);
        if (count > 0) {
            return BaseResponse.error("当前模板正在使用中!", this.getClass().getName());
        }
        boolean remove = this.removeById(id);
        return remove ? BaseResponse.ok() : BaseResponse.error("删除失败", LogisticsTemplateServiceImpl.class.getName());
    }

    @Override
    public BaseResponse<String> getLogisticsInfo(String logisticsNo, String com) {
        SynQueryUtils synQueryUtils = new SynQueryUtils();
        String result = synQueryUtils.synQueryData("yuantong", logisticsNo, "", "", "", 0);

        return StrUtil.isEmpty(result) ? BaseResponse.error("查询失败!", LogisticsTemplate.class.getName()) :
                BaseResponse.ok(result);
    }

    @Override
    public List<LogisticsTemplate> getReceiveList(Set<Integer> templateId) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.in("parent_id", templateId);
        List<LogisticsTemplate> templateList = this.list(wrapper);
        return templateList;
    }


}
