package cn.wise.sc.pay.domain.eshopping.model.query;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel
public class EshopGoodsQuery implements Serializable {

    private static final long serialVersionUID = 4065072772447018088L;
    /**
     * 商品主键标识
     */
    @ApiModelProperty(value = "商品id", name = "goodsId", required = false)
    private Integer goodsId;
    /**
     * 商品名称
     */
    @NotBlank(message = "商品名称不能为空")
    @ApiModelProperty(value = "商品名称", name = "goodsName", required = true)
    private String goodsName;
    /**
     * 商品编号
     */
    @ApiModelProperty(value = "商品编号", name = "sn", required = false)
    private String sn;
    /**
     * 品牌id
     */
    @ApiModelProperty(value = "商品品牌id", name = "brandId", required = false)
    private Integer brandId;
    /**
     * 分类id
     */
    @NotNull(message = "商品分类不能为空")
    @ApiModelProperty(value = "商品分类id", name = "categoryId", required = true)
    private Integer categoryId;
    /**
     * 重量
     */
    @NotNull(message = "商品重量不能为空")
    @ApiModelProperty(value = "重量", name = "weight", required = true)
    private Double weight;
    /**
     * 详情
     */
    @ApiModelProperty(value = "描述", name = "intro", required = false)
    private String intro;
    /**
     * 商品价格
     */
    @NotNull(message = "商品价格不能为空")
    @ApiModelProperty(value = "价格", name = "price", required = true)
    private Double price;
    /**
     * 成本价格
     */
    @NotNull(message = "商品成本不能为空")
    @ApiModelProperty(value = "成本", name = "cost", required = true)
    private Double cost;
    /**
     * 市场价格
     */
    @ApiModelProperty(value = "市场价格", name = "mktprice", required = false)
    private Double mktprice;
    /**
     * 是否有规格0没有 1有
     */
    @NotNull(message = "商品规格标识不能为空")
    @ApiModelProperty(value = "是否有规格（0没有 1 有）", name = "haveSpec", required = true)
    private Integer haveSpec;
    /**
     * 库存
     */
    @NotNull(message = "商品库存不能为空")
    @ApiModelProperty(value = "库存", name = "quantity", required = true)
    private Integer quantity;
    /**
     * 可用库存
     */
    @ApiModelProperty(value = "可用库存", name = "enableQuantity", required = false)
    private Integer enableQuantity;
    /**
     * 原图路径
     */
    @NotNull(message = "商品图片不能为空")
    @ApiModelProperty(value = "商品图片", name = "original", required = true)
    private String original;
    /**
     * 运费模板id
     */
    @ApiModelProperty(value = "运费模板id", name = "templateId", required = true)
    private Integer templateId;
    /**
     * 谁承担运费0：买家承担，1：卖家承担
     */
    @NotNull(message = "需有人承担运费")
    @ApiModelProperty(value = "承担运费（0买家 1卖家）", name = "goodsTransfeeCharge", required = true)
    private Integer goodsTransfeeCharge;

    /**
     * 商品描述或 简介
     */
    @ApiModelProperty(value = "商品描述或 简介")
    private String describ;
    /**
     * 商品规格
     */
    @ApiModelProperty(value = "商品规格", name = "spcQuery", required = false)
    private List<SpecificationQuery> spcQuery;
}
