package cn.wise.sc.pay.domain.eshopping.query;

import cn.wise.sc.pay.domain.eshopping.model.bean.LogisticsTemplate;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.beans.BeanUtils;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel
/**
 * 物流模板子参数
 */
public class LogisticsChildQuery implements Serializable {
    /**
     * 主键id
     */
    @ApiModelProperty(value = "主键id")
    private Integer id;

    /**
     * 物流模板父id
     */
    @ApiModelProperty(value = "主键id")
    private Integer parentId;

    /**
     * 可配送区域
     */
    @NotEmpty(message = "可配送区域不能为空")
    @ApiModelProperty(value = "可配送区域")
    private List<String> regions;

    /**
     * 可配送区域
     */
    @NotEmpty(message = "可配送区域id不能为空")
    @ApiModelProperty(value = "可配送区域id集合")
    private List<Integer> regionIds;

    /**
     * 首重或首件
     */
    @NotNull(message = "首件不能为空")
    @ApiModelProperty(value = "首件")
    private Integer firstNum;

    /**
     * 续件或续重
     */
    @NotNull(message = "续件不能为空")
    @ApiModelProperty(value = "续件")
    private Integer continueNum;


    /**
     * 首重或首件
     */
    @NotNull(message = "计费方式不能为空")
    @ApiModelProperty(value = "首重")
    private Double firstWeight;
    /**
     * 续件或续重
     */
    @NotNull(message = "续重不能为空")
    @ApiModelProperty(value = "续重")
    private Double continueWeight;
    /**
     * 运费
     */
    @NotNull(message = "运费不能为空")
    @ApiModelProperty(value = "运费")
    private Double fee;
    /**
     * 续费
     */
    @NotNull(message = "续费不能为空")
    @ApiModelProperty(value = "续费")
    private Double continueFee;

    public static LogisticsTemplate query2Entity(LogisticsChildQuery logisticsChildQuery) {
        LogisticsTemplate logisticsTemplate = new LogisticsTemplate();
        BeanUtils.copyProperties(logisticsChildQuery, logisticsTemplate);
        logisticsTemplate.setRegion(JSON.toJSONString(logisticsChildQuery.getRegions()));
        logisticsTemplate.setRegionId(JSON.toJSONString(logisticsChildQuery.getRegionIds()));
        return logisticsTemplate;
    }
}
