package cn.wise.sc.pay.business.exhibition.controller;


import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.points.service.IPointDetailService;
import cn.wise.sc.pay.api.points.service.IPointMemberShipService;
import cn.wise.sc.pay.api.points.service.IPointTypeService;
import cn.wise.sc.pay.business.exhibition.service.IEntranceQRcodeService;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionStoreService;
import cn.wise.sc.pay.business.exhibition.service.IOrderService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.mq.client.IProducer;
import cn.wise.sc.pay.domain.exhibition.bean.Exhibition;
import cn.wise.sc.pay.domain.exhibition.query.GoodsQuery;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 订单类 前端控制器
 * </p>
 *
 * @author qh
 * @since 2019-11-22
 */
@Api(description = "展览订单")
@RestController
@RequestMapping("/order")
@CrossOrigin
public class OrderController {

    final
    IOrderService iOrderService;
    final
    IEntranceQRcodeService iEXQRCode;
    final
    IProducer iProducer;
    @Autowired
    IOrdersService iOrdersService;
    //    @Autowired
    private IPointMemberShipService pointMemberShipService;

    //    @Autowired
    private IPointMemberShipService memberService;

    //    @Autowired
    private IPointTypeService pointTypeService;

    //    @Autowired
    private IPointDetailService pointDetailService;

    public OrderController(IOrderService iOrderService, IEntranceQRcodeService iEXQRCode, IProducer iProducer) {
        this.iOrderService = iOrderService;
        this.iEXQRCode = iEXQRCode;
        this.iProducer = iProducer;
    }


    @PostMapping
    @ApiOperation("预下单")
    @ApiImplicitParam(name = "orderQuery", value = "下单", paramType = "body", dataType = "OrderQuery")
    public BaseResponse preOrder(@RequestBody OrderDetailQuery<GoodsQuery> orderQuery) {

        return iOrderService.placeOrder(orderQuery);
    }

    public static void main(String[] args) {
        OrderDetailQuery query = new OrderDetailQuery();
        query.setField("31");
        List<GoodsQuery> list = new ArrayList<>();
        GoodsQuery goodsQuery = new GoodsQuery();
        goodsQuery.setId(43L);
        goodsQuery.setName("yu联人");
        goodsQuery.setNum(1);
        list.add(goodsQuery);
        GoodsQuery goodsQuery1 = new GoodsQuery();
        goodsQuery1.setId(42L);
        goodsQuery1.setName("yu联人");
        goodsQuery1.setNum(1);
        list.add(goodsQuery1);
        query.setGoods(list);
        query.setSellerId(1L);
        query.setPayPrice(200D);
        System.out.println(JSON.toJSONString(query));
    }

    @GetMapping("/status/{orderNo}")
    @ApiOperation("获取展览订单状态")
    @ApiImplicitParam(name = "orderNo", value = "预下单号", paramType = "path", dataType = "String")
    public BaseResponse getOrderStatus(@PathVariable("orderNo") String orderNo) {
        return iOrderService.getOrderStatus(orderNo);
    }

    @GetMapping("/preOderNo")
    @ApiOperation("获取唯一订单号(下单前)")
    public BaseResponse getPreOrderNo() {
        Snowflake snowflake = IdUtil.getSnowflake(8, 8);
        String newIdStr = snowflake.nextIdStr();
        iOrderService.getPreOrderNo();
        return BaseResponse.ok(newIdStr);
    }

    @PostMapping("/prePay")
    @ApiOperation("预支付订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse prePayOrder(String orderNo) {

        BaseResponse responseEntity = iOrderService.payOrder(orderNo);
        return responseEntity;
    }

    @PostMapping("/pay/success")
    @ApiOperation("支付成功回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderSuccess(String orderNo) {

        if (StrUtil.isEmpty(orderNo)) {
            return BaseResponse.error("订单号为空", Exhibition.class);
        }
        //获取支付订单信息
        Map condition = new HashMap();
        condition.put("order_no", orderNo);
        Orders orders = iOrdersService.getOneOrder(JSON.toJSONString(condition)).getData();
        //修改订单状态为待收货
        //状态修改成功时生成二维码
        return iEXQRCode.getQRCode(orders);
    }


    @PostMapping("/pay/fail")
    @ApiOperation("支付失败回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderFail(String orderNo) {

        //支付成功
        iOrderService.payOrderFail(orderNo);

        return BaseResponse.error("订单支付失败", new Orders());
    }


    @Autowired
    IExhibitionStoreService iExhibitionStoreService;

    @GetMapping("/test/qh")
    public void test() {
        iExhibitionStoreService.newStore(99L, 100);
        iExhibitionStoreService.addStore(99L, 100);
        iExhibitionStoreService.decreaseStore(99L, 50);
        iExhibitionStoreService.saveDB(99L);
    }
}

