package cn.wise.sc.pay.domain.exhibition.vo;

import cn.wise.sc.pay.domain.exhibition.bean.Exhibition;
import cn.wise.sc.pay.domain.exhibition.bean.Ticket;
import cn.wise.sc.pay.domain.exhibition.bean.TicketNotice;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: app端的展览实体
 * @author: qh
 * @create: 2019-11-19 16:15
 **/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ExhibitionAPPVO implements Serializable {

    private static final long serialVersionUID = -705776156944070888L;

    private Long id;
    /**
     * 标签
     */
    private String tag;

    /**
     * 标题
     */
    private String title;

    /**
     * 场次
     */
    private String ground;

    /**
     * 封面路径
     */
    private String coverUrl;


    /**
     * 地址
     */
    private String address;

    /**
     * 市场价
     */
    private Double price;

    /**
     * 折扣价
     */
    private Double discountPrice;
    /**
     * 是否开启广告位 0:不开启 1:开启
     */
    private Integer advertisingType;
    /**
     * 广告路径
     */
    private String advertisingUrl;
    /**
     * 详情描述
     */
    private String describe;

    /**
     * 状态 预售 进行中 结束
     */
    private String status;

    /**
     * 类型
     */
    private String type;

    /**
     * 想看人数
     */
    private Integer wantWatch;

    private List<TicketNoticeVO> ticketNoticeVOS;

    private List<TicketVO> ticketVOS;

    public ExhibitionAPPVO(Exhibition exhibition) {

        this.address = exhibition.getAddress();
        this.coverUrl = exhibition.getCoverUrl();
        this.advertisingUrl= exhibition.getAdvertisingUrl();
        this.advertisingType = exhibition.getAdvertisingType();
        this.price = exhibition.getPrice();
        this.discountPrice = exhibition.getDiscountPrice();
        this.id = exhibition.getId();
        this.tag = exhibition.getTag();
        this.title = exhibition.getTitle();
        this.wantWatch = exhibition.getWantWatch();
        this.describe = exhibition.getDescribe();
        this.status = getStatus(exhibition.getStatus());
        this.ground = exhibition.getGround();
        this.type = exhibition.getType();
    }

    /**
     * 详情 添加了购票须知
     *
     * @param exhibition
     * @param ticketNotices
     * @return
     */
    public static ExhibitionAPPVO getExDetailAppVO(Exhibition exhibition, List<Ticket> tickets, List<TicketNotice> ticketNotices) {

        ExhibitionAPPVO exhibitionAppVO = new ExhibitionAPPVO(exhibition);

        List<TicketNoticeVO> ticketNoticeVOList = new ArrayList<>(ticketNotices.size());
        for (TicketNotice ticketNotice : ticketNotices) {
            TicketNoticeVO ticketNoticeVO = new TicketNoticeVO(ticketNotice);
            ticketNoticeVOList.add(ticketNoticeVO);
        }

        List<TicketVO> ticketVOS = new ArrayList<>();
        for (Ticket ticket : tickets) {
            if (ticket.getAssociationId().equals(exhibition.getId())) {

                TicketVO ticketVO = TicketVO.builder()
                        .name(ticket.getName())
                        .id(ticket.getId())
                        .price(ticket.getPrice())
                        .build();
                ticketVOS.add(ticketVO);
            }
        }
        exhibitionAppVO.setTicketVOS(ticketVOS);

        exhibitionAppVO.setTicketNoticeVOS(ticketNoticeVOList);

        return exhibitionAppVO;
    }

    private String getStatus(Integer status) {
        if (status == 1) {
            return "进行中";
        }
        if (status == 2) {
            return "预售中";
        } else {
            return "已结束";
        }
    }
}
