package cn.wise.sc.pay.api.merchant.service;

import cn.wise.sc.pay.api.merchant.service.hystrix.BannerImageServiceImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.BannerImage;
import cn.wise.sc.pay.domain.merchant.query.BannerImageQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;

@FeignClient(value = "saas-v1-merchant", path = "merchant/banner/", fallback = BannerImageServiceImpl.class)
public interface IBannerImageService {

    @PutMapping("updateBannerImage")
    BaseResponse addDevice(@SpringQueryMap BannerImage bImage);

    @GetMapping("bannerImages")
    BaseResponse getBannerImageList();

    @GetMapping("advertImages")
    BaseResponse getAdvertImageList();

    @GetMapping("typeList")
    BaseResponse getTypeList();

    @DeleteMapping("deleteBannerImage")
    BaseResponse deletePic(@RequestParam("id") Integer id);

    @GetMapping("eshopImages")
    BaseResponse getEshopImageList();

    /**
     * 收银端根据商家id获取商家广告图
     *
     * @param merchantId
     * @return
     */
    @GetMapping("cashierAdvert")
    BaseResponse getCashierAdvert(@RequestParam("merchantId") Long merchantId);

    /**
     * 添加收银端广告图
     *
     * @param bannerImageQuery
     * @return
     */
    @PostMapping("cashierAdvert")
    BaseResponse<BannerImage> addCashierAdvert(@RequestBody BannerImageQuery bannerImageQuery);

    @GetMapping("/list/merchants")
    BaseResponse listBannerByMerchants(@RequestParam("pageNo") Integer pageNo,
                                       @RequestParam("pageSize") Integer pageSize,
                                       @RequestParam("name") String name);
}
