package cn.wise.sc.pay.api.merchant.service;

import cn.wise.sc.pay.api.merchant.service.hystrix.BankPermitServiceImpl;
import cn.wise.sc.pay.api.merchant.service.hystrix.MerchDiscountImpl;
import cn.wise.sc.pay.api.merchant.service.hystrix.MerchantImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchDiscount;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * @description:
 * @author: qh
 * @create: 2020-09-07 11:05
 **/
@FeignClient(value = "saas-v1-merchant", path = "/merchant/discount", fallback = MerchDiscountImpl.class)
public interface IMerchDiscountService {

    @GetMapping("/{seller}")
    @ApiOperation("根据商户id获取折扣配置")
    @ApiImplicitParam(name = "seller", value = "商户id")
    BaseResponse<List<MerchDiscount>> getMerchDiscountBySeller(@PathVariable("seller") Long seller);

    @PostMapping("/updateMerchDiscount")
    @ApiOperation("更新商户折扣信息")
    BaseResponse<Boolean> updateMerchDiscount(@RequestBody MerchDiscount merchDiscount);

    @PostMapping("/addOrUpdateMerchDiscount")
    @ApiOperation("新增一个折扣信息")
    BaseResponse<Boolean> addNewMerchDiscount(@RequestBody MerchDiscount merchDiscount);

    @PostMapping("/delete/{id}")
    @ApiOperation("删除一个折扣信息")
    BaseResponse<Boolean> deleteMerchDiscount(@PathVariable("id") Long id);

    @PostMapping("/deleteBySeller/{seller}")
    @ApiOperation("按照商户id删除折扣信息")
    BaseResponse<Boolean> deleteMerchDiscountBySeller(@PathVariable("seller") String seller);
}
