package cn.wise.sc.pay.business.merchant.controller;

import cn.wise.sc.pay.business.merchant.service.IBannerImageService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.BannerImage;
import cn.wise.sc.pay.domain.merchant.bean.BannerType;
import cn.wise.sc.pay.domain.merchant.query.BannerImageQuery;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author lqk
 * @since 2019/10/08 09:09
 */
@RestController
@CrossOrigin
@RequestMapping("merchant/banner/")
@Slf4j
public class BannerImageController {

    @Autowired
    IBannerImageService bannerImageService;

    @PostMapping("uploadPic")
    @ApiOperation("添加焦点图")
    @ApiImplicitParam(name = "file", value = "图片", paramType = "body", dataType = "MultipartFile")
    public BaseResponse<BannerImage> uploadPic(MultipartFile file) {
        return bannerImageService.addBannerImage(file, 0);
    }

    @PostMapping("uploadAdvert")
    @ApiOperation("添加广告图")
    @ApiImplicitParam(name = "file", value = "图片", paramType = "body", dataType = "MultipartFile")
    public BaseResponse<BannerImage> uploadAdvert(MultipartFile file) {
        return bannerImageService.addBannerImage(file, 1);
    }

    /**
     * 添加收银端广告图
     *
     * @param bannerImageQuery
     * @return
     */
    @PostMapping("cashierAdvert")
    @ApiOperation("添加收银端广告图")
    public BaseResponse<BannerImage> addCashierAdvert(@RequestBody BannerImageQuery bannerImageQuery) {
        return bannerImageService.addCashierAdvert(bannerImageQuery, bannerImageQuery.getType());
    }

    @PostMapping("uploadEshopBanner")
    @ApiOperation("添加广告图")
    @ApiImplicitParam(name = "file", value = "图片", paramType = "body", dataType = "MultipartFile")
    public BaseResponse<BannerImage> uploadEshopBanner(MultipartFile file) {
        return bannerImageService.addBannerImage(file, 2);
    }

    @PutMapping("updateBannerImage")
    public BaseResponse addDevice(@RequestBody BannerImage bImage) {
        if (bImage.getId() == null) {
            return BaseResponse.error("当前焦点图有误", BannerImage.class);
        }
        boolean b = bannerImageService.updateById(bImage);
        return b ? BaseResponse.ok(bImage) : BaseResponse.error("焦点图修改失败", bImage);
    }

    @GetMapping("bannerImages")
    public BaseResponse getBannerImageList() {
        List<BannerImage> list = bannerImageService.getBannerImages("0");
        return BaseResponse.ok(list);
    }

    @GetMapping("advertImages")
    public BaseResponse getAdvertImageList() {
        List<BannerImage> list = bannerImageService.getBannerImages("1");
        return BaseResponse.ok(list);
    }

    /**
     * 收银端根据商家id获取商家广告图
     *
     * @param merchantId
     * @return
     */
    @GetMapping("cashierAdvert")
    @ApiOperation(value = "根据商户id获取商家广告图")
    @ApiImplicitParam(name = "merchantId", value = "商家id", dataType = "Long", paramType = "query")
    public BaseResponse getCashierAdvert(@NotNull Long merchantId) {
        BaseResponse response = bannerImageService.getCashierAdvert(merchantId, -2);
        return response;
    }

    @GetMapping("eshopImages")
    public BaseResponse getEshopImageList() {
        List<BannerImage> list = bannerImageService.getBannerImages("2");
        return BaseResponse.ok(list);
    }

    @GetMapping("typeList")
    public BaseResponse getTypeList() {
        return BaseResponse.ok(BannerType.list());
    }

    @DeleteMapping("deleteBannerImage")
    public BaseResponse deletePic(Integer id) {
        return bannerImageService.deleteBannerImage(id);
    }

    @GetMapping("/list/merchants")
    /**
     * 根据店铺名称分页店铺和店铺对应的广告图
     */
    public BaseResponse listBannerByMerchants(Integer pageNo, Integer pageSize, String name) {
        BaseResponse response = bannerImageService.listBannerByMerchants(pageNo, pageSize,name);
        return response;
    }
}
