package cn.wise.sc.pay.business.merchant.controller;

import cn.wise.sc.pay.business.merchant.service.IMerchDiscountService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchDiscount;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 商户折扣控制器
 * @author: qh
 * @create: 2020-09-07 09:43
 **/
@CrossOrigin
@RestController
@Api(tags = "折扣控制器")
@RequestMapping("/merchant/discount")
public class MerchDiscountController {

    final
    IMerchDiscountService iMerchDiscountService;

    public MerchDiscountController(IMerchDiscountService iMerchDiscountService) {
        this.iMerchDiscountService = iMerchDiscountService;
    }

    @GetMapping("/{seller}")
    @ApiOperation("根据商户id获取折扣配置")
    @ApiImplicitParam(name = "seller",value = "商户id")
    public BaseResponse<List<MerchDiscount>> getMerchDiscountBySeller(@PathVariable("seller") Long seller) {
        return iMerchDiscountService.getBySellerId(seller);
    }

//    @PostMapping("/updateMerchDiscount")
//    @ApiOperation("更新商户折扣信息")
//    public BaseResponse<Boolean> updateMerchDiscount(@RequestBody MerchDiscount merchDiscount){
//        return iMerchDiscountService.updateMerchDiscount(merchDiscount);
//    }

    @PostMapping("/addOrUpdateMerchDiscount")
    @ApiOperation("新增一个折扣信息")
    public BaseResponse<Boolean> addNewMerchDiscount(@RequestBody MerchDiscount merchDiscount){
        return iMerchDiscountService.addNewMerchDiscount(merchDiscount);
    }

    public static void main(String[] args) {

        MerchDiscount merchDiscount = new MerchDiscount();
        merchDiscount.setSellerId(99L);

        List<MerchDiscount.DiscountSetting> list = new ArrayList<>();
        MerchDiscount.DiscountSetting setting = new MerchDiscount.DiscountSetting();
        setting.setDiscount(88D);
        setting.setId("1");
        MerchDiscount.DiscountSetting setting2 = new MerchDiscount.DiscountSetting();
        setting2.setDiscount(88D);
        setting2.setId("1");

        list.add(setting);
        list.add(setting2);

        String s = JSON.toJSONString(list);
        merchDiscount.setDiscountStr(s);
        System.out.println(JSON.toJSONString(merchDiscount));
    }

    @PostMapping("/delete/{id}")
    @ApiOperation("根据ID删除一个折扣信息")
    public BaseResponse<Boolean> deleteMerchDiscount(@PathVariable("id") Long id){
        return iMerchDiscountService.deleteMerchDiscount(id);
    }

    @PostMapping("/deleteBySeller/{seller}")
    @ApiOperation("根据商户ID删除折扣信息")
    public BaseResponse<Boolean> deleteMerchDiscountBySeller(@PathVariable("seller") Long seller){
        return iMerchDiscountService.deleteMerchDiscountBySeller(seller);
    }

}
