package cn.wise.sc.pay.business.merchant.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.merchant.service.IDeviceUserService;
import cn.wise.sc.pay.business.merchant.service.IMerchantService;
import cn.wise.sc.pay.common.core.model.BaseController;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.merchant.bean.DeviceUser;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.merchant.excelvo.MerchantExcelVO;
import cn.wise.sc.pay.domain.merchant.query.MerchantQuery;
import cn.wise.sc.pay.domain.merchant.vo.StoreVo;
import cn.wise.sc.pay.laas.userbac.business.service.UserRoleService;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUserRole;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author lqk
 * @since 2019/12/17
 */
@RestController
@CrossOrigin
@RequestMapping("/merchant/")
@Slf4j
@Validated
public class MerchantController extends BaseController<IMerchantService, Merchant> {

    @Autowired
    IMerchantService merchantService;

    @Autowired
    UserService userService;

    @Autowired
    IDeviceUserService deviceUserService;

    @Autowired
    UserRoleService userRoleService;

    public MerchantController(IMerchantService iService) {
        super(iService);
    }

    @GetMapping("/getMerchNameByIds")
    public BaseResponse getMerchantByIds(List<Long> ids) {
        if (ids == null || ids.size() == 0) {
            return BaseResponse.error("参数有误", Merchant.class);
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.in("id", ids);
        wrapper.ne("del_status", 1);
        return BaseResponse.ok(merchantService.list(wrapper));
    }

    @GetMapping("like")
    public BaseResponse getMerchList(@RequestParam(value = "merchName", required = false) String merchName) {
        QueryWrapper wrapper = new QueryWrapper();
        if (StringUtils.isNotBlank(merchName)) {
            wrapper.like("merch_name", merchName);
            wrapper.ne("del_status", 1);
        }
        return BaseResponse.ok(merchantService.list(wrapper));
    }

    @PostMapping("addMerchant")
    public BaseResponse addMerchant(@RequestBody MerchantQuery merchantQuery) {
        //判断商户基本信息是否合法
        if (StringUtils.isBlank(merchantQuery.getMerchName())) {
            return BaseResponse.error("商户名称为空", Merchant.class);
        }
        return merchantService.registerMerchant(merchantQuery);
    }

    @DeleteMapping("deleteMerchant")
    public BaseResponse deleteMerchant(Long merchantId) {
        if (merchantId == null) {
            return BaseResponse.error("删除商户出现错误", Merchant.class);
        }
        return merchantService.deleteMerchant(merchantId);
    }

    @PutMapping("updateMerchant")
    public BaseResponse updatemerchant(@RequestBody Merchant merchant) {
        if (merchant.getId() == null) {
            return BaseResponse.error("参数错误", Merchant.class);
        }
        return merchantService.updateMerchant(merchant);
    }

    @PostMapping("getMerchantPage")
    public BaseResponse<Page<Merchant>> getMerchantPage(Integer status, String merchantName,
                                                        Integer pageNo, Integer pageSize, Long start, Long end) {
        TimeBucket timeBucket = new TimeBucket();
        timeBucket.setStartTime(start);
        timeBucket.setEndTime(end);
        PageQuery pageQuery = new PageQuery();
        pageQuery.setPageNo(pageNo);
        pageQuery.setPageSize(pageSize);
        return merchantService.getMerchantPage(status, merchantName, pageQuery, timeBucket);
    }

    @GetMapping("getMerchantInfo")
    public BaseResponse<Merchant> getMerchantInfo(Long id) {
        if (id != null) {
            Merchant merchant = merchantService.getById(id);
            return BaseResponse.ok(merchant);
        }
        return BaseResponse.ok(null);
    }

    @PostMapping("getMerchNames")
    public BaseResponse getMerchNameByIds(@RequestBody List<Long> merchIds) {
        if (merchIds != null && merchIds.size() > 0) {
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.in("id", merchIds);
            wrapper.ne("del_status", 1);
            List<Merchant> list = merchantService.list(wrapper);
            BaseResponse baseResponse = BaseResponse.ok(new Merchant());
            baseResponse.setList(list);
            return baseResponse;
        }
        return BaseResponse.error("查找失败", Merchant.class.getName());
    }

    @GetMapping("getMerchOrders")
    public BaseResponse getMerchantOrders(String merchAppid, long start, long end, String status) {
        TimeBucket timeBucket = new TimeBucket();
        timeBucket.setStartTime(start);
        timeBucket.setEndTime(end);
        return merchantService.getMerchantOrders(merchAppid, timeBucket, status);
    }

    @GetMapping("merchStaff")
    public BaseResponse getMerchStaff(Long merchId, String username, Integer pageNo, Integer pageSize) {
        if (merchId == null) {
            return BaseResponse.error("店铺信息异常", null);
        }
        Page pg = new Page();
        if (pageNo != null && pageSize != null) {
            pg.setCurrent(pageNo);
            pg.setSize(pageSize);
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("merch_id", merchId);
        if (StringUtils.isNotBlank(username)) {
            wrapper.like("username", username);
        }
        pg = (Page) userService.page(pg, wrapper);

        List<SysUser> list = pg.getRecords();
        if (list == null || list.size() == 0) {
            return BaseResponse.ok(pg);
        }
        List<Long> merchatIds = list.stream().map(arg -> arg.getMerchId()).collect(Collectors.toList());
        List<Merchant> merchants = this.getMerchNameByIds(merchatIds).getList();

        List<Integer> userIds = list.stream().map(arg -> arg.getUserId()).collect(Collectors.toList());
        List<SysUser> newList = new ArrayList<>();
        QueryWrapper qw = new QueryWrapper();
        qw.in("user_id", userIds);
        List<DeviceUser> deviceUsers = deviceUserService.list(qw);
        list.stream().forEach(arg -> {
            Merchant merchant = merchants.stream().filter(
                    param -> param.getId() == arg.getMerchId().longValue())
                    .findFirst().orElse(null);
            List<DeviceUser> deviceUserList = deviceUsers.stream().filter(
                    param -> param.getUserId() == arg.getUserId().intValue())
                    .collect(Collectors.toList());

            SysUser sysUser = new SysUser();
            BeanUtil.copyProperties(arg, sysUser);
            if (merchant != null) {
                sysUser.setFiled2(merchant.getMerchName());
                sysUser.setFiled1("");
                if (deviceUserList != null && deviceUserList.size() > 0) {
                    for (DeviceUser deviceUser : deviceUserList) {
                        sysUser.setFiled1(sysUser.getFiled1() + " " + deviceUser.getDeviceCode());
                    }
                } else {
                    sysUser.setFiled1("");
                }
            }
            newList.add(sysUser);
        });
        pg.setRecords(newList);
        return BaseResponse.ok(pg);
    }

    @PostMapping("/userStaff")
    public BaseResponse getMerchantStaff(Long merchId, Integer roleId, String username, Integer pageNo, Integer pageSize) {
        Page pg = new Page();
        if (pageNo != null && pageSize != null) {
            pg.setCurrent(pageNo);
            pg.setSize(pageSize);
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.orderByDesc("update_time");
        if (roleId != null) {
            QueryWrapper roleUsers = new QueryWrapper();
            roleUsers.eq("role_id", roleId);
            List<SysUserRole> users = userRoleService.list(roleUsers);
            Set<Integer> userIds = new HashSet<>();
            for (SysUserRole uRole : users) {
                userIds.add(uRole.getUserId());
            }
            wrapper.in("user_id", userIds);
        }
        if (merchId != null) {
            wrapper.eq("merch_id", merchId);
        } else {
            wrapper.isNotNull("merch_id");
        }
        if (StringUtils.isNotBlank(username)) {
            wrapper.like("username", username);
        }
        //收银端店员分页
        pg = (Page) userService.page(pg, wrapper);

        List<SysUser> list = pg.getRecords();
        if (list.size() == 0) {
            return BaseResponse.ok(pg);
        }
        List<Long> merchatIds = list.stream().map(arg -> arg.getMerchId()).collect(Collectors.toList());
        List<Merchant> merchants = this.getMerchNameByIds(merchatIds).getList();

        List<Integer> userIds = list.stream().map(arg -> arg.getUserId()).collect(Collectors.toList());
        List<SysUser> newList = new ArrayList<>();
        QueryWrapper qw = new QueryWrapper();
        qw.in("user_id", userIds);
        List<DeviceUser> deviceUsers = deviceUserService.list(qw);

        list.stream().forEach(arg -> {

            Merchant merchant = merchants.stream().filter(
                    param -> param.getId() == arg.getMerchId().longValue())
                    .findFirst().orElse(null);
            List<DeviceUser> deviceUserList = deviceUsers.stream().filter(
                    param -> param.getUserId() == arg.getUserId().intValue())
                    .collect(Collectors.toList());

            SysUser sysUser = new SysUser();
            BeanUtil.copyProperties(arg, sysUser);
            if (merchant != null) {
                sysUser.setFiled2(merchant.getMerchName());
                sysUser.setFiled1("");
                if (deviceUserList != null && deviceUserList.size() > 0) {
                    for (DeviceUser deviceUser : deviceUserList) {
                        sysUser.setFiled1(sysUser.getFiled1() + " " + deviceUser.getDeviceCode());
                    }
                } else {
                    sysUser.setFiled1("");
                }
            }
            newList.add(sysUser);
        });
        pg.setRecords(newList);
//        pg.setTotal(newList.size());
        return BaseResponse.ok(pg);
    }

    @ApiOperation(value = "人员管理条件查询列表")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "username", value = "用户名", paramType = "query", dataType = "String"),
                    @ApiImplicitParam(name = "shopName", value = "店铺名", paramType = "query", dataType = "String")
            }
    )
    @GetMapping("/userMerchant/page")
    public BaseResponse userMerchant(PageQuery pageQuery, String username, String shopName) {
        BaseResponse baseResponse = merchantService.userMerchant(pageQuery, username, shopName);
        return baseResponse;

    }

    @GetMapping("merchantExport")
    public void exportMerchantList(@RequestParam("status") Integer status, HttpServletResponse response) {
        try {
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");

            String fileName = URLEncoder.encode("商户列表" + System.currentTimeMillis(), "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            QueryWrapper wrapper = new QueryWrapper();
            if (status != null) {
                wrapper.eq("manage_status", status);
            }
            wrapper.ne("del_status", 1);
            List<Merchant> result = merchantService.list(wrapper);
            List<MerchantExcelVO> excelVOS = new ArrayList<>();
            result.stream().forEach(arg -> {
                MerchantExcelVO merchantExcelVO = new MerchantExcelVO(arg);
                excelVOS.add(merchantExcelVO);
            });
            EasyExcel.write(response.getOutputStream(), MerchantExcelVO.class).sheet("模板").doWrite(excelVOS);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @ApiOperation(value = "根据商户名称寻找商户id")
    @ApiImplicitParam(name = "merchantName", value = "商户名称", dataType = "String", paramType = "query")
    @GetMapping("name")
    public BaseResponse<List<Long>> getMerchantIdByName(String merchantName) {
        BaseResponse<List<Long>> merchantIdByName = merchantService.getMerchantIdByName(merchantName);
        return merchantIdByName;
    }

    //根据商户id取消商家白名单
    @ApiOperation(value = "根据id取消白名单")
    @ApiImplicitParam(name = "id", value = "商户id", dataType = "long", paramType = "path")
    @PutMapping("cancelWhiteList/{id}")
    public BaseResponse cancelWhiteList(@PathVariable Long id) {

        BaseResponse responseEntity = merchantService.cancelWhiteList(id);
        return responseEntity;
    }

    //根据商户id添加商家白名单
    @ApiOperation(value = "根据id添加白名单")
    @ApiImplicitParam(name = "id", value = "商户id", dataType = "long", paramType = "path")
    @PutMapping("addWhiteList/{id}")
    public BaseResponse addWhiteList(@PathVariable Long id) {
        BaseResponse responseEntity = merchantService.addWhiteList(id);
        return responseEntity;
    }

    /**
     * 获取店铺详情和额外的点评信息
     *
     * @param id 店铺id
     * @return
     */
    @GetMapping("storeInfo/{id}")
    BaseResponse<StoreVo> getStoreInfo(@PathVariable("id") Long id) {
        return merchantService.getStoreInfo(id);
    }

    @PostMapping("storeInfo/page")
    public BaseResponse getStoreInfoPage(@RequestBody PageWrapperQuery pageWrapperQuery) {
        return merchantService.getStoreInfoPage(pageWrapperQuery);
    }

    @PutMapping("storeInfo")
    BaseResponse updateStore(@RequestBody StoreVo storeVo) {
        return merchantService.updateStore(storeVo);
    }

    /**
     * 验证敏感操作密码
     *
     * @param seller 商户id
     * @param pwd    密码
     * @return bool
     */
    @PostMapping("/check/pwd")
    BaseResponse<Boolean> checkPwd(String seller, String pwd) {
        return merchantService.checkPwd(seller, pwd);
    }

    /**
     * 新增商户授权码
     *
     * @param seller 商户id
     * @param sensitivePwd    授权码
     * @return String
     */
//    @PostMapping("addSensitivePwd")
//    @ApiImplicitParams(
//            {
//                    @ApiImplicitParam(name = "seller", value = "商户id", paramType = "query", dataType = "String"),
//                    @ApiImplicitParam(name = "sensitivePwd", value = "授权码", paramType = "query", dataType = "String")
//            }
//    )
//    BaseResponse<String> addSensitivePwd(String seller,String sensitivePwd){
//        return merchantService.addSensitivePwd(seller,sensitivePwd);
//    }

    /**
     * 修改商户授权码
     *
     * @param seller 商户id
     * @param sensitivePwd    授权码
     * @return String
     */
    @ApiOperation(value = "修改授权码")
    @PostMapping("updateSensitivePwd")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "seller", value = "商户id", paramType = "query", dataType = "String"),
                    @ApiImplicitParam(name = "sensitivePwd", value = "授权码", paramType = "query", dataType = "String")
            }
    )
    BaseResponse<String> updateSensitivePwd(String seller,String sensitivePwd){
        return merchantService.updateSensitivePwd(seller,sensitivePwd);
    }

    /**
     * 删除商户授权码
     *
     * @param seller 商户id
     * @return String
     */
    @ApiOperation(value = "删除商户授权码")
    @PostMapping("deleteSensitivePwd")
    @ApiImplicitParams(
                    @ApiImplicitParam(name = "seller", value = "商户id", paramType = "query", dataType = "Integer")
    )
    BaseResponse<String> deleteSensitivePwd(String seller){
        return merchantService.deleteSensitivePwd(seller);
    }
}
