package cn.wise.sc.pay.business.merchant.controller;

import cn.wise.sc.pay.business.merchant.service.ICategoryService;
import cn.wise.sc.pay.business.merchant.service.IStoreService;
import cn.wise.sc.pay.common.core.model.BaseController;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.Store;
import cn.wise.sc.pay.domain.merchant.query.StoreQuery;
import cn.wise.sc.pay.domain.merchant.vo.StoreVo;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 商户店铺控制器
 */
@RestController
@CrossOrigin
@RequestMapping("/merchant/store")
@Slf4j
public class StoreController extends BaseController<IStoreService, Store> {

    private final IStoreService storeService;

    private final ICategoryService categoryService;

    public StoreController(IStoreService iService, ICategoryService categoryService, IStoreService storeService) {
        super(iService);
        this.categoryService = categoryService;
        this.storeService = storeService;
    }

    @ApiOperation(value = "添加店铺信息")
    @PostMapping(path = "/add")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "storeQuery", value = "店铺信息对象", paramType = "body")
            }
    )
    public BaseResponse addStore(@RequestBody StoreQuery storeQuery) {

        BaseResponse responseEntity = storeService.addStore(storeQuery);

        return responseEntity;
    }

    @PutMapping("/update")
    @ApiOperation(value = "更新店铺店铺信息")
    @ApiImplicitParam(name = "storeQuery", value = "店铺参数传值对象", dataType = "StoreQuery", paramType = "body")
    public BaseResponse updateStore(@RequestBody StoreQuery storeQuery) {

        BaseResponse responseEntity = storeService.updateStore(storeQuery);

        return responseEntity;
    }


    @PutMapping("/category/label")
    @ApiOperation(value = "经营类目接口")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "categoryIds", value = "店铺分类id，多个分类用id / 隔开", dataType = "string", paramType = "query"),
                    @ApiImplicitParam(name = "labelIds", value = "店铺标签id，多个标签id用 , 隔开", dataType = "string", paramType = "query"),
                    @ApiImplicitParam(name = "storeId", value = "当前店铺id", dataType = "int", paramType = "query")
            }
    )
    public BaseResponse updateBusinessCategory(String categoryIds, String labelIds, Long storeId) {


        BaseResponse responseEntity = storeService.businessCategory(categoryIds, labelIds, storeId);

        return responseEntity;
    }


    @GetMapping("/storeInfo/{id}")
    public BaseResponse<StoreVo> getStoreInfo(@PathVariable("id") Long id){
       return storeService.getStoreInfo(id);
    }

}
