package cn.wise.sc.pay.business.merchant.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.merchant.query.MerchantQuery;
import cn.wise.sc.pay.domain.merchant.vo.StoreVo;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
public interface IMerchantService extends IService<Merchant> {

    /**
     * 新增商户
     *
     * @param merchantQuery
     * @return
     */
    BaseResponse registerMerchant(MerchantQuery merchantQuery);

    /**
     * 逻辑删除商户
     *
     * @param merchantId
     * @return
     */
    BaseResponse deleteMerchant(Long merchantId);

    /**
     * 修改商户基本信息
     *
     * @param merchant
     * @return
     */
    BaseResponse updateMerchant(Merchant merchant);

    /**
     * 根据条件获取商户列表
     *
     * @param status
     * @param merchantName
     * @param pageQuery
     * @param timeBucket
     * @return
     */
    BaseResponse<Page<Merchant>> getMerchantPage(Integer status, String merchantName,
                                                 PageQuery pageQuery, TimeBucket timeBucket);

    /**
     * 获取奥索商户订单
     *
     * @param merchAppid
     * @param timeBucket
     * @param status
     * @return
     */
    BaseResponse getMerchantOrders(String merchAppid, TimeBucket timeBucket, String status);

    /**
     * 商户登录收银系统
     *
     * @param merchantAppid
     * @param password
     * @return
     */
    BaseResponse loginCahier(String merchantAppid, String password);

    /**
     * 根据商户名称查询id
     *
     * @param merchantName
     * @return
     */
    BaseResponse<List<Long>> getMerchantIdByName(String merchantName);

    BaseResponse cancelWhiteList(Long id);

    BaseResponse addWhiteList(Long id);

    /**
     * @param pageQuery 分页参数对象
     * @param username  姓名
     * @param shopName  店铺名字
     * @return
     */
    BaseResponse userMerchant(PageQuery pageQuery, String username, String shopName);

    /**
     * 根据店铺名称筛选 分页
     *
     * @return
     */
    BaseResponse<IPage<Merchant>> listMerchant(Integer pageNo, Integer pageSize,String name);

    BaseResponse<StoreVo> getStoreInfo(Long id);

    /**
     * 获取店铺列表 分页/有评分等信息
     * @param pageWrapperQuery 分页参数
     * @return
     */
    BaseResponse getStoreInfoPage(PageWrapperQuery pageWrapperQuery);

    /**
     * 更新店铺详情
     * @param storeVo 店铺实体
     * @return
     */
    BaseResponse updateStore(StoreVo storeVo);


    /**
     * 验证敏感操作密码
     * @param seller 商户id
     * @param pwd 密码
     * @return bool
     */
    BaseResponse<Boolean> checkPwd(String seller,String pwd);

    /**
     * 新增商户授权码
     *
     * @param seller 商户id
     * @param sensitivePwd    授权码
     * @return bool
     */
    BaseResponse<String> addSensitivePwd(String seller,String sensitivePwd);

    /**
     * 维护商户授权码
     *
     * @param seller 商户id
     * @param sensitivePwd    授权码
     * @return bool
     */
    BaseResponse<String> updateSensitivePwd(String seller,String sensitivePwd);

    /**
     * 删除商户授权码
     *
     * @param seller 商户id
     * @return bool
     */
    BaseResponse<String> deleteSensitivePwd(String seller);
}
