package cn.wise.sc.pay.business.merchant.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.crypto.SecureUtil;
import cn.wise.sc.pay.business.merchant.mapper.DeviceMapper;
import cn.wise.sc.pay.business.merchant.service.IDeviceService;
import cn.wise.sc.pay.business.merchant.service.IMerchantService;
import cn.wise.sc.pay.business.merchant.service.ITypeService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.Device;
import cn.wise.sc.pay.domain.merchant.bean.DeviceType;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

/**
 * @ClassName BankPermitServiceImpl
 * @Author Administrator
 * @Date 2019/12/18
 * @Version V1.0
 **/
@Service
public class DeviceServiceImpl extends ServiceImpl<DeviceMapper, Device> implements IDeviceService {
    private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();
    @Autowired
    IMerchantService merchantService;

    @Autowired
    ITypeService typeService;

    @Override
    public BaseResponse insertDevice(Device device) {
        Merchant merchant = merchantService.getById(device.getMerchId());
        if (merchant != null) {
            device.setMerchName(merchant.getMerchName());
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("name", device.getTypeName());
        wrapper.last(" limit 1 ");
        DeviceType deviceType = typeService.getOne(wrapper);
        if (deviceType == null) {
            DeviceType type = DeviceType.builder()
                    .name(device.getTypeName())
                    .createTime(System.currentTimeMillis())
                    .build();
            typeService.save(type);
            deviceType = typeService.getOne(wrapper);
        }
        device.setTypeId(deviceType.getId());
        boolean result = false;
        if (device.getId() == null) {
            //根据唯一主键生成MD5串
            String uniqueStr = SecureUtil.md5(new Snowflake(1L, 1L).nextIdStr());
            device.setUniqueFlag(ENCODER.encode(uniqueStr));
            result = this.save(device);
            if (result == true) {
                return BaseResponse.ok("设备绑定成功", uniqueStr);
            }
        } else {
            result = this.updateById(device);
        }
        return result ? BaseResponse.ok("设备绑定成功") :
                BaseResponse.error("设备绑定失败", Device.class);
    }

}
