package cn.wise.sc.pay.business.merchant.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.merchant.mapper.MerchDiscountMapper;
import cn.wise.sc.pay.business.merchant.service.IMerchDiscountService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchDiscount;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import java.util.List;
/**
 * @description:
 * @author: qh
 * @create: 2020-09-07 09:41
 **/
@Service
public class MerchDiscountServiceImpl extends ServiceImpl<MerchDiscountMapper, MerchDiscount>
        implements IMerchDiscountService {

    @Override
    public BaseResponse<List<MerchDiscount>> getBySellerId(Long seller) {

        if (seller == null || seller == 0L) {
            return BaseResponse.error("seller不能为空!", MerchDiscount.class.getName());
        }

        QueryWrapper<MerchDiscount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("seller_id", seller);

        try {
            List<MerchDiscount> list = this.list(queryWrapper);
            return BaseResponse.ok(list);
        }catch (Exception e){
            return BaseResponse.ok();
        }
    }

    @Override
    public BaseResponse<Boolean> updateMerchDiscount(MerchDiscount merchDiscount) {

        //判断折扣信息是否有效
        String discountStr = merchDiscount.getDiscountStr();
        if (StrUtil.isEmpty(discountStr)) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        if (merchDiscount.getId() == null || merchDiscount.getId() <= 0 ||
                merchDiscount.getSellerId() == null || merchDiscount.getSellerId() == 0) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        try {
            List<MerchDiscount.DiscountSetting> discountSettings =
                    JSON.parseArray(discountStr, MerchDiscount.DiscountSetting.class);
        } catch (Exception ex) {
            return BaseResponse.error("折扣信息配置格式不正确!",
                    MerchDiscount.class.getName());
        }

        boolean flag = this.updateById(merchDiscount);

        if (flag) {
            return BaseResponse.ok(true);
        } else {
            return BaseResponse.error("更新失败", Merchant.class.getName());
        }
    }

    @Override
    public BaseResponse<Boolean> addNewMerchDiscount(MerchDiscount merchDiscount) {

        //判断折扣信息是否有效
        String discountStr = merchDiscount.getDiscountStr();
        if (StrUtil.isEmpty(discountStr)) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        if (merchDiscount.getSellerId() == null || merchDiscount.getSellerId() == 0) {
            return BaseResponse.error("折扣信息不能为空!", MerchDiscount.class.getName());
        }
        QueryWrapper<MerchDiscount> qw = new QueryWrapper<>();
        qw.lambda().eq(MerchDiscount::getSellerId,merchDiscount.getSellerId());
        MerchDiscount one = this.getOne(qw);
        try {
            List<MerchDiscount.DiscountSetting> discountSettings =
                    JSON.parseArray(discountStr, MerchDiscount.DiscountSetting.class);
        } catch (Exception ex) {
            return BaseResponse.error("折扣信息配置格式不正确!",
                    MerchDiscount.class.getName());
        }
        if (one == null){
            //添加新的折扣信息
            boolean save = this.save(merchDiscount);
            if (save){
                return BaseResponse.ok("添加成功",true);
            }else {
                return BaseResponse.error("添加失败!",false);
            }
        }else {
            one.setDiscountStr(merchDiscount.getDiscountStr());
            boolean update = this.updateById(one);
            if (update){
                return BaseResponse.ok("修改成功",true);
            }else {
                return BaseResponse.error("修改失败!",false);
            }
        }
    }

    @Override
    public BaseResponse<Boolean> deleteMerchDiscount(Long id){
        if (id == null){
            return BaseResponse.error("折扣ID不能为空",String.class.getName());
        }

        boolean delete = this.removeById(id);
        if (delete){
            return BaseResponse.ok("删除成功",true);
        }else {
            return BaseResponse.error("删除失败",String.class.getName());
        }

    }

    @Override
    public BaseResponse<Boolean> deleteMerchDiscountBySeller(Long seller) {
        if (seller == 0L){
            return BaseResponse.error("请输入商户id",String.class.getName());
        }
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("seller_id",seller);

        boolean remove = this.remove(queryWrapper);
        if (remove){
            return BaseResponse.ok("删除成功",true);
        }else {
            return BaseResponse.error("删除失败",false);
        }
    }

}
