package cn.wise.sc.pay.business.merchant.service.impl;

import cn.wise.sc.pay.business.merchant.mapper.ProfitAllocationMapper;
import cn.wise.sc.pay.business.merchant.service.IProfitAllocationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.ProfitAllocation;
import cn.wise.sc.pay.domain.merchant.excelvo.ProfitAllocationVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName ProfitAllocationServiceImpl
 * @Author Administrator
 * @Date 2019/12/18
 * @Version V1.0
 **/
@Service
public class ProfitAllocationServiceImpl extends ServiceImpl<ProfitAllocationMapper, ProfitAllocation>
        implements IProfitAllocationService {

    @Autowired
    ProfitAllocationMapper profitAllocationMapper;

    @Override
    public boolean saveProfit(ProfitAllocation profit) {
        boolean result = false;
        if (profit.getId() == null) {
            try {
                result = this.save(profit);
            } catch (Exception ex) {
                log.error("索引冲突!支付方式和店铺唯一!");
                return false;
            }
        } else {
            ProfitAllocation profitRes = this.getById(profit.getId());
            if (profitRes.getMerchId().longValue() != profit.getMerchId().longValue()) {
                return false;
            }
            result = this.updateById(profit);
        }
        return result;
    }

    @Override
    public IPage page(Page page, Long merchId, String payment) {
        if (StringUtils.isBlank(payment)) {
            payment = null;
        }
        return baseMapper.getMerchProfitPage(page, merchId, payment);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveProfits(List<ProfitAllocation> profits) {
        List<ProfitAllocation> batchs = new ArrayList<>();
        List<ProfitAllocation> updateBatchs = new ArrayList<>();
        profits.stream().forEach(arg -> {
            String payment = arg.getPayment();
            Integer cmbRate = arg.getCmbRate();
            Integer aosRate = arg.getAosRate();
            Long merchId = arg.getMerchId();
            if (merchId != null && StringUtils.isNotBlank(payment) && (cmbRate >= 0 && cmbRate <= 1000) &&
                    (aosRate >= 0 && aosRate <= 1000) && (cmbRate + aosRate) <= 1000) {
                if (arg.getId() == null) {
                    arg.setCreateTime(System.currentTimeMillis());
                    arg.setUpdateTime(System.currentTimeMillis());
                    batchs.add(arg);
                } else {
                    arg.setUpdateTime(System.currentTimeMillis());
                    updateBatchs.add(arg);
                }
            }
        });
        if (batchs.size() > 0) {
            try {
                this.saveBatch(batchs);
            }catch (Exception e){
                log.error("索引冲突!profit_allocation表merchant_id和payment索引冲突！");
                return false;
            }
        }
        if (updateBatchs.size() > 0) {
            this.updateBatchById(updateBatchs);
        }
        return true;
    }
}
