package cn.wise.sc.pay.api.order.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.excel.OrderExcel;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @description: 订单断路器
 * @author: qh
 * @create: 2019-12-14 12:06
 **/
@Component
public class IOrderImpl implements IOrdersService {


    @Override
    public OrderResponse createOrder(OrderDetailQuery orderDetailQuery) {
        return OrderResponse.error("服务器繁忙!请稍后再试！");
    }

    @Override
    public OrderResponse getOrders(PageWrapperQuery pageWrapperQuery) {
        error();
        return OrderResponse.ok(new ArrayList<>());
    }

    @Override
    public BaseResponse<List<OrderExcel>> orderInfoExcelByShop(Long shopId, Long start, Long end) {
        error();
        return BaseResponse.ok(new ArrayList<>());
    }

    @Override
    public OrderResponse cancelOrder(String orderNo) {
        error();
        return OrderResponse.error("(O x O)~  小O,走丢了~ so,不能帮您取消订单呢~");
    }

    @Override
    public OrderResponse delOrder(String orderNo, String conditionJson) {
        error();
        return OrderResponse.error("删除订单失败!");
    }

    @Override
    public OrderResponse changeOrderStatus(String orderNo, OrderStatus orderStatus, String conditionJson) {
        error();
        return OrderResponse.error("改变订单状态失败!");
    }

    @Override
    public OrderResponse changeOrderStatusByUserId(String orderNo, String userId, OrderStatus orderStatus, String conditionJson) {
        error();
        return OrderResponse.error("改变订单状态失败!");
    }

    @Override
    public OrderResponse editOrder(OrderDetailQuery orderDetailQuery) {
        error();
        return OrderResponse.error("改变订单失败!");
    }

    @Override
    public BaseResponse<String> getOrderNo() {
        error();
        Snowflake snowflake = new Snowflake(1L, 1L);
        return BaseResponse.ok(snowflake.nextIdStr());
    }

    @Override
    public OrderResponse getOrdersByUserId(String userId, OrderStatus status) {
        error();
        return OrderResponse.ok(new ArrayList<>());
    }

    @Override
    public OrderResponse getOrders(String conditionJson) {
        error();
        return OrderResponse.ok(new ArrayList<>());
    }

    @Override
    public OrderResponse getOneOrder(String conditionJson) {
        error();
        return OrderResponse.error(null);
    }

    @Override
    public BaseResponse<Long> getCount(Long start, Long end, OrderStatus status, String sellerId) {
        error();
        return BaseResponse.ok(-1L);
    }


    @Override
    public OrderResponse page(PageWrapperQuery pageWrapperQuery) {
        error();
        return OrderResponse.error(null);
    }

    @Override
    public BaseResponse<Boolean> updateBatch(List<OrderDetailQuery> ordersCancels) {
        error();
        return BaseResponse.ok(false);
    }

    @Override
    public BaseResponse statisticsByStatus(int strategy, String shopname, long start, long end) {
        error();
        return BaseResponse.error("统计失败!",this.getClass().getName());
    }

    @Override
    public BaseResponse statisticsChart(int strategy, String shopname, long start, long end) {
        error();
        return BaseResponse.error("统计失败!",this.getClass().getName());

    }

    @Override
    public BaseResponse statisticsMoneyByPatMethod(int strategy, String shopname, long start, long end) {
        error();
        return BaseResponse.error("统计失败!",this.getClass().getName());

    }

    @Override
    public BaseResponse statisticsCountByPatMethod(int strategy, String shopname, long start, long end) {
        error();
        return BaseResponse.error("统计失败!",this.getClass().getName());

    }

    @Override
    public BaseResponse statisticsList(int strategy, String shopname, long start, long end, int payment) {
        error();
        return BaseResponse.error("统计失败!",this.getClass().getName());

    }

    @Override
    public BaseResponse<HashMap<String, BigDecimal>> statisticsAggregate(PageWrapperQuery pageWrapperQuery) {
        error();
        return BaseResponse.error("统计失败!",this.getClass().getName());

    }

    @Override
    public BaseResponse statisticsGoodsMarketMoneyGroupSellerGood(PageWrapperQuery pageWrapperQuery) {
        error();
        return BaseResponse.error("统计失败!",this.getClass().getName());

    }

    Logger logger = LoggerFactory.getLogger(this.getClass());

    /**
     * 服务异常
     *
     * @return
     */
    private void error() {

        logger.error("订单服务异常,请检查服务状态!");
    }
}
