package cn.wise.sc.pay.business.order.controller;

import cn.wise.sc.pay.business.order.service.IMerchantPayProService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.order.model.MerchantPayPro;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @description: 招行商户号配置
 * @author: qh
 * @create: 2020-09-24 12:37
 **/
@RestController
@CrossOrigin
@RequestMapping("/pay/setting")
@Slf4j
public class ZHPayMerController {

    @Autowired
    IMerchantPayProService iMerchantPayProService;

    @Autowired
    RedissonClient redissonClient;
    private final String MERCHANT_PAY_INFO_PRE = "MERCHANT:PAY:INFO:";

    @PostMapping("/create")
    public BaseResponse<Boolean> create(@RequestBody MerchantPayPro merchantPayPro) {
        boolean save = iMerchantPayProService.save(merchantPayPro);
        String key = MERCHANT_PAY_INFO_PRE + merchantPayPro.getSellerId();
        RBucket<Object> bucket = redissonClient.getBucket(key);
        bucket.delete();
        if (save) {
            return BaseResponse.ok(true);
        } else {
            return BaseResponse.error("请检测必填元素!", MerchantPayPro.class.getName());
        }
    }

    @DeleteMapping("/{id}")
    public BaseResponse<Boolean> delete(@PathVariable("id") Long id) {

        boolean removeById = iMerchantPayProService.removeById(id);
        if (removeById) {
            return BaseResponse.ok(true);
        } else {
            return BaseResponse.error("删除失败!", MerchantPayPro.class.getName());
        }
    }


    @GetMapping("/{sellerId}")
    public BaseResponse<MerchantPayPro> getBySeller(@PathVariable("sellerId") Long sellerId) {
        MerchantPayPro bySellerId = iMerchantPayProService.getBySellerId(sellerId + "");
        if (bySellerId == null) {
            return BaseResponse.error("没找到!", this.getClass().getName());
        }
        return BaseResponse.ok(bySellerId);
    }
}
