package cn.wise.sc.pay.business.order.model;

import cn.wise.sc.payment.core.model.BaseModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * @ClassName WxPayParams
 * @Description :微信统一下单参数
 * @Author Administrator
 * @Date 2019/12/10
 * @Version V1.0
 **/
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class WxPayParams extends BaseModel {
    /**
     * 版本号
     */
    private String version;
    /**
     * 编码方式
     */
    private String encoding;
    /**
     * 签名
     */
    private String sign;
    /**
     * 签名方法 01：RSA2
     */
    private String signMethod;
    /**
     * 商户号 商户在招行完成商户进件后分配的招行商户号
     */
    private String merId;
    /**
     * 设备号 PC网页或公众号内支付请传"WEB"
     */
    private String deviceInfo;
    /**
     * 子公众账号ID      商户微信公众号appid，app支付时,为在微信开放平台上申请的APPID
     *                  小程序支付：当前调起支付的小程序APPID
     *                  APP支付：子商户在微信开放平台上申请的APPID
     *                  公众号支付：公众号APPID
     */
    private String subAppId;
    /**
     * 商户订单号 商户端生成，要求此订单号在整个商户下唯一
     */
    private String orderId;
    /**
     * 交易类型         APP支付：APP
     *                  H5支付：MWEB
     *                  公众号支付：JSAPI
     *                  小程序支付：JSAPI
     */
    private String tradeType;
    /**
     * 交易场景         OFFLINE:线下场景
     *                  ONLINE：线上场景（APP支付时填写）
     *                  INSURANCE：保险
     *                  CHARITY：公益
     *                  trade_type=JSAPI，此参数必传（保险，公益除外）。商户需上送正确的取值，否则交易有被微信拒绝的风险。
     */
    private String tradeScene;
    /**
     * 收银员      招行系统生成的收银员，每个收银员均会对应到一个门店。若商户无门店、收银员概念则上送默认的虚拟收银员即可。
     */
    private String userId;
    /**
     * 商品描述  商品描述交易字段格式根据不同的应用场景按照以下格式：
     *          APP支付：需传入应用市场上的APP名字-实际商品名称，天天爱消除-游戏充值；
     *          H5支付：应用在浏览器网页上的场景，传入浏览器打开的移动网页的主页title名-实际商品名称，例如：腾讯充值中心-QQ会员充值；
     *          公众号支付：传入公众号名称-实际商品名称，例如：腾讯形象店- image-QQ公仔；
     *          小程序支付：传入商家名称-销售商品类目，如：腾讯-游戏；
     */
    private String body;
    /**
     * 商品详情
     */
    private String goodsDetail;
    /**
     * 商品标记 商品标记，代金券或立减优惠功能的参数，详见微信官网文档中的代金券或立减优惠说明。
     */
    private String goodsTag;
    /**
     * 附加数据
     */
    private String attach;
    /**
     * 商户保留域 用于传输商户自定义数据，在支付结果查询和交易结果通知时原样返回
     */
    private String mchReserved;
    /**
     * 支付有效 支付数据的有效时间，单位为秒，应不小于60
     */
    private String payValidTime;
    /**
     * 交易通知地址 支付结果将送到本通知
     */
    private String notifyUrl;
    /**
     * 交易金额 单位为分
     */
    private String txnAmt;
    /**
     * 交易币种 默认156，目前只支持人民币（156）
     */
    private String currencyCode;
    /**
     * 限定支付方式       no_credit--指定不能使用信用卡支付；
     *                  为空表示无限制
     */
    private String limitPay;
    /**
     * 终端IP 必须传正确的用户端IP
     */
    private String spbillCreateIp;
    /**
     * 用户标识         trade_type=JSAPI，此参数必传，用户在商户appid下的唯一标识。
     *                  openid如何获取，可参考【获取openid】。企业号请使用【企业号OAuth2.0接口】获取企业号内成员userid，
     *                  再调用【企业号userid转openid接口】进行转换
     */
    private String openId;
    /**
     * 用户子标识       trade_type=JSAPI，此参数必传，用户在子商户appid下的唯一标识。
     *                  openid和sub_openid可以选传其中之一，如果选择传sub_openid,则必须传sub_appid。
     *                  下单前需要调用【网页授权获取用户信息】接口获取到用户的Openid
     */
    private String subOpenId;
    /**
     * 场景信息         H5支付支付必填，公众号、APP支付和小程序支付不填
     *                    1，IOS移动应用
     *                    {"h5_info": //h5支付固定传"h5_info"
     *                    {"type": "",  //场景类型
     *                    "app_name": "",  //应用名
     *                    "bundle_id": ""  //bundle_id
     *                    }
     *                    }
     *                    2，安卓移动应用
     *                    {"h5_info": //h5支付固定传"h5_info"
     *                    {"type": "",  //场景类型
     *                    "app_name": "",  //应用名
     *                    "package_name": ""  //包名
     *                   }
     *                    }
     *                    3，WAP网站应用
     *                    {"h5_info": //h5支付固定传"h5_info"
     *                    {"type": "",  //场景类型
     *                    "wap_url": "",//WAP网站URL地址
     *                    "wap_name": ""  //WAP 网站名
     *                    }
     *                    }
     */
    private String sceneInfo;
    /**
     * 实名支付信息       实名支付信息
     *                  {"need_check": "T", //是否校验身份，T是F否
     *                  "name": "",  //名字
     *                  "number": "",  //证件号
     *                  "mobile": "",  //手机号
     *                  "type": "IDCARD" //证件类型，只支持身份证
     *                  }
     */
    private String identity;
    /**
     * 保单单号         保险实名支付对接中保信使用，由保险公司生成，须与实名认证上送中保信的“支付单号”一致。
     *                  如果交易有上送此字段则交易成功后会通知中保信做见费出单操作。
     *                  该字段和region需同时出现或同时不出现。
     */
    private String policyNo;
    /**
     * 地区码          缴税系统地区码按指引填写
     *                  全国： 000000
     *                  上海地区： 310000
     *                  深圳地区： 440300
     *                  该字段和policyNo需同时出现或同时不出现。
     */
    private String region;

}
