package cn.wise.sc.pay.business.order.service;

import cn.wise.sc.pay.business.order.configure.WxPayConfigure;
import cn.wise.sc.pay.business.order.model.ApplyRefundParams;
import cn.wise.sc.pay.business.order.model.BaseResult;
import cn.wise.sc.pay.business.order.model.BillUrlParams;
import cn.wise.sc.pay.business.order.model.CloseOrderParams;
import cn.wise.sc.pay.domain.order.model.MerchantPayPro;
import cn.wise.sc.pay.business.order.model.PayRefundNoticeParams;
import cn.wise.sc.pay.business.order.model.PayRefundQueryParams;
import cn.wise.sc.pay.business.order.model.PayResultNoticeParams;
import cn.wise.sc.pay.business.order.model.PayResultParams;
import cn.wise.sc.pay.business.order.model.ReceivingQRParam;
import cn.wise.sc.pay.business.order.model.ResultDataCMB;
import cn.wise.sc.pay.business.order.model.ServicePayParams;
import cn.wise.sc.pay.business.order.model.SetSecretKeyParams;
import cn.wise.sc.pay.business.order.model.WxPayParams;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.payment.core.model.BaseModel;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName WxPayService
 * @Author Administrator
 * @Date 2019/12/11
 * @Version V1.0
 **/
@Service
@Slf4j
public class WxPayService {


    private final String WX_PAY = "https://api.cmbchina.com/polypay/v1.0/mchorders/onlinepay";
    private final String REFUND_QUERY = "https://api.cmbchina.com/polypay/v1.0/mchorders/refundquery";
    private final String APPLY_REFUND = "https://api.cmbchina.com/polypay/v1.0/mchorders/refund";
    private final String CLOSE_ORDER = "https://api.cmbchina.com/polypay/v1.0/mchorders/close";
    private final String BILL_URL = "https://api.cmbchina.com/polypay/v1.0/mchorders/statementurl";
    private final String KEY_SET = "https://api.cmbchina.com/polypay/v1.0/mchkey/keyset";
    private final String ALI_PAY = "https://api.cmbchina.com/polypay/v1.0/mchorders/servpay";
    private final String PAY_QUERY = "https://api.cmbchina.com/polypay/v1.0/mchorders/orderquery";
    private final String WX_PAY_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/onlinepay";
    private final String REFUND_QUERY_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/refundquery";
    private final String APPLY_REFUND_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/refund";
    private final String CLOSE_ORDER_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/close";
    private final String BILL_URL_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/statementurl";
    private final String KEY_SET_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchkey/keyset";
    private final String ALI_PAY_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/servpay";
    private final String PAY_QUERY_TEST = "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/orderquery";
    @Autowired
    WxPayConfigure wxPayConfigure;
    @Autowired
    MerchantPayService merchantPayService;

    /**
     * 招行订单号和商户订单号是否都为空
     *
     * @param origOrderId
     * @param origCmbOrderId
     * @return
     */
    private Boolean orderIdsBank(String origOrderId, String origCmbOrderId) {
        return StringUtils.isBlank(origCmbOrderId) && StringUtils.isBlank(origOrderId);
    }

    /**
     * 验证外层 sign
     *
     * @param result
     * @return
     */
    private BaseResponse checkOutSign(BaseResult result) {

        boolean checkResult = wxPayConfigure.parseResultData(result);
        BaseResponse baseResponse;
        if (checkResult) {
            String returnCode = result.getRespCode();
            String respCode = result.getRespCode();
            if ("SUCCESS".equals(returnCode) && "SUCCESS".equals(respCode)) {
                String payData = result.getBiz_content();
                baseResponse = BaseResponse.ok(payData);
                baseResponse.setClassName("聚合支付.class");
                return baseResponse;
            }
        }
        baseResponse = new BaseResponse();
        baseResponse.setCode(400);
        baseResponse.setMsg(result.getRespMsg());
        baseResponse.setClassName("招行聚合支付.class");
        baseResponse.setData(null);
        return baseResponse;

    }


    /**
     * 加签发起请求
     *
     * @param params
     * @param url
     * @return
     */
    private String request(BaseModel params, String url) {

        String jsonObj = JSON.toJSONString(params.toMap());
        Map map = new HashMap(16);
        map.put("biz_content", jsonObj);
        String dataSign = wxPayConfigure.initDataSign(map, params);
        map.put("sign", dataSign);

        //报文头apiSign签名 header
        Map apiMap = new HashMap(16);
        apiMap.put("sign", dataSign);
        HttpHeaders requestHeaders = wxPayConfigure.initHeaders(apiMap, params.getCmbAppId(), params.getSecretKey());
        RestTemplate template = new RestTemplate();
        HttpEntity<String> requestEntity = new HttpEntity<>(JSON.toJSONString(map), requestHeaders);
        log.info("=============================请求参数================");
        log.info(JSON.toJSONString(requestEntity));
        log.info("============================参数结束==========================");
        ResponseEntity<String> response =
                template.exchange(url,
                        HttpMethod.POST, requestEntity, String.class);

        return response.getBody();
    }

    /**
     * 加签发起请求 并验签
     *
     * @param params
     * @param url
     * @return
     */
    private BaseResponse requestAndcheckOutSign(BaseModel params, String url) {

        String response = request(params, url);

        BaseResult applyRefundCMB = JSON.parseObject(response, BaseResult.class);

        return checkOutSign(applyRefundCMB);
    }

    /**
     * 验证参数是否为空
     *
     * @param params
     * @return
     */
    private boolean paramIsBlank(String... params) {
        for (int i = 0; i < params.length; i++) {
            if (StringUtils.isBlank(params[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * 统一下单
     *
     * @return
     */
    public BaseResponse wxPay(String requestIp, String orderId, String txnAmt,
                              String openId, String body, String sellerId) {
        if (StringUtils.isBlank(orderId) || StringUtils.isBlank(openId)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行聚合支付");
        }
        if (!StringUtils.isNumeric(txnAmt)) {
            return BaseResponse.error("订单金额值有误，无法发起支付", "招行聚合支付");
        }

        //bizContent json字符串
        MerchantPayPro merchantPayPro = merchantPayService.getBySellerId(sellerId);
        WxPayParams params = wxPayConfigure.initWxPayParam(requestIp, orderId, txnAmt, openId, body);
        params.setMerId(merchantPayPro.getMerId());
        params.setUserId(merchantPayPro.getCashier());
        params.setPrivateKey(merchantPayPro.getMerchantPrivateKey());
        params.setCmbAppId(merchantPayPro.getCmbAppId());
        params.setSecretKey(merchantPayPro.getSecretKey());

        String response = request(params, WX_PAY);
        ResultDataCMB resultDataCMB = JSON.parseObject(response, ResultDataCMB.class);

        return checkOutSign(resultDataCMB);

    }

    /**
     * 支付结果通知
     */
    @Deprecated
    public BaseResponse payResultNotice(String cmbOrderId, String orderId,
                                        String txnAmt, String dscAmt,
                                        String payType, String txnTime) {

        if (paramIsBlank(cmbOrderId, orderId, txnAmt, dscAmt, payType, txnTime)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行聚合支付");
        }

        PayResultNoticeParams params = wxPayConfigure.initPayResultNoticeParams(cmbOrderId,
                orderId, txnAmt, dscAmt, payType, txnTime);

        return requestAndcheckOutSign(params, "https://api.cmburl.cn:8065/polypay/v1.0/mchorders/close");
    }

    /**
     * 退款主动查询
     *
     * @param orderId
     * @param origOrderId
     * @return
     */
    public BaseResponse payRefundQuery(String orderId, String origOrderId, String sellerId) {

        if (orderIdsBank(origOrderId, orderId)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行聚合支付");
        }
        MerchantPayPro merchantPayPro = merchantPayService.getBySellerId(sellerId);
        PayRefundQueryParams params = wxPayConfigure.initPayRefundParams(orderId, origOrderId);
        params.setMerId(merchantPayPro.getMerId());
        params.setUserId(merchantPayPro.getCashier());
        params.setPrivateKey(merchantPayPro.getMerchantPrivateKey());
        params.setCmbAppId(merchantPayPro.getCmbAppId());
        params.setSecretKey(merchantPayPro.getSecretKey());
        return requestAndcheckOutSign(params, REFUND_QUERY);
    }

    /**
     * 申请退款
     */
    public BaseResponse applyForRefund(String orderId, String origOrderId,
                                       String txnAmt, String refundAmt,
                                       String refundReason, String sellerId) {

        if (paramIsBlank(orderId, txnAmt, refundAmt, origOrderId)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行聚合支付");
        }
        MerchantPayPro merchantPayPro = merchantPayService.getBySellerId(sellerId);
        ApplyRefundParams params = wxPayConfigure.initApplyRefundParams(orderId, origOrderId,
                txnAmt, refundAmt, refundReason);
        params.setMerId(merchantPayPro.getMerId());
        params.setUserId(merchantPayPro.getCashier());
        params.setPrivateKey(merchantPayPro.getMerchantPrivateKey());
        params.setCmbAppId(merchantPayPro.getCmbAppId());
        params.setSecretKey(merchantPayPro.getSecretKey());
        return requestAndcheckOutSign(params, APPLY_REFUND);
    }

    /**
     * 退款通知
     */
    @Deprecated
    public BaseResponse payRefundNotice(String orderId, String cmbOrderId, String refundAmt, String refundDscAmt, String couponInfo, String payType, String txnTime, String endDate, String endTime, String mchReserved) {

        if (paramIsBlank(refundAmt, refundDscAmt, payType, txnTime) || orderIdsBank(orderId, cmbOrderId)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行支付");
        }

        PayRefundNoticeParams params = wxPayConfigure.initPayRefundNoticeParams(orderId, cmbOrderId, refundAmt, refundDscAmt,
                couponInfo, payType, txnTime, endDate, endTime, mchReserved);
        return requestAndcheckOutSign(params, "");
    }

    /**
     * 关闭订单
     *
     * @param origOrderId
     * @param origCmbOrderId
     * @return
     */
    public BaseResponse closeOrder(String origOrderId, String origCmbOrderId, String sellerId) {
        if (orderIdsBank(origOrderId, origCmbOrderId)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行支付");
        }
        MerchantPayPro merchantPayPro = merchantPayService.getBySellerId(sellerId);
        CloseOrderParams params = wxPayConfigure.initCloseOrderParams(origOrderId, origCmbOrderId);
        params.setMerId(merchantPayPro.getMerId());
        params.setUserId(merchantPayPro.getCashier());
        params.setPrivateKey(merchantPayPro.getMerchantPrivateKey());
        params.setCmbAppId(merchantPayPro.getCmbAppId());
        params.setSecretKey(merchantPayPro.getSecretKey());
        return requestAndcheckOutSign(params, CLOSE_ORDER);
    }


    /**
     * 获取对账地址
     *
     * @param billDate
     * @return
     */
    public BaseResponse getBillUrl(String billDate, String sellerId) {

        if (paramIsBlank(billDate)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行支付");
        }
        MerchantPayPro merchantPayPro = merchantPayService.getBySellerId(sellerId);
        BillUrlParams params = wxPayConfigure.initBillUrlParams(billDate);
        params.setMerId(merchantPayPro.getMerId());
        params.setPrivateKey(merchantPayPro.getMerchantPrivateKey());
        params.setCmbAppId(merchantPayPro.getCmbAppId());
        params.setSecretKey(merchantPayPro.getSecretKey());
        return requestAndcheckOutSign(params, BILL_URL);
    }


    /**
     * 设置密钥
     *
     * @return
     */
    public BaseResponse setSecretKey(String platformId, String digEvpEncMth, String digEvp, String encryptMethod,
                                     String encryptInfo, String keyTpye, String keyLen, String mchKey, String updateUrl) {
        if (paramIsBlank(platformId, digEvp, digEvpEncMth, encryptInfo, encryptMethod, keyLen, keyTpye, mchKey)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行支付");
        }
        SetSecretKeyParams params = wxPayConfigure.initSetSecretKeyParams(platformId, digEvpEncMth, digEvp, encryptMethod,
                encryptInfo, keyTpye, keyLen, mchKey, updateUrl);
        return requestAndcheckOutSign(params, KEY_SET);
    }

    /**
     * 服务窗口支付
     *
     * @return
     */
    public BaseResponse servicePay(String orderId, String txnAmt, String buyerId,
                                   String mchReserved, String sellerId) {

        MerchantPayPro merchantPayPro = merchantPayService.getBySellerId(sellerId);

        if (paramIsBlank(orderId, txnAmt, buyerId)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行支付");
        }
        ServicePayParams params = wxPayConfigure.initServicePayParams(txnAmt, orderId, buyerId, mchReserved);
        params.setMerId(merchantPayPro.getMerId());
        params.setUserId(merchantPayPro.getCashier());
        params.setPrivateKey(merchantPayPro.getMerchantPrivateKey());
        params.setCmbAppId(merchantPayPro.getCmbAppId());
        params.setSecretKey(merchantPayPro.getSecretKey());
        return requestAndcheckOutSign(params, ALI_PAY);
    }

    /**
     * 支付结果查询
     *
     * @param cmbOrderId
     * @param orderId
     * @return
     */
    public BaseResponse payResultQuery(String cmbOrderId, String orderId, String sellerId) {
        if (orderIdsBank(cmbOrderId, orderId)) {
            return BaseResponse.error("信息有误，无法发起支付", "招行聚合支付");
        }
        MerchantPayPro merchantPayPro = merchantPayService.getBySellerId(sellerId);
        PayResultParams params = wxPayConfigure.initPayResultQuery(orderId);
        params.setMerId(merchantPayPro.getMerId());
        params.setUserId(merchantPayPro.getCashier());
        params.setPrivateKey(merchantPayPro.getMerchantPrivateKey());
        params.setCmbAppId(merchantPayPro.getCmbAppId());
        params.setSecretKey(merchantPayPro.getSecretKey());
        return requestAndcheckOutSign(params, PAY_QUERY);
    }


    /**
     * 收款码申请
     *
     * @param txnAmt  金额 分
     * @param orderId 订单号
     * @param body    保留域
     * @return 二维码
     */
    public BaseResponse applyPayQR(String txnAmt, String orderId, String body) {
        if (paramIsBlank(txnAmt, orderId)) {
            return BaseResponse.error("信息有误，无法获取收款二维码", "招行聚合支付");
        }
        ReceivingQRParam receivingQRParam = wxPayConfigure.initReceivingQRParam(txnAmt, orderId, body);
        return requestAndcheckOutSign(receivingQRParam, "https://api.cmbchina.com/polypay/v1.0/mchorders/qrcodeapply");
    }

}