package cn.wise.sc.pay.business.order.utils;

import cn.hutool.core.util.CharsetUtil;
import cn.hutool.http.HttpException;
import com.alibaba.fastjson.JSON;
import net.coobird.thumbnailator.Thumbnails;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

/**
 * @description: ugc图文内容验证
 * @author: qh
 * @create: 2020-06-16 13:49
 **/
public class UgcContentUtil {

    static final String imgSecCheckUrl = "https://api.weixin.qq.com/wxa/img_sec_check?access_token=";
    static final String msgSecCheckUrl = "https://api.weixin.qq.com/wxa/msg_sec_check?access_token=";

    /**
     * 验证内容
     *  todo  问题: 招行的云创服务器配置负载均衡器会全部验证通过,
     *              使用集群单独提供服务。
     * @param content 文章
     * @return
     * @throws HttpException
     */
    public static boolean checkMsg(String content, String accessToken) throws HttpException {

        String url = msgSecCheckUrl + accessToken;
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPost httpPost = new HttpPost(url);
        CloseableHttpResponse response = null;
        httpPost.addHeader("Content-Type", "application/json");
        Map map = new HashMap<>();
        map.put("content",content);
        String body = com.alibaba.fastjson.JSONObject.toJSONString(map);
        httpPost.setEntity(new StringEntity(body,ContentType.create("text/json", "UTF-8")));
        //读取响应流
        try {
            response = httpclient.execute(httpPost);
            HttpEntity entity = response.getEntity();
            String result = EntityUtils.toString(entity, CharsetUtil.UTF_8);
            EntityUtils.consume(entity);
            WxContentRsp wxContentRsp = JSON.parseObject(result, WxContentRsp.class);
            if ("0".equals(wxContentRsp.getErrcode())) {
                return true;
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            try {
                httpclient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    private static String getContent(CloseableHttpResponse response) {
        // 获取响应实体
        HttpEntity entity = response.getEntity();
        String content = "";
        try {
            // 用string接收响应实体
            content = EntityUtils.toString(entity, "UTF_8");
            // 消耗响应实体，并关闭相关资源占用
            EntityUtils.consume(entity);
            if (response != null) {
                response.close();
            }
        } catch (Exception ex) {
            return "";
        }
        return content;
    }

    /**
     * 根据url路径验证图片是否合法
     *
     * @param imgUrl 待验证图片地址
     * @return 成功否
     * @throws HttpException
     */
    public static boolean checkImgByImgUrl(String imgUrl, String accessToken) throws HttpException {

        InputStream inputStream = image2InputStream(imgUrl);
        //图片验证

        if (inputStream == null) {
            return false;
        }
        String url = imgSecCheckUrl + accessToken;
        HttpClient httpclient = HttpClients.createDefault();

        HttpPost request = new HttpPost(url);
        request.addHeader("Content-Type", "application/octet-stream");

        try {
            byte[] byt = new byte[inputStream.available()];
            inputStream.read(byt);
            request.setEntity(new ByteArrayEntity(byt, ContentType.create("image/jpg")));
            HttpResponse response = httpclient.execute(request);
            HttpEntity entity = response.getEntity();
            // 转成string
            String result = EntityUtils.toString(entity, "UTF-8");
            WxContentRsp wxContentRsp = JSON.parseObject(result, WxContentRsp.class);
            if ("0".equals(wxContentRsp.getErrcode())) {
                return true;
            }
            return false;
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return false;
    }

    /**
     * @param connection
     * @param num
     * @return
     * @throws IOException
     */
    public InputStream compressImage(URLConnection connection, long num) throws IOException {
        if (num == 0 || connection == null) {
            return null;
        }
        double multiple = 1.0 / (num / (1024 * 1024) + 1.0);
        BufferedImage image = Thumbnails.of(connection.getInputStream()).scale(multiple).asBufferedImage();
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        ImageIO.write(image, "jpg", os);
        InputStream is = new ByteArrayInputStream(os.toByteArray());
        return is;
    }

    /**
     * 通过图片的url获取图片的base64字符串
     *
     * @param imgUrl 图片url
     * @return 返回图片base64的字符串
     */
    public static InputStream image2InputStream(String imgUrl) {

        URL url = null;
        InputStream is = null;
        ByteArrayOutputStream outStream = null;
        HttpURLConnection httpUrl = null;
        try {
            url = new URL(imgUrl);
            httpUrl = (HttpURLConnection) url.openConnection();
            httpUrl.connect();
            return httpUrl.getInputStream();
        } catch (Exception ex) {
            return null;
        }
    }



}

/**
 * 微信图片认证  响应对象
 */
class WxContentRsp {
    private String errcode;
    private String errmsg;

    public String getErrcode() {
        return errcode;
    }

    public void setErrcode(String errcode) {
        this.errcode = errcode;
    }

    public String getErrmsg() {
        return errmsg;
    }

    public void setErrmsg(String errmsg) {
        this.errmsg = errmsg;
    }
}

