package cn.wise.sc.pay.domain.order.query;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.validation.annotation.Validated;

import javax.validation.constraints.NotNull;

/**
 * 退款请求描述
 *
 * @author qh
 */
@Data
@ApiModel
@Validated
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class SafeguardDetailQuery {

    @NotNull(message = "维权订单号不能为空!")
    private String orderNo;

    @ApiModelProperty("退款类型 0:钱  1:人和钱我都要")
    private Integer refundType;

    @ApiModelProperty("退款原因")
    private String reason;

    @ApiModelProperty("原因图片")
    private String safeguardUrl;

    /**
     * @see cn.wise.sc.pay.domain.order.SafeguardUtil
     */
    @ApiModelProperty("(0~8代表含有: \n" +
            "     * 退款状态 APPLY,\n" +
            "     * 审核通过 AUDIT_YES,\n" +
            "     * 审核不通过  AUDIT_NO,\n" +
            "     * 全部入库   STORAGE_ALL,\n" +
            "     * 部分入库  STORAGE_PORTION,\n" +
            "     * 取消申请售后 APPLY_CANCEL,\n" +
            "     * 退款中  REFUND,\n" +
            "     * 退款失败  REFUND_FAIL,\n" +
            "     * 售后完成 REFUND_COMPLETE\n")
    private Integer status;

    @ApiModelProperty("退款订单号")
    private String refundNo;
}
