package cn.wise.sc.payment.cmb.controller;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.mq.client.dto.TransactionMessage;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.payment.cmb.model.PayResult;
import cn.wise.sc.payment.cmb.model.cmbnoenet.TokenInfoQuery;
import cn.wise.sc.payment.cmb.service.CMBOneNetPayService;
import cn.wise.sc.payment.cmb.service.Producer;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * @description: 招行一卡通
 * @author: qh
 * @create: 2020-04-09 17:05
 */
@Slf4j
@CrossOrigin
@RestController
@RequestMapping("/cmbOneNet")
public class CMBOneNetController {

    final
    IMerchantProfitService merchantProfitService;
    final
    IOrdersService iOrdersService;
    final
    Producer producer;
    final CMBOneNetPayService CMBOneNetPayService;

    public CMBOneNetController(CMBOneNetPayService CMBOneNetPayService, IOrdersService iOrdersService, Producer producer, IMerchantProfitService merchantProfitService) {
        this.CMBOneNetPayService = CMBOneNetPayService;
        this.iOrdersService = iOrdersService;
        this.producer = producer;
        this.merchantProfitService = merchantProfitService;
    }

    @ApiOperation("获取招行app支付订单信息")
    @PostMapping("/fmordsubOrderInfo")
    public BaseResponse fmordsubOrderInfo(
                String orderNo, String txnAmt, String receAccount, String receName, String receBank, String body)
            throws Exception {
        return CMBOneNetPayService.oneNetOrderInfo(orderNo, txnAmt, receAccount, receName, receBank, body);
    }

    @ApiOperation("订单支付回调")
    @PostMapping("/payNotice")
    public BaseResponse payNotice(HttpServletRequest request) {

        //获取订单信息
        String orderNo = request.getParameter("reford");
        String orderQueryStatus = request.getParameter("ordsta");
        String orderRtsStatus = request.getParameter("ordsts");
        String reserve = request.getParameter("detinf");
        //如果订单请求和订单处理结果都是D 表示成功
        if ("D".equals(orderQueryStatus) && "D".equals(orderRtsStatus)) {
            BaseResponse response = changeOrderStatusAfterPay(orderNo, null);
            PayResult payResult = new PayResult();
            log.info("================  afterSuccess()方法  ==============");
            log.info(JSON.toJSONString(response));
            log.info("================  afterSuccess()方法  ==============");

            if (response.getCode() == 200) {
                payResult.setReturnCode("SUCCESS");
                payResult.setRespCode("SUCCESS");
                Map map = new HashMap<>();
                map.put("order_no", orderNo);
                OrderResponse orderResponse = iOrdersService.getOneOrder(JSON.toJSONString(map));
                Orders orders = orderResponse.getData();
                // 佣金推送
                if (orders != null && orderResponse.getCode() == 200) {
                    System.out.println(orderNo + "订单状态改变：" + orderResponse.code + "|" + orders.getOrderStatus());
                    if (OrderStatus.WAIT_COMMENT.name().equals(orders.getOrderStatus())) {
                        merchantProfitService.addMoneyData(orders.getSellerId(),
                                orders.getId(), orders.getPaymentMethodName(), orders.getPayPrice(), orders.getCreateTime());
                    }
                }

                //推送小票消息
                pushReceiptMsg(orders, reserve);
                //如果订单状态支付完成  向消费记录mq发消息
                orderCompleteEvent(orders);
                //推送积分增加信息
                pushPointsAndExpMsg(orders);
            } else {
                payResult.setRespCode("FAIL");
                payResult.setRespMsg("当前订单已经完成支付");
            }
            log.info(JSON.toJSONString(payResult));
            return BaseResponse.ok();
        }

        return BaseResponse.okMsg("招行一卡通支付回调!", CMBOneNetPayService.getClass());
    }

    @ApiOperation("解析握手令牌token,获取招行App用户信息")
    @PostMapping("/getUserInfo")
    public BaseResponse getUserInfo(@RequestBody TokenInfoQuery tokenInfoQuery) {

        return CMBOneNetPayService.getUserInfo(tokenInfoQuery.getSignature(), tokenInfoQuery.getData());
    }

    @ApiOperation("申请Api访问令牌")
    @PostMapping("/applyInfo")
    public BaseResponse getApplyInfo(@RequestBody String authCode) {
        return CMBOneNetPayService.getApplyInfo(authCode);
    }

    @ApiOperation("续费Api访问令牌")
    @PutMapping("/reNewApplyInfo")
    public BaseResponse reNewApplyInfo(@RequestBody String token) {
        return CMBOneNetPayService.renewApplyInfo(token);
    }

    @ApiOperation("获取用户手机号")
    @PutMapping("/userPhoneAndPosition")
    public BaseResponse getUserPhone(String userId, String token) {
        return CMBOneNetPayService.getUserPhone(userId, token);
    }

    /**
     * 推送积分和经验增长消息
     *
     * @param order
     */
    private void pushPointsAndExpMsg(Orders order) {
        TransactionMessage transactionMessage = new TransactionMessage();
        transactionMessage.setMessage(JSON.toJSONString(order));
        transactionMessage.setQueue("pointAndExpQue");
        transactionMessage.setRouteKey("pointAndExpQue");
        try {
            log.info("积分和经验增加 msg :{}", JSON.toJSONString(transactionMessage));
            mqpost(transactionMessage);
        } catch (IOException e) {
            log.error("积分和经验增长msg  推送失败了哒!");
            e.printStackTrace();
        }
    }

    /**
     * 订单完成时 发布事件
     */
    private void orderCompleteEvent(Orders orders) {
        if (orders == null) {
            throw new RuntimeException();
        }
        String msg = JSON.toJSONString(orders);
        log.info("===================消费记录mq推送======================");
        log.info("======msg: {}", msg);
        producer.send("create_xpense", msg);
    }

    /**
     * 支付成功后修改订单状态
     *
     * @param orderNo 订单号
     * @return
     */
    private BaseResponse changeOrderStatusAfterPay(String orderNo, String unifiedOrderNo) {

        //获取订单
        Map condition = new HashMap(2);
        condition.put("order_status", OrderStatus.CONFIRM);
        condition.put("order_no", orderNo);
        Orders orders = iOrdersService.getOneOrder(JSON.toJSONString(condition)).getData();
        //修改订单状态
        if (orders != null) {
            OrderDetailQuery orderDetailQuery = OrderDetailQuery.builder()
                    .orderNo(orderNo)
                    .unifiedOrderNo(unifiedOrderNo)
                    .payStatus(PayStatus.PAY_YES.name())
                    .build();
            //修改订单状态  网上商城 --> 支付完 --> 待收货
            if ("ESHOP".equals(orders.getPaas())) {
                orderDetailQuery.setStatus(OrderStatus.WAIT_DELIVERING.name());
            } else {
                //收银端和展览支付完成 --> 待评价
                orderDetailQuery.setStatus(OrderStatus.WAIT_COMMENT.name());
            }
            return iOrdersService.editOrder(orderDetailQuery);
        } else {
            return BaseResponse.error("支付成功后订单状态改变失败!为获取到符合规则的订单", "Cashiers.class");
        }
    }

    public OrderResponse mqpost(@RequestBody TransactionMessage msg) throws IOException {
        /*TransactionMessage msg = new TransactionMessage();
        msg.setMessage(message); //发送的信息
        msg.setRouteKey(SN); //sn号
        msg.setQueue(SN);    //sn号
        msg.setSendSystem(system);//发消息系统
        */
        msg.setCreateTime(LocalDateTime.now());
        producer.createQueue(msg.getQueue());
        System.out.println(msg.getQueue());
        System.out.println(msg.getRouteKey());
        producer.send(msg.getRouteKey(), JSON.toJSONString(msg));
        //String msgs = (String) rabbitTemplate.receiveAndConvert("test1",2000l);
        //String transactionMessage = (String) rabbitTemplate.receiveAndConvert("test1");

        return OrderResponse.ok("发送成功");
    }

    private void pushReceiptMsg(Orders orders, String msgInfo) {
        if (StrUtil.isEmpty(msgInfo)) {
            log.error("打印小票的消息失败！预保留域为空！");
        }
        //收银系统 打印小票
        if ("cashier".equals(orders.getPaas())) {
            String[] strs = msgInfo.split("\\|");
            if (strs.length >= 2) {
                String msg = strs[strs.length - 3];
                String sn = strs[strs.length - 2];
                TransactionMessage transactionMessage = new TransactionMessage();
                transactionMessage.setMessage(msg);
                transactionMessage.setQueue(sn);
                transactionMessage.setRouteKey(sn);
                try {
                    log.info("打印小票 msg :{}", JSON.toJSONString(transactionMessage));
                    mqpost(transactionMessage);
                } catch (IOException e) {
                    log.error("收银端打印小票,mq消息推送失败!");
                    e.printStackTrace();
                }
            }
        }
    }
}
