package cn.wise.sc.payment.cmb.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.order.service.ISafeguardService;
import cn.wise.sc.pay.business.goods.service.ISpcDetailService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.mq.client.dto.TransactionMessage;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.model.Safeguard;
import cn.wise.sc.pay.domain.order.model.SafeguardStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.domain.order.query.SafeguardDetailQuery;
import cn.wise.sc.payment.ali.AlipayUgcUtil;
import cn.wise.sc.payment.cmb.configure.WX_Util;
import cn.wise.sc.payment.cmb.model.BizContent;
import cn.wise.sc.payment.cmb.model.PayResult;
import cn.wise.sc.payment.cmb.model.WxContentQuery;
import cn.wise.sc.payment.cmb.model.WxUser;
import cn.wise.sc.payment.cmb.model.zh.PayParamQuery;
import cn.wise.sc.payment.cmb.service.Producer;
import cn.wise.sc.payment.cmb.service.WeiXinDaoImpl;
import cn.wise.sc.payment.cmb.service.WxPayService;
import cn.wise.sc.payment.cmb.utils.UgcContentUtil;
import cn.wise.sc.payment.cmb.weichat.model.OAuthAccessToken;
import cn.wise.sc.payment.core.kit.IpKit;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * @author neo.shu
 * @since 2019/10/08 09:09
 */
@RestController
@CrossOrigin
@RequestMapping("/sms")
@Slf4j
public class WXController {

    final
    WxPayService wxPayService;
    final
    IOrdersService iOrdersService;
    final
    ISpcDetailService iSpcDetailService;
    final
    IMerchantProfitService merchantProfitService;
    final
    ISafeguardService iSafeguardService;
    final
    Producer producer;
    @Value("${wechat.suqiancredit.appId}")
    String WECHAT_APPID;
    @Value("${wechat.suqiancredit.secret}")
    String WECHAT_APPSECRET;
    @Value("${wechat.suqiancredit.token}")
    String WECHAT_TOKEN;

    public WXController(WxPayService wxPayService,
                        IOrdersService iOrdersService,
                        ISpcDetailService iSpcDetailService,
                        IMerchantProfitService merchantProfitService,
                        Producer producer,
                        ISafeguardService iSafeguardService) {
        this.wxPayService = wxPayService;
        this.iOrdersService = iOrdersService;
        this.iSpcDetailService = iSpcDetailService;
        this.merchantProfitService = merchantProfitService;
        this.producer = producer;
        this.iSafeguardService = iSafeguardService;
    }

    /**
     * @param wxContentQuery UGC安全对象
     * @return
     * @throws Exception
     */
    @PostMapping("/wx/ugc")
    @ApiOperation("WX图文验证")
    public BaseResponse WXCheckContent(@RequestBody WxContentQuery wxContentQuery) throws Exception {
        List<String> imgUrl = wxContentQuery.getUrlImg();
        String content = wxContentQuery.getContent();
        WeiXinDaoImpl dao = new WeiXinDaoImpl();
        OAuthAccessToken oac = dao.getNomalToken(WECHAT_APPID, WECHAT_APPSECRET);
        assert oac != null;
        String accessToken = oac.getAccessToken();
        //图文验证
        if (imgUrl.size() != 0) {
            //验证图片
            for (String url : imgUrl) {
                if (!UgcContentUtil.checkImgByImgUrl(url, accessToken)) {
                    return BaseResponse.error("非法图片!", this.getClass().getName());
                }
            }
        }
        //验证内容
        if (!UgcContentUtil.checkMsg(content, accessToken)) {
            return BaseResponse.error("内容非法!", this.getClass().getName());
        }
        return BaseResponse.ok();
    }

    /**
     * @param content UGC安全对象
     * @return
     * @throws Exception
     */
    @PostMapping("/ali/ugc")
    @ApiOperation("ali图文验证")
    public BaseResponse AliCheckContent(@RequestBody String content) throws Exception {
        if (AlipayUgcUtil.alipayCheckMsg(content)) {
            return BaseResponse.ok();
        } else {
            return BaseResponse.error("内容非法!", this.getClass().getName());
        }
    }

    /**
     * 公众号获取openid
     *
     * @param code
     * @return
     */
    @RequestMapping(value = "/getUserMessage", method = RequestMethod.GET)
    public ResponseEntity<Map<String, Object>> getUserMessage(String code) {
        WeiXinDaoImpl dao = new WeiXinDaoImpl();
        WxUser entity = null;
        Map<String, Object> map = new HashMap<>();
        try {
            OAuthAccessToken oac = dao.getOAuthAccessToken(WECHAT_APPID, WECHAT_APPSECRET, code);
            if (oac == null) {
                map.put("status", 500);
                map.put("message", "获取用户信息失败！");
                map.put("user", null);
                map.put("error", "获取用户信息失败");
                log.error("code编码", code);
            }
            OAuthAccessToken oacd = dao.refershOAuthAccessToken(WECHAT_APPID, oac.getRefreshToken());
            entity = dao.acceptOAuthUserNews(oacd.getAccessToken(), oacd.getOpenid());
            map.put("status", 200);
            map.put("message", "获取用户信息成功！");
            map.put("user", entity);
            return ResponseEntity.ok(map);
        } catch (Exception e) {
            e.printStackTrace();
            map.put("status", 500);
            map.put("message", "获取用户信息失败！");
            map.put("user", null);
            map.put("error", e.toString());
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(map);
    }

    /**
     * 小程序获取openid
     *
     * @param code
     * @return
     */
    @RequestMapping(value = "/getOpenid", method = RequestMethod.GET)
    public ResponseEntity<Map<String, Object>> getUserOpenId(String code) {
        WeiXinDaoImpl dao = new WeiXinDaoImpl();
        WxUser entity = null;
        Map<String, Object> map = new HashMap<>();
        try {
            OAuthAccessToken oac = dao.getOpenIdByCode(WECHAT_APPID, WECHAT_APPSECRET, code);
            if (oac == null) {
                map.put("status", 500);
                map.put("message", "获取用户Openid失败！");
                map.put("user", null);
                map.put("error", "获取用户Openid失败");
                log.error("code编码", code);
                return ResponseEntity.badRequest().body(map);
            }
            map.put("status", 200);
            map.put("message", "获取用户Openid成功！");
            map.put("Openid", oac.getOpenid());
            map.put("sessionKey", oac.getSessionKey());
            return ResponseEntity.ok(map);
        } catch (Exception e) {
            e.printStackTrace();
            map.put("status", 500);
            map.put("message", "获取用户Openid失败！");
            map.put("user", null);
            map.put("error", e.toString());
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(map);
    }

    /**
     * 微信url token验证。公众号的链接配置和跳转配置需要走这个验证。
     *
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping("/wxVaGet")
    @ResponseBody
    public void weixininter(HttpServletRequest request, HttpServletResponse response) throws Exception {
        //微信获取解析信息
        String str = WX_Util.check_Url(request, WECHAT_TOKEN);
        PrintWriter out = response.getWriter();
        out.print(str);
        out.flush();
        out.close();
    }

    /**
     * 发起服务窗口支付
     *
     * @return
     */
    @PostMapping("/servicePay")
    @ApiOperation("支付宝发起服务窗口支付")
    public BaseResponse servicePay(String orderId, String txnAmt, String buyerId, String body) {

        //支付宝支付时的参数
        PayParamQuery payParamQuery = PayParamQuery.builder()
                .openId(buyerId)
                .payMethod(1)
                .orderNo(orderId)
                .build();
        //将订单号存到保留域
        String mchReserved = body + "|" + orderId;
        try {
            Integer price = Integer.parseInt(txnAmt);
            payParamQuery.setTxnAmt(price + "");
        } catch (NumberFormatException ex) {
            return BaseResponse.error("金额不正确!", "支付宝服务窗口支付");
        }
        //微信下单时 支付方式
        Function<PayParamQuery, BaseResponse> zfbUnifiedFuc = arg ->
                wxPayService.servicePay(arg.getUnifiedOrderNo(), arg.getTxnAmt(),
                        arg.getOpenId(), mchReserved, payParamQuery.getSellerId() + "");

        return unifiedOrder(orderId, zfbUnifiedFuc, payParamQuery);
    }

    @PostMapping("/unifiedOrder")
    @ApiOperation("统一下单")
    public BaseResponse wxUnifiedOrder(HttpServletRequest request, String orderId,
                                       String openId, String body, String txnAmt) {
        String requestIp = IpKit.getRealIp(request);

        log.info("=====================requestIp==========================");
        log.info(requestIp);

        //微信支付时的参数
        PayParamQuery payParamQuery = PayParamQuery.builder()
                .requestIp(requestIp)
                .openId(openId)
                .unifiedOrderNo(orderId)
                .payMethod(0)
                .build();
        //将订单号存到保留域
        String mchReserved = body + "|" + orderId;
        try {
            Integer price = Integer.parseInt(txnAmt);
            payParamQuery.setTxnAmt(price + "");
        } catch (NumberFormatException ex) {
            return BaseResponse.error("金额不正确!", "支付宝服务窗口支付");
        }
        //微信下单时 支付方式
        Function<PayParamQuery, BaseResponse> wxUnifiedFuc = arg ->
                wxPayService.wxPay(arg.getRequestIp(), arg.getUnifiedOrderNo(),
                        arg.getTxnAmt(), arg.getOpenId(), mchReserved, payParamQuery.getSellerId() + "");

        return unifiedOrder(orderId, wxUnifiedFuc, payParamQuery);
    }

    @PostMapping("/payResultQuery")
    @ApiOperation("支付结果主动查询")
    public BaseResponse payResultQuery(String cmbOrderId, String orderId, String selleId) {
        return wxPayService.payResultQuery(cmbOrderId, orderId, selleId);
    }

    /**
     * 支付结果通知
     *
     * @return
     */
    @PostMapping("/payResultNotice")
    @ApiOperation("支付结果通知")
    public String payResultNotice(HttpServletRequest request) {

        String content = request.getParameter("biz_content");
        String sign = request.getParameter("sign");
        BizContent bizContent = JSON.parseObject(content, BizContent.class);
        log.info("============支付回调参数============");
        log.info(content);
        log.info(sign);
        String[] str = bizContent.getMchReserved().split("\\|");
        String orderNo = str[str.length - 1];
        log.info("===========================保留域订单号=======================");
        log.info(orderNo);
        log.info("=============================================================");
        //文档明确只有支付成功才会有通知
        if (bizContent == null || StrUtil.isEmpty(orderNo)) {
            log.info("========================== bizContent 为空! ===========================");
            PayResult payResult = new PayResult();
            payResult.setReturnCode("FAIL");
            payResult.setSign(sign);
            log.info(JSON.toJSONString(payResult));
            return JSON.toJSONString(payResult);
        } else {
            BaseResponse response = changeOrderStatusAfterPay(orderNo, bizContent.getOrderId());
            PayResult payResult = new PayResult();
            log.info("================  afterSuccess()方法  ==============");
            log.info(JSON.toJSONString(response));
            log.info("================  afterSuccess()方法  ==============");

            if (response.getCode() == 200) {
                payResult.setReturnCode("SUCCESS");
                payResult.setRespCode("SUCCESS");
                Map map = new HashMap<>();
                map.put("order_no", orderNo);
                OrderResponse orderResponse = iOrdersService.getOneOrder(JSON.toJSONString(map));
                Orders orders = orderResponse.getData();


                // 佣金推送
                if (orders != null && orderResponse.getCode() == 200) {
                    System.out.println(orderNo + "订单状态改变：" + orderResponse.code + "|" + orders.getOrderStatus());
                    if (OrderStatus.WAIT_COMMENT.name().equals(orders.getOrderStatus())) {
                        merchantProfitService.addMoneyData(orders.getSellerId(),
                                orders.getId(), orders.getPaymentMethodName(), orders.getPayPrice(), orders.getCreateTime());
                    }
                }

                //推送小票消息
                pushReceiptMsg(orders, bizContent.getMchReserved());
                //如果订单状态支付完成  向消费记录mq发消息
                orderCompleteEvent(orders);
                //推送积分增加信息
                pushPointsAndExpMsg(orders);
                //推送厨房出餐信息
                pushKitchenCookInfo(orders);
            } else {
                payResult.setRespCode("FAIL");
                payResult.setRespMsg("当前订单已经完成支付");
            }
            payResult.setSign(sign);
            log.info(JSON.toJSONString(payResult));
            return JSON.toJSONString(payResult);
        }
    }

    /**
     * 推送厨房备餐信息
     *
     * @param orders 订单信息
     */
    private void pushKitchenCookInfo(Orders orders) {
        if (orders != null && "ONLINE".equals(orders.getPaas())) {
            TransactionMessage transactionMessage = new TransactionMessage();
            transactionMessage.setMessage(JSON.toJSONString(orders));
            transactionMessage.setQueue("pointAndExpQue");
            transactionMessage.setRouteKey("pointAndExpQue");
            try {
                log.info("积分和经验增加 msg :{}", JSON.toJSONString(transactionMessage));
                mqpost(transactionMessage);
            } catch (IOException e) {
                log.error("积分和经验增长msg  推送失败了哒!");
                e.printStackTrace();
            }
        }
    }

    /**
     * 退款主动查询
     *
     * @param orderId
     * @return
     */
    @PostMapping("/payRefundQuery")
    @ApiOperation("退款主动查询")
    public BaseResponse payRefundQuery(String orderId, String origCmbOrderId, String sellerId) {
        return wxPayService.payRefundQuery(orderId, origCmbOrderId, sellerId);
    }

    /**
     * 申请退款
     *
     * @return
     */
    @PostMapping("/applyForRefund")
    @ApiOperation("申请退款")
    public BaseResponse applyForRefund(String orderId, String origOrderId,
                                       String txnAmt, String refundAmt, String refundReason,
                                       String sellerId) {
        return wxPayService.applyForRefund(orderId, origOrderId, txnAmt, refundAmt, refundReason, sellerId);
    }

    /**
     * 退款通知
     */
    @PostMapping("/refundNotice")
    @ApiOperation("退款通知")
    public String payRefundNotice(HttpServletRequest request) {

        String content = request.getParameter("biz_content");
        String sign = request.getParameter("sign");
        BizContent bizContent = JSON.parseObject(content, BizContent.class);
        log.info("============退款回调参数============");
        log.info(content);
        log.info(sign);
        String orderNo = bizContent.getOrderId();
        log.info("退款订单号========{}", orderNo);
        //文档明确只有退款成功才会有通知
        if (bizContent != null) {
            //获取退款订单信息
            BaseResponse refundDetailsResponse = iSafeguardService.refundDetails(orderNo);
            if (refundDetailsResponse.getCode() == 200) {
                HashMap hashMap = (HashMap) refundDetailsResponse.getData();
                Safeguard safeguard = BeanUtil.mapToBean(hashMap, Safeguard.class, true);
                Assert.notNull(safeguard, "退款订单信息不为空");
                //退款单状态为已退款  退出
                if (safeguard.getStatus().equals(SafeguardStatus.REFUND_COMPLETE.name())) {
                    return "";
                }
                //修改原始订单状态
                BaseResponse response = changeOrderStatusAfterRefund(safeguard.getOrderNo());
                PayResult payResult = new PayResult();
                log.info("================ 退款通知 改变订单状态方法响应结果  ==============");
                log.info(JSON.toJSONString(response));
                log.info("================ 退款通知 改变订单状态方法响应结果  ==============");
                //修改退款订单表状态
                if (response.getCode() == 200) {
                    SafeguardDetailQuery safeguardDetailQuery = new SafeguardDetailQuery();
                    safeguardDetailQuery.setRefundNo(orderNo);
                    safeguardDetailQuery.setStatus(8);
                    BaseResponse<String> edit = iSafeguardService.edit(safeguardDetailQuery);
                    if (edit.getCode() == 200) {
                        log.info("============退款改变退款单状态成功!===============");
                    } else {
                        log.error("================退款单状态改变失败!==================");
                    }
                }
                payResult.setSign(sign);
                log.info(JSON.toJSONString(payResult));
                return JSON.toJSONString(payResult);
            }
        }
        log.info("========================== bizContent 为空! ===========================");
        PayResult payResult = new PayResult();
        payResult.setReturnCode("FAIL");
        payResult.setSign(sign);
        log.info(JSON.toJSONString(payResult));
        return JSON.toJSONString(payResult);
    }

    /**
     * 修改退款订单状态
     *
     * @param orderNo 订单号(退款订单号)
     * @return
     */
    private BaseResponse changeOrderStatusAfterRefund(String orderNo) {

        //获取订单
        Map condition = new HashMap(1);
        condition.put("unified_order_no", orderNo);

        Orders orders = iOrdersService.getOneOrder(JSON.toJSONString(condition)).getData();
        if (orders != null) {
            OrderDetailQuery orderDetailQuery = OrderDetailQuery.builder()
                    .orderNo(orders.getOrderNo())
                    .refund("退款成功")
                    .unifiedOrderNo(orderNo)
                    .status(OrderStatus.REFUND.name())
                    .payStatus(PayStatus.PAY_YES.name())
                    .cashier(OrderStatus.REFUND.name())
                    .build();
            return iOrdersService.editOrder(orderDetailQuery);
        } else {
            log.info("没有找到对应的原始订单数据!  统一下单号为:{}", orderNo);
            return BaseResponse.error("反向修改订单状态失败!", this.getClass().getName());
        }

    }

    /**
     * 支付成功后修改订单状态
     *
     * @param orderNo 订单号
     * @return
     */
    private BaseResponse changeOrderStatusAfterPay(String orderNo, String unifiedOrderNo) {

        //获取订单
        Map condition = new HashMap(2);
        condition.put("order_status", OrderStatus.CONFIRM);
        condition.put("order_no", orderNo);
        Orders orders = iOrdersService.getOneOrder(JSON.toJSONString(condition)).getData();
        //修改订单状态
        if (orders != null) {
            OrderDetailQuery orderDetailQuery = OrderDetailQuery.builder()
                    .orderNo(orderNo)
                    .unifiedOrderNo(unifiedOrderNo)
                    .payStatus(PayStatus.PAY_YES.name())
                    .build();
            //修改订单状态  网上商城 --> 支付完 --> 待收货
            if ("ESHOP".equals(orders.getPaas())) {
                orderDetailQuery.setStatus(OrderStatus.WAIT_DELIVERING.name());
            } else {
                //收银端和展览支付完成 --> 待评价
                orderDetailQuery.setStatus(OrderStatus.WAIT_COMMENT.name());
            }
            return iOrdersService.editOrder(orderDetailQuery);
        } else {
            return BaseResponse.error("支付成功后订单状态改变失败!为获取到符合规则的订单", "Cashiers.class");
        }
    }

    /**
     * 关闭订单
     *
     * @param origOrderId
     * @param origCmbOrderId
     * @return
     */
    @PostMapping("/closeOrder")
    @ApiOperation("关闭订单")
    public BaseResponse closeOrder(String origOrderId, String origCmbOrderId, String sellerId) {

        return wxPayService.closeOrder(origOrderId, origCmbOrderId, sellerId);
    }

    /**
     * 获取对账地址
     *
     * @return
     */
    @PostMapping("/getBillUrl")
    @ApiOperation("获取对账地址")
    public BaseResponse getBillUrl(String billDate, String sellerId) {
        return wxPayService.getBillUrl(billDate, sellerId);
    }

    /**
     * 设置密钥
     *
     * @return
     */
    @PostMapping("/setSecretKey")
    public BaseResponse setSecretKey(String platformId, String digEvpEncMth, String digEvp, String encryptMethod,
                                     String encryptInfo, String keyTpye,
                                     String keyLen, String mchKey, String updateUrl) {

        return wxPayService.setSecretKey(platformId, digEvpEncMth, digEvp, encryptMethod, encryptInfo, keyTpye, keyLen, mchKey, updateUrl);
    }


    @GetMapping("/callBack")
    public ResponseEntity wxPayCallBack() {
        System.out.println("支付成功回调处理");
        return ResponseEntity.ok("支付成功");
    }

    /**
     * 订单完成时 发布事件
     */
    private void orderCompleteEvent(Orders orders) {
        if (orders == null) {
            throw new RuntimeException();
        }
        String msg = JSON.toJSONString(orders);
        log.info("===================消费记录mq推送======================");
        log.info("======msg: {}", msg);
        producer.send("create_xpense", msg);
    }

    /**
     * 统一下单 封装
     *
     * @param orderId       订单号
     * @param function      微信/支付宝支付
     * @param payParamQuery 参数
     * @return
     */
    private BaseResponse unifiedOrder(String orderId, Function<PayParamQuery, BaseResponse> function,
                                      PayParamQuery payParamQuery) {

        //由于招行下单订单号唯一，不得重复下单 防止招行下单成功，本数据库更新失败照成的重复下单, 先更新本数据库
        Map condition = MapUtil.newConcurrentHashMap();
        condition.put("order_no", orderId);
        Orders orders = iOrdersService.getOneOrder(JSON.toJSONString(condition)).getData();
        //获取临时统一下单 单号
        String unifiedOrderNo = uniqueNo();
        orders.setUnifiedOrderNo(unifiedOrderNo);
        payParamQuery.setUnifiedOrderNo(unifiedOrderNo);
        payParamQuery.setSellerId(orders.getSellerId());
        payParamQuery.setTxnAmt(orders.getPayPrice()
                .multiply(new BigDecimal(100))
                .intValue() + "");
        //订单不为待支付/支付中
        if (orders == null || (
                !OrderStatus.WAIT_PAY.name().equals(orders.getOrderStatus())
                        && !OrderStatus.CONFIRM.name().equals(orders.getOrderStatus())
        )) {
            log.error("统一下下单失败,未找到符合支付的订单");
            return BaseResponse.error("当前订单不可支付", "cn.wise.sc.payment.cmb.controller.WXController.wxUnifiedOrder");
        } else {

            OrderDetailQuery query = new OrderDetailQuery();
            query.setStatus(OrderStatus.CONFIRM.name());
            query.setOrderNo(orders.getOrderNo());
            query.setUnifiedOrderNo(unifiedOrderNo);
            //支付方式判断
            if (payParamQuery.getPayMethod() == 0) {
                query.setPayment("微信");
            } else if (payParamQuery.getPayMethod() == 1) {
                query.setPayment("支付宝");
            } else if (payParamQuery.getPayMethod() == 2) {
                query.setPayment("微信");
            } else {
                query.setPayment("第三方支付");
            }
            //修改订单状态
            BaseResponse baseResponse1 = iOrdersService.editOrder(query);
            if (baseResponse1.getCode() == 200) {
                //发起招行这边的统一下单

                log.info("====================================unifiedOrder--requestIp==========================");
                log.info(payParamQuery.getRequestIp());
                BaseResponse baseResponse = function.apply(payParamQuery);
                //招行下单失败时 回退本数据库订单状态  第一次,如果不能回退成功,交由xxl-job处理
                if (baseResponse.getCode() != 200) {
                    query.setStatus(OrderStatus.WAIT_PAY.name());
                    iOrdersService.editOrder(query);
                }
                return baseResponse;
            } else {
                return BaseResponse.error("下单失败", "cn.wise.sc.payment.cmb.controller.WXController.wxUnifiedOrder");
            }
        }
    }

    @PostMapping("/apply/payQR")
    public BaseResponse applyPayQR(String orderId, String body) {

        //判断订单是否存在
        if (StrUtil.isEmpty(orderId)) {
            return BaseResponse.error("订单号不能为空!", this.getClass().getName());
        }

        PayParamQuery payParamQuery = PayParamQuery.builder()
                .orderNo(orderId)
                .payMethod(8)
                .build();

        Function<PayParamQuery, BaseResponse> wxUnifiedFuc = arg ->
                wxPayService.applyPayQR(
                        arg.getTxnAmt(),
                        arg.getUnifiedOrderNo(),
                        body);

        return unifiedOrder(orderId, wxUnifiedFuc, payParamQuery);
    }


    /**
     * 获取唯一标识
     *
     * @return String
     */
    private String uniqueNo() {
        Snowflake snowflake = new Snowflake(2L, 2L);
        return snowflake.nextIdStr();
    }

    /**
     * 推送积分和经验增长消息
     *
     * @param order
     */
    private void pushPointsAndExpMsg(Orders order) {
        TransactionMessage transactionMessage = new TransactionMessage();
        transactionMessage.setMessage(JSON.toJSONString(order));
        transactionMessage.setQueue("pointAndExpQue");
        transactionMessage.setRouteKey("pointAndExpQue");
        try {
            log.info("积分和经验增加 msg :{}", JSON.toJSONString(transactionMessage));
            mqpost(transactionMessage);
        } catch (IOException e) {
            log.error("积分和经验增长msg  推送失败了哒!");
            e.printStackTrace();
        }
    }

    /**
     * 推送打印小票的消息
     *
     * @param orders
     * @param msgInfo
     */
    private void pushReceiptMsg(Orders orders, String msgInfo) {
        if (StrUtil.isBlank(msgInfo)) {
            log.error("打印小票的消息失败！预保留域为空！");
        }
        //收银系统 打印小票
        if ("cashier".equals(orders.getPaas())) {
            String[] strs = msgInfo.split("\\|");
            if (strs.length >= 2) {
                String msg = strs[strs.length - 3];
                String sn = strs[strs.length - 2];
                TransactionMessage transactionMessage = new TransactionMessage();
                transactionMessage.setMessage(msg + "|" + orders.getOrderNo());
                transactionMessage.setQueue(sn);
                transactionMessage.setRouteKey(sn);
                try {
                    log.info("打印小票 msg :{}", JSON.toJSONString(transactionMessage));
                    mqpost(transactionMessage);
                } catch (IOException e) {
                    log.error("收银端打印小票,mq消息推送失败!");
                    e.printStackTrace();
                }
            }
        }
    }

    public OrderResponse mqpost(@RequestBody TransactionMessage msg) throws IOException {
        /*TransactionMessage msg = new TransactionMessage();
        msg.setMessage(message); //发送的信息
        msg.setRouteKey(SN); //sn号
        msg.setQueue(SN);    //sn号
        msg.setSendSystem(system);//发消息系统
        */
        msg.setCreateTime(LocalDateTime.now());
        producer.createQueue(msg.getQueue());
        System.out.println(msg.getQueue());
        System.out.println(msg.getRouteKey());
        producer.send(msg.getRouteKey(), JSON.toJSONString(msg));
        //String msgs = (String) rabbitTemplate.receiveAndConvert("test1",2000l);
        //String transactionMessage = (String) rabbitTemplate.receiveAndConvert("test1");

        return OrderResponse.ok("发送成功");
    }

  /*  *
     * 获取微信授权code

     @GetMapping("/wxCodeGet/{payMoney}") public ResponseEntity<String> getAuthCode(@PathVariable("payMoney") BigDecimal payMoney) {
     WeiXinDaoImpl dao = new WeiXinDaoImpl();
     try {
     String codeURl = dao.getCodeUrl("wxbf3d40289ebc2cfc", "http://127.0.0.1:8870/sms/getUserMessage",
     "snsapi_login,snsapi_userinfo", payMoney.toString());
     return ResponseEntity.ok(codeURl);
     } catch (Exception e) {
     e.printStackTrace();
     return ResponseEntity.badRequest().body(e.getMessage());
     }*/
}

