package cn.wise.sc.pay.api.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.query.MemberQuery;
import cn.wise.sc.pay.domain.point.vo.MembershipVO;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 会员相关
 */
@FeignClient(value = "saas-v1-points", path = "/membership", fallback = MemberShipServiceImpl.class)
public interface IMemberShipService {

    /**
     * 查询所有(条件)
     *
     * @param pageWrapperQuery 关键字
     * @return
     */
    @PostMapping("/page")
    BaseResponse<Page<MembershipVO>> page(@RequestBody PageWrapperQuery pageWrapperQuery);

    /**
     * 根据用户id获取他的会员等级
     *
     * @param userId 用户id
     * @return
     */
    @GetMapping("/user/{userId}")
    BaseResponse getShipByUserId(@PathVariable("userId") long userId);


    /**
     * 查询单个
     *
     * @param id 会员id
     * @return
     */
    @GetMapping("/{id}")
    BaseResponse getMembership(@PathVariable("id") int id);

    /**
     * 激活禁止
     *
     * @param id     用户id
     * @param status 状态 1:激活 0:禁用
     * @return
     */
    @PutMapping("/{id}/{status}")
    BaseResponse forbiddenOrActive(@PathVariable("id") int id, @PathVariable("status") int status);

    /**
     * 根据id编辑用户信息
     *
     * @param memberQuery 用户请求实体
     * @return
     */
    @PutMapping
    BaseResponse editMembership(@RequestBody MemberQuery memberQuery);

    /**
     * 根据用户id删除用户
     *
     * @param id 用户id
     * @return
     */
    @DeleteMapping("/{id}")
    BaseResponse deleteUsers(@PathVariable("id") int id);

    /**
     * 新增用户
     *
     * @param membershipVo 用户请求实体
     * @return
     */
    @PostMapping("/new")
    BaseResponse newMembership(@RequestBody MembershipVO membershipVo);


    /**
     * 上传图片
     *
     * @param file
     * @return
     */
    @PostMapping("/upload")
    BaseResponse upload(@RequestBody MultipartFile file);

    /**
     * 根据多个id查询数据
     *
     * @param memberShipIds
     * @return
     */
    @GetMapping("/list/ids")
    BaseResponse<List<Membership>> listByIds(@RequestParam("memberShipIds") List<Long> memberShipIds);


    /**
     * 获取最初级的会员等级对象
     */
    @GetMapping("/primary")
    BaseResponse<Membership> getPrimaryMemberShip();

    /**
     * 根据会员id获取会员信息以及会员权益信息
     *
     * @param memberId
     * @return
     */
    @GetMapping("/getMemberInfo")
    BaseResponse<UserExtraVOApp> getMemberInfo(@RequestParam("memberId") Long memberId);

}
