package cn.wise.sc.pay.api.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.query.UserExreaQuery;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @description: 会员积分 会员等级 经验 扩展业务
 * @author: qh
 * @create: 2020-01-06 15:34
 **/
@FeignClient(value = "saas-v1-points", path = "/user/PointsMShipExtra", fallback = UserPointsMShipExtraServiceImpl.class)
public interface IUserPointsMShipExtraService {

    /**
     * 给用户关联一个新的   积分、经验、会员等级信息
     *
     * @param userExreaQuery 积分 等级
     * @return
     */
    @PostMapping
    BaseResponse addNewUserExtra(@RequestBody UserExreaQuery userExreaQuery);

    /**
     * 修改会员的  积分、经验\、会员等级信息
     *
     * @param userExreaQuery 积分 等级
     * @return
     */
    @PutMapping
    BaseResponse editUserExtra(@RequestBody UserExreaQuery userExreaQuery);

    /**
     * 根据会员id 查询积分  等级信息 o(╥﹏╥)o
     *
     * @param userId 会员id
     * @return
     */
    @GetMapping("/{userId}")
    BaseResponse<UserExtraVOApp> getUserExtra(@PathVariable("userId") Long userId, @RequestParam("flag") Integer flag);

    /**
     * 根据用户id集合  获取用户扩展 积分和会员等级
     *
     * @param userIds 会员id集合 body
     * @return
     */
    @PostMapping("/list")
    BaseResponse getAllUserExtra(@RequestBody List<Long> userIds);

    /**
     * 根据会员id 删除其扩展记录
     *
     * @param userId 会员id
     * @return
     */
    @DeleteMapping("/{userId}")
    BaseResponse delUserExtra(@PathVariable("userId") Long userId);

    /**
     * 根据membership_id查询记录
     *
     * @param membershipId
     * @return
     */
    @GetMapping("/membershipId/{id}")
    BaseResponse<List<UserExtra>> getByMemberId(@PathVariable(name = "id") Long membershipId);

    /**
     * 根据用户多个id查询记录
     *
     * @param userIds
     * @return
     */
    @GetMapping("/list/userIds")
    BaseResponse<List<UserExtra>> getListByUserId(@RequestParam("userIds") List<Long> userIds);
}
