package cn.wise.sc.pay.business.points.consumer;

import cn.wise.sc.pay.business.points.consumer.model.ArticleMq;
import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.business.points.service.IPointSourceV1Service;
import cn.wise.sc.pay.business.points.service.IUserExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.PointSourceV1;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.enums.PointSourceEnum;
import cn.wise.sc.pay.domain.point.query.MemberScoreQuery;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @description: 发布文章获取积分和经验
 * @author: qh
 * @create: 2020-05-14 14:39
 **/
@Slf4j
@Component
public class ArticlePointsExpListener {
    @Autowired
    IUserExtraService iUserExtraService;
    @Autowired
    IPointDetailService pointDetailService;
    @Autowired
    IPointSourceV1Service iPointSourceV1Service;

    @RabbitListener(queues = "pointArticleAndExpQue")
    public void processMessage(String content) {

        ArticleMq articleMq = JSON.parseObject(content, ArticleMq.class);

        //根据用户等级获取积分
        QueryWrapper<PointSourceV1> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("way", PointSourceEnum.ARTICLE.getCode());
        PointSourceV1 pointSource = iPointSourceV1Service.getOne(queryWrapper);
        Long userId = articleMq.getAuthorId();
        double points = 10;
        if (pointSource != null) {
            if (pointSource.getSetType() == 0) {
                double conversion = pointSource.getUnionScore().doubleValue();
                points = conversion;
            } else if (pointSource.getSetType() == 1) {
                //按会员等级来
                String json = pointSource.getMemberScores();
                List<MemberScoreQuery> list = JSON.parseArray(json, MemberScoreQuery.class);
                //用户传过来不为空时
                QueryWrapper<UserExtra> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("user_id", userId);
                UserExtra userExtra = iUserExtraService.getOne(queryWrapper1);
                MemberScoreQuery memberScoreQuery = list.stream().filter(arg -> arg.getMemberShipId().longValue() == userExtra.getMembershipId())
                        .findFirst().orElse(null);
                if (memberScoreQuery == null){
                    return;
                }
                //用户的会员等级 匹配积分获取比例
                points = memberScoreQuery.getMemberScore();
            }
            BaseResponse pointResponse = iUserExtraService.changePoints(userId, points, true);
            if (pointResponse.getCode() == 400) {
                log.error("processMessage>>pointAndExpQue添加积分错误: {}", pointResponse.getMsg());
            }
            BaseResponse expResponse = iUserExtraService.changeExperience(userId, points, true);
            if (expResponse.getCode() == 400) {
                log.error("processMessage>>pointAndExpQue添加经验值错误: {}", expResponse.getMsg());
            }
            //添加积分记录
            PointDetailQuery pointDetailQuery = PointDetailQuery.builder()
                    .userId(userId)
                    .score(points)
                    .inOrOut(1)
                    .pathWay(PointSourceEnum.ARTICLE.getCode())
                    .build();
            BaseResponse pointDetailResponse = pointDetailService.insertPointDetailInfo(pointDetailQuery);
            if (pointDetailResponse.getCode() == 400) {
                log.error("processMessage>>pointAndExpQue添加积分记录错误: {}", pointDetailResponse.getMsg());
            }

        }
    }
}
