package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.ICouponTypeService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.CouponTypeQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 优惠券类型控制器
 * @author jiawei
 */
@RestController
@RequestMapping("/couponType")
@CrossOrigin
@Api(tags = "优惠券类型")
public class CouponTypeController {
    @Autowired
    ICouponTypeService couponTypeService;

    @ApiOperation(value = "添加一条优惠券类型记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "couponTypeQuery", value = "类型参数对象", dataType = "CouponTypeQuery", paramType = "body")
            }
    )
    @PostMapping
    public BaseResponse add(@RequestBody CouponTypeQuery couponTypeQuery) {
        BaseResponse response = couponTypeService.add(couponTypeQuery);
        return response;
    }

    @GetMapping("/page")
    @ApiOperation(value = "根据类型名称条件分页")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "name", value = "类型名称", dataType = "String", paramType = "query")
            }
    )
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery) {
        BaseResponse response = couponTypeService.pageOnCondition(name, pageQuery);
        return response;
    }

    @PutMapping("/disable/{id}")
    @ApiOperation(value = "根据id禁用")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse disable(@PathVariable Integer id) {
        List<Integer> ids = new ArrayList<>();
        ids.add(id);
        BaseResponse response = couponTypeService.disable(ids);
        return response;
    }

    @PutMapping("/enable/{id}")
    @ApiOperation(value = "根据id启用")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse enable(@PathVariable Integer id) {
        List<Integer> ids = new ArrayList<>();
        ids.add(id);
        BaseResponse response = couponTypeService.enable(ids);
        return response;
    }

    @PutMapping
    @ApiOperation(value = "修改数据信息，传入数据时请传入id值")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "couponTypeQuery", value = "参数封装对象", dataType = "CouponTypeQuery", paramType = "body")
            }
    )
    public BaseResponse edit(@RequestBody CouponTypeQuery couponTypeQuery) {
        BaseResponse response = couponTypeService.edit(couponTypeQuery);
        return response;
    }

    @DeleteMapping("/{id}")
    @ApiOperation(value = "根据id值删除记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse remove(@PathVariable Integer id) {
        BaseResponse response = couponTypeService.discard(id);
        return response;
    }

    @GetMapping("{id}")
    @ApiOperation(value = "根据id值查询记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "long", paramType = "path")
            }
    )
    public BaseResponse infoById(@PathVariable Long id) {
        BaseResponse response = couponTypeService.infoById(id);
        return response;
    }


    @GetMapping("/list/available")
    @ApiOperation(value = "列表显示可用优惠券类型")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "long", paramType = "path")
            }
    )
    public BaseResponse listAvailable() {
        BaseResponse response = couponTypeService.listAvailable();
        return response;
    }
}
