package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.ICouponsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.CouponsQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;

/**
 * 优惠券实体控制器
 * @author jiawei
 */
@RestController
@RequestMapping("/coupons")
@CrossOrigin
@Api(tags = "优惠券")
public class CouponsController {
    @Autowired
    ICouponsService couponsService;


    @ApiOperation(value = "添加一条优惠券记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "couponsQuery", value = "类型参数对象", dataType = "CouponsQuery", paramType = "body")
            }
    )
    @PostMapping
    public BaseResponse add(@RequestBody CouponsQuery couponsQuery) {
        BaseResponse response = couponsService.add(couponsQuery);
        return response;
    }

    @GetMapping("/page")
    @ApiOperation(value = "根据名称与类型id条件分页")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "name", value = "券名称", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "typeId", value = "类型名称", dataType = "int", paramType = "query")
            }
    )
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery, Integer typeId) {
        BaseResponse response = couponsService.pageOnCondition(name, pageQuery, typeId);
        return response;
    }

    @DeleteMapping("/{id}")
    @ApiOperation(value = "根据id删除")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse deleteById(@PathVariable Integer id) {
        List<Integer> ids = new ArrayList<>();
        ids.add(id);
        BaseResponse response = couponsService.discard(ids);
        return response;
    }

    @DeleteMapping("/list")
    @ApiOperation(value = "根据id批量删除")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "ids", value = "id集合", dataType = "int", paramType = "body")
            }
    )
    public BaseResponse deleteByIds(@RequestBody List<Integer> ids) {
        BaseResponse response = couponsService.discard(ids);
        return response;
    }

    @PutMapping
    @ApiOperation(value = "修改数据信息，传入数据时请传入id值")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "couponsQuery", value = "参数封装对象", dataType = "CouponsQuery", paramType = "body")
            }
    )
    public BaseResponse edit(@RequestBody CouponsQuery couponsQuery) {
        BaseResponse response = couponsService.edit(couponsQuery);
        return response;
    }

    @GetMapping("{id}")
    @ApiOperation(value = "根据id值查询记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "long", paramType = "path")
            }
    )
    public BaseResponse infoById(@PathVariable Long id) {
        BaseResponse response = couponsService.infoById(id);
        return response;
    }

    @PutMapping("enable/{id}")
    @ApiOperation(value = "根据id启用记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "long", paramType = "path")
            }
    )
    public BaseResponse enableById(@NotNull(message = "id不能为空") @PathVariable Long id) {
        BaseResponse response = couponsService.enableById(id);
        return response;
    }


    @PutMapping("disable/{id}")
    @ApiOperation(value = "根据id禁用记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "long", paramType = "path")
            }
    )
    public BaseResponse disableById(@NotNull(message = "id不能为空") @PathVariable Long id) {
        BaseResponse response = couponsService.disableById(id);
        return response;
    }

    @GetMapping("/list")
    @ApiOperation(value = "查询优惠券列表")
    @ApiImplicitParams(
            {
            }
    )
    public BaseResponse listCoupons() {
        BaseResponse response = couponsService.listCoupons();
        return response;
    }

    @GetMapping("/list/available")
    @ApiOperation(value = "查询可用优惠券列表")
    @ApiImplicitParams(
            {
            }
    )
    public BaseResponse listAvailable() {
        BaseResponse response = couponsService.listAvailable();
        return response;
    }
}
