package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.IPointSourceService;
import cn.wise.sc.pay.domain.point.query.PointSourceQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@Api(description = "积分来源设置")
@CrossOrigin
@RestController

@RequestMapping("/pointSource")
public class PointSourceController {

    @Autowired
    private IPointSourceService pointSourceService;

    @PostMapping("/add")
    @ApiOperation(value = "添加积分来源设置")
    @ApiImplicitParam(name = "pointSourceQuery",value = "积分来源参数对象",dataType = "PointSourceQuery",paramType = "body")
    public ResponseEntity add(@RequestBody PointSourceQuery pointSourceQuery) {
        ResponseEntity responseEntity = pointSourceService.add(pointSourceQuery);
        return responseEntity;
    }

    @GetMapping
    @ApiOperation(value = "根据积分来源关键字分页查询")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "pageNo",value = "当前页",dataType = "int",paramType = "query"),
                    @ApiImplicitParam(name = "pageSize",value = "每页显示数量",dataType = "int",paramType = "query"),
                    @ApiImplicitParam(name = "keyWord",value = "关键字",dataType = "String",paramType = "query")
            }
    )
    public ResponseEntity pageKeyWord(Integer pageNo, Integer pageSize, String keyWord) {
        ResponseEntity responseEntity = pointSourceService.pageKeyWord(pageNo, pageSize, keyWord);
        return responseEntity;
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页查询列表")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "pageNo",value = "当前页",dataType = "int",paramType = "query"),
                    @ApiImplicitParam(name = "pageSize",value = "每页显示数量",dataType = "int",paramType = "query")
            }
    )
    public ResponseEntity pageList(Integer pageNo, Integer pageSize){
        ResponseEntity responseEntity = this.pageKeyWord(pageNo, pageSize, null);
        return responseEntity;
    }

    @PutMapping
    @ApiOperation(value = "更新积分来源设置",notes = "需要传入id参数")
    @ApiImplicitParam(name = "pointSourceQuery",value = "积分来源参数对象",dataType = "PointSourceQuery",paramType = "body")
    public ResponseEntity update(@RequestBody PointSourceQuery pointSourceQuery){
        ResponseEntity responseEntity = pointSourceService.update(pointSourceQuery);
        return responseEntity;
    }

    @PutMapping("/disable/{id}")
    @ApiOperation(value = "禁用积分来源设置")
    @ApiImplicitParam(name = "id",value = "id值",dataType = "long",paramType = "path")
    public ResponseEntity disable(@PathVariable Long id){
        ResponseEntity disable = pointSourceService.disable(id);
        return disable;
    }

    @DeleteMapping("/{id}")
    @ApiOperation(value = "根据id删除积分来源设置")
    @ApiImplicitParam(name = "id",value = "id值",dataType = "long",paramType = "path")
    public ResponseEntity deleteOne(@PathVariable Long id){
        ResponseEntity responseEntity = pointSourceService.deleteOne(id);
        return  responseEntity;
    }

    @GetMapping("/score")
    @ApiOperation(value = "根据会员id与途径获取积分值",notes = "正值为增加，负值为消费")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "memberId", value = "会员等级id", dataType = "long", paramType = "query"),
                    @ApiImplicitParam(name = "pathway", value = "积分来源途径", dataType = "String", paramType = "query")
            }
    )
    public Double getScore(Long memberId,String pathway){
        Double score = pointSourceService.getScore(memberId, pathway);
        return score;
    }

    @PutMapping("/enable/{id}")
    @ApiOperation(value = "启用积分来源设置")
    @ApiImplicitParam(name = "id",value = "id值",dataType = "long",paramType = "path")
    public ResponseEntity enable(@PathVariable Long id){

        ResponseEntity enable = pointSourceService.enable(id);
        return enable;
    }

}
