package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.IUserExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.query.UserExreaQuery;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @description: 会员积分 会员等级扩展业务
 * @author: qh
 * @create: 2020-01-06 15:03
 **/
@CrossOrigin
@RestController
@Api(description = "会员额外业务  积分和会员等级")
@RequestMapping("/user/PointsMShipExtra")
public class UserPointsMShipExtraController {

    @Autowired
    IUserExtraService iUserExtraService;

    /**
     * 给用户关联一个新的   积分、经验、会员等级信息
     *
     * @param userExreaQuery 积分 等级
     * @return
     */
    @PostMapping
    public BaseResponse addNewUserExtra(@RequestBody UserExreaQuery userExreaQuery) {

        return iUserExtraService.addNewUserExtra(userExreaQuery);
    }

    /**
     * 修改会员的  积分、经验、会员等级信息
     *
     * @param userExreaQuery 积分 等级
     * @return
     */
    @PutMapping
    public BaseResponse editUserExtra(@RequestBody UserExreaQuery userExreaQuery) {
        return iUserExtraService.editUserExtra(userExreaQuery);
    }

    /**
     * 根据用户id集合  获取用户扩展 积分和会员等级
     *
     * @param userIds 会员id集合 body
     * @return
     */
    @PostMapping("/list")
    public BaseResponse getAllUserExtra(@RequestBody List<Long> userIds) {
        return iUserExtraService.getList(userIds);
    }

    /**
     * 根据会员id 删除其扩展记录
     *
     * @param userId 会员id
     * @return
     */
    @DeleteMapping("/{userId}")
    public BaseResponse delUserExtra(@PathVariable("userId") Long userId) {
        return iUserExtraService.delUserExtra(userId);
    }

    /**
     * 根据会员id 查询积分  等级信息 o(╥﹏╥)o
     *
     * @param userId 会员id
     * @param flag   标记，0是获取当前会员等级积分经验，其它为获取下一等级的积分与经验
     * @return
     */
    @GetMapping("/{userId}")
    public BaseResponse<UserExtraVOApp> getUserExtra(@PathVariable("userId") Long userId, @RequestParam("flag") Integer flag) {
        return iUserExtraService.getUserExtra(userId, flag);
    }

    /**
     * 根据membership_id查询记录
     *
     * @param membershipId
     * @return
     */
    @GetMapping("/membershipId/{id}")
    public BaseResponse<List<UserExtra>> getByMemberId(@PathVariable(name = "id") Long membershipId) {
        BaseResponse byMemberId = iUserExtraService.getByMemberId(membershipId);
        return byMemberId;
    }

    /**
     * 根据用户多个id查询记录
     *
     * @param userIds
     * @return
     */
    @GetMapping("/list/userIds")
    public BaseResponse<List<UserExtra>> getListByUserId(@RequestParam("userIds") List<Long> userIds) {
        BaseResponse<List<UserExtra>> listByUserId = iUserExtraService.getListByUserId(userIds);
        return listByUserId;
    }
}
