package cn.wise.sc.pay.business.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.query.UserExreaQuery;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;


/**
 * @description: 用户额外字段 服务层
 * @author: qh
 * @create: 2020-01-06 11:05
 **/
public interface IUserExtraService extends IService<UserExtra> {
    /**
     * 编辑用户经验 积分 会员等级
     *
     * @param userExreaQueryer 编辑请求类
     * @return
     */
    BaseResponse editUserExtra(UserExreaQuery userExreaQueryer);

    /**
     * 根据用户id获取其积分和会员等级
     *
     * @return
     */
    BaseResponse getList(List<Long> userIds);

    /**
     * 根据用户id删除用户额外扩展
     *
     * @param userId 用户id
     * @return
     */
    BaseResponse delUserExtra(Long userId);

    /**
     * 改变会员积分
     *
     * @param userId     会员id
     * @param point      积分
     * @param isIncrease 是否增加  默认增加
     * @return
     */
    BaseResponse changePoints(Long userId, double point, boolean isIncrease);

    /**
     * 改变会员经验
     *
     * @param userId     会员id
     * @param experience 经验值
     * @param isIncrease 是否增加  默认增加
     * @return
     */
    BaseResponse changeExperience(Long userId, double experience, boolean isIncrease);

    /**
     * 根据会员id 查询会员权益  积分  等级
     *
     * @param userId
     * @return
     */
    BaseResponse<UserExtraVOApp> getUserExtra(Long userId,Integer flag);

    /**
     * 给用户关联一个新的   积分、经验、会员等级信息
     *
     * @param userExreaQuery 积分 等级
     * @return
     */
    BaseResponse addNewUserExtra(UserExreaQuery userExreaQuery);

    /**
     * 根据membership_id查询记录
     *
     * @param membershipId
     * @return
     */
    BaseResponse<List<UserExtra>> getByMemberId(Long membershipId);

    /**
     * 根据用户多个id查询记录
     *
     * @param userIds 用户id集合
     * @return list记录
     */
    BaseResponse<List<UserExtra>> getListByUserId(List<Long> userIds);

    UserExtra getUserExtraByUserId(Long userId);
}
