package cn.wise.sc.pay.business.points.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.points.mapper.PointDetailMapper;
import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.business.points.service.IUserExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.PointDetail;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import cn.wise.sc.pay.domain.point.vo.PointDetailVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-11-13
 */
@Service
public class PointDetailServiceImpl extends ServiceImpl<PointDetailMapper, PointDetail> implements IPointDetailService {

    @Autowired
    PointDetailMapper pointDetailMapper;
    @Autowired
    IUserExtraService iUserExtraService;

    @Override
    public BaseResponse<Page<PointDetailVO>> listByUserId(int userId, PageQuery pageQuery) {

        QueryWrapper ew = new QueryWrapper();
        Page page = new Page(pageQuery.getPageNo(), pageQuery.getPageSize());
        ew.eq("user_id", userId);
        ew.orderByDesc("create_time");
        page = (Page) this.page(page, ew);

        List<PointDetail> pointDetails = page.getRecords();
        List<PointDetailVO> pointDetailVOS = Lists.newArrayListWithCapacity(pointDetails.size());
        for (PointDetail pointDetail : pointDetails) {
            pointDetailVOS.add(new PointDetailVO(pointDetail));
        }

        page.setRecords(pointDetailVOS);

        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse addPointDetail(PointDetailQuery pointDetailQuery) {

        //验证积分详情参数是否有效
        if (!checkPointDetail(pointDetailQuery)) {
            return BaseResponse.error("积分详情参数无效", PointDetailServiceImpl.class.getName());
        }

        PointDetail pointDetail = PointDetail.builder()
                .inOut(pointDetailQuery.getInOrOut())
                .pathway(pointDetailQuery.getPathWay())
                .score(pointDetailQuery.getScore())
                .userId(pointDetailQuery.getUserId())
                .build();

        iUserExtraService.changePoints(pointDetailQuery.getUserId(), pointDetailQuery.getScore(), pointDetailQuery.getInOrOut() > 0);

        try {
            this.save(pointDetail);
            return BaseResponse.ok(pointDetail);
        } catch (Exception ex) {
            return BaseResponse.error("添加失败", PointDetailServiceImpl.class.getName());
        }
    }

    @Override
    public BaseResponse insertPointDetailInfo(PointDetailQuery pointDetailQuery) {
        //验证积分详情参数是否有效
        if (!checkPointDetail(pointDetailQuery)) {
            return BaseResponse.error("积分详情参数无效", PointDetailServiceImpl.class.getName());
        }
        PointDetail pointDetail = PointDetail.builder()
                .inOut(pointDetailQuery.getInOrOut())
                .pathway(pointDetailQuery.getPathWay())
                .score(pointDetailQuery.getScore())
                .userId(pointDetailQuery.getUserId())
                .createTime(System.currentTimeMillis())
                .build();
        boolean save = this.save(pointDetail);
        return save ? BaseResponse.ok(pointDetail) : BaseResponse.error("添加积分详情失败", PointDetailServiceImpl.class.getName());
    }

    @Override
    public BaseResponse getListByUser(long userId, int inOrOut) {

        if (userId == 0 || inOrOut < 0) {
            return BaseResponse.error("参数错误", null);
        }

        QueryWrapper qw = Wrappers.query();
        qw.eq("user_id", userId);
        this.list(qw);
        return null;
    }

    @Override
    public BaseResponse listByCOrI(Integer pageNo, Integer pageSize, Long userId, Integer condition) {
        QueryWrapper<PointDetail> pointDetailQueryWrapper = new QueryWrapper<>();
        IPage page = new Page(pageNo, pageSize);

        pointDetailQueryWrapper.eq("user_id", userId)
                .orderByDesc("create_time");
        if (condition == 0) {
            pointDetailQueryWrapper.eq("in_out", 0);
        } else {
            pointDetailQueryWrapper.eq("in_out", 1);
        }
        page = this.page(page, pointDetailQueryWrapper);
        List<PointDetail> pointDetails = page.getRecords();
        if (pointDetails == null || pointDetails.size() == 0) {
            return BaseResponse.error("获取列表数据失败,或者数据不存在", PointDetailServiceImpl.class.getName());
        }
        List<PointDetailVO> pointDetailVOS = Lists.newArrayListWithCapacity(pointDetails.size());
        for (PointDetail pointDetail : pointDetails) {
            pointDetailVOS.add(new PointDetailVO(pointDetail));
        }
        page.setRecords(pointDetailVOS);

        return BaseResponse.ok(page);
    }

    /**
     * 验证积分详情设置参数是否有效
     *
     * @param pointDto 积分请求参数
     * @return
     */
    private boolean checkPointDetail(PointDetailQuery pointDto) {

        return pointDto.getUserId() != 0 && pointDto.getScore() != 0
                && !StrUtil.isEmpty(pointDto.getPathWay());
    }
}
