package cn.wise.sc.pay.business.points.service.impl;

import cn.wise.sc.pay.business.points.mapper.PointSourceMapper;
import cn.wise.sc.pay.business.points.service.IMemberPointService;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.business.points.service.IPointSourceService;
import cn.wise.sc.pay.domain.point.bean.MemberPoint;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.bean.PointSource;
import cn.wise.sc.pay.domain.point.query.MemberPointsQuery;
import cn.wise.sc.pay.domain.point.query.PointSourceQuery;
import cn.wise.sc.pay.domain.point.vo.PointSourceVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 *
 */
@Service
@Transactional
public class PointSourceServiceImpl extends ServiceImpl<PointSourceMapper, PointSource> implements IPointSourceService {
    @Autowired
    private IMemberPointService memberPointService;


    @Autowired
    private IMembershipService membershipService;

    @Override
    public ResponseEntity add(PointSourceQuery pointSourceQuery) {
        //获取参数设置类型
        Integer setType = pointSourceQuery.getSetType();

        QueryWrapper qw = new QueryWrapper();
        qw.eq("path_way", pointSourceQuery.getPathWay());
        qw.eq("del_flag", 1);
        int count = this.count(qw);
        if (count > 0) {
            return ResponseEntity.ok("已存在!");
        }
        //统一设置
        if (setType == 0) {
            PointSource pointSource = PointSource.builder()
                    .createTime(System.currentTimeMillis())
                    .delFlag(0)
                    .description(pointSourceQuery.getDescription())
                    .typeFlag(pointSourceQuery.getTypeFlag())
                    .pathWay(pointSourceQuery.getPathWay())
                    .score(pointSourceQuery.getScore())
                    .setType(pointSourceQuery.getSetType())
                    .status(0)
                    .build();
            boolean save = this.save(pointSource);
            return save ? ResponseEntity.ok("添加成功") : ResponseEntity.badRequest().body("添加积分来源失败");

        } else {
            //会员设置
            if (setType == 1) {
                PointSource pointSource = PointSource.builder()
                        .pathWay(pointSourceQuery.getPathWay())
                        .status(0)
                        .setType(pointSourceQuery.getSetType())
                        .description(pointSourceQuery.getDescription())
                        .delFlag(0)
                        .typeFlag(pointSourceQuery.getTypeFlag())
                        .createTime(System.currentTimeMillis())
                        .build();
                //多个会员等级与积分
                List<MemberPoint> memberPoints = new ArrayList<>();
                List<MemberPointsQuery> memberPointsQueries = pointSourceQuery.getMemberPointsQueries();
                //遍历设值
                for (MemberPointsQuery memberPointsQuery : memberPointsQueries) {
                    MemberPoint memberPoint = new MemberPoint();
                    memberPoint.setMemberLevelId(memberPointsQuery.getMemberLevelId());
                    memberPoint.setMemberScore(memberPointsQuery.getScore());
                    memberPoint.setPathWay(pointSourceQuery.getPathWay());
                    memberPoint.setSetType(pointSourceQuery.getSetType());
                    memberPoint.setTypeFlag(pointSourceQuery.getTypeFlag());
                    memberPoints.add(memberPoint);
                }
                boolean saveBatch = memberPointService.saveBatch(memberPoints);
                boolean save = this.save(pointSource);
                return saveBatch && save ? ResponseEntity.ok("添加成功") : ResponseEntity.badRequest().body("添加失败");
            }

        }
        return ResponseEntity.badRequest().body("添加积分来源失败 参数传递有误！");
    }

    @Override
    public ResponseEntity pageKeyWord(Integer pageNo, Integer pageSize, String keyWord) {
        //构造参数
        IPage<PointSource> pointSourcePage = new Page<>(pageNo, pageSize);
        QueryWrapper<PointSource> pointSourceQueryWrapper = new QueryWrapper<>();
        //参数不为空构造条件
        if (StringUtils.isNotEmpty(keyWord)) {
            pointSourceQueryWrapper.like("path_way", keyWord);
        }
        IPage<PointSource> page = this.page(pointSourcePage, pointSourceQueryWrapper);
        //取出记录，重构数据
        List<PointSource> records = page.getRecords();
        //显示数据
        List<PointSourceVo> pointSourceVos = new ArrayList<>();
        //=============================================

        //会员设置记录的pathWay
        List<String> collect = records.stream().filter(s -> s.getSetType() == 1).map(s -> s.getPathWay()).collect(Collectors.toList());
        //存在会员设置
        if (collect != null) {
            QueryWrapper<MemberPoint> memberPointQueryWrapper1 = new QueryWrapper<>();
            memberPointQueryWrapper1.in("path_way", collect);
            //会员积分对象集合
            List<MemberPoint> memberPoints1 = memberPointService.list(memberPointQueryWrapper1);
            //会员id集合
            List<Long> collect1 = memberPoints1.stream().map(s -> s.getMemberLevelId()).distinct().collect(Collectors.toList());
            //会员等级对象集合
            List<Membership> memberships1 = membershipService.getListByIds(collect1).getData();
            //=============================================

            // 遍历records
            for (PointSource record : records) {
                PointSourceVo pointSourceVo = new PointSourceVo();
                //分页记录是会员设置类型
                if (record.getSetType() == 1) {
                    pointSourceVo.setPathWay(record.getPathWay());
                    pointSourceVo.setDescription(record.getDescription());
                    pointSourceVo.setStatus(record.getStatus());
                    pointSourceVo.setId(record.getId());

                    String pathWay = record.getPathWay();
                    //从会员集合中取出当前是pathWay的记录
                    List<MemberPoint> collect2 = memberPoints1.stream().filter(s -> s.getPathWay().equals(pathWay)).collect(Collectors.toList());
                    //从会员集合中取出当前是pathWay的会员id记录
                    List<Long> collect3 = collect2.stream().map(s -> s.getMemberLevelId()).collect(Collectors.toList());
                    HashMap<Long, String> longStringHashMap = new HashMap<>();
                    for (Long aLong : collect3) {
                        for (int i = 0; i < memberships1.size(); i++) {
                            if (aLong.equals(memberships1.get(i).getId())) {
                                longStringHashMap.put(aLong, memberships1.get(i).getName());
                            }
                        }
                    }
                    StringBuilder stringBuilder1 = new StringBuilder();
                    for (int i = 0; i < collect2.size(); i++) {
                        if (i < collect2.size() - 1) {
                            stringBuilder1.append(longStringHashMap.get(collect2.get(i).getMemberLevelId())).append(" ").append(collect2.get(i).getMemberScore()).append("\\");
                        }
                        stringBuilder1.append(longStringHashMap.get(collect2.get(i).getMemberLevelId())).append(" ").append(collect2.get(i).getMemberScore());
                    }
                    pointSourceVo.setScore(stringBuilder1.toString());
                    pointSourceVos.add(pointSourceVo);

                    //不是会员设置类型，即统一设置
                } else {
                    pointSourceVo.setScore(String.valueOf(record.getScore()));
                    pointSourceVo.setId(record.getId());
                    pointSourceVo.setStatus(record.getStatus());
                    pointSourceVo.setDescription(record.getDescription());
                    pointSourceVo.setPathWay(record.getPathWay());
                    pointSourceVos.add(pointSourceVo);
                }
            }
        } else {
            //不存在会员设置
            for (PointSource record : records) {
                PointSourceVo pointSourceVo = new PointSourceVo();
                pointSourceVo.setScore(String.valueOf(record.getScore()));
                pointSourceVo.setId(record.getId());
                pointSourceVo.setStatus(record.getStatus());
                pointSourceVo.setDescription(record.getDescription());
                pointSourceVo.setPathWay(record.getPathWay());
                pointSourceVos.add(pointSourceVo);
            }
        }

        Page<PointSourceVo> voPage = new Page<>();
        voPage.setRecords(pointSourceVos);
        voPage.setSize(page.getSize());
        voPage.setCurrent(page.getCurrent());
        voPage.setTotal(page.getTotal());
        return page == null ? ResponseEntity.badRequest().body("获取数据失败") : ResponseEntity.ok(voPage);
    }

    @Override
    public ResponseEntity update(PointSourceQuery pointSourceQuery) {
        //获取参数id
        Long queryId = pointSourceQuery.getId();
        if (queryId == null) {
            return ResponseEntity.badRequest().body("需要 传入id");
        } else {
            //根据id查询数据库，查看数据库设置类型
            PointSource byId = this.getById(queryId);
            Integer byIdSetType = byId.getSetType();
            //数据库中是统一设置
            if (byIdSetType == 0) {
                //统一设置改统一设置
                if (pointSourceQuery.getSetType() == 0) {
                    PointSource build = PointSource.builder()
                            .id(pointSourceQuery.getId())
                            .updateTime(System.currentTimeMillis())
                            .setType(0)
                            .score(pointSourceQuery.getScore())
                            .build();
                    //判断参数是否传值
                    assertCondition(pointSourceQuery, build);
                    //通过id更新数据库
                    boolean b = this.updateById(build);
                    return b == true ? ResponseEntity.ok("修改成功") : ResponseEntity.badRequest().body("修改失败");
                } else {
                    // 统一设置改会员设置
                    if (pointSourceQuery.getSetType() == 1) {
                        PointSource build = PointSource.builder()
                                .id(pointSourceQuery.getId())
                                .score(null)
                                .setType(1)
                                .updateTime(System.currentTimeMillis())
                                .build();
                        assertCondition(pointSourceQuery, build);
                        //更新积分来源表
                        boolean b = this.updateById(build);
                        //
                        List<MemberPoint> memberPoints = new ArrayList<>();
                        List<MemberPointsQuery> memberPointsQueries = pointSourceQuery.getMemberPointsQueries();
                        //遍历会员等级id与积分
                        for (MemberPointsQuery memberPointsQuery : memberPointsQueries) {
                            MemberPoint memberPoint = new MemberPoint();
                            memberPoint.setMemberLevelId(memberPointsQuery.getMemberLevelId());
                            memberPoint.setMemberScore(memberPointsQuery.getScore());
                            memberPoint.setPathWay(pointSourceQuery.getPathWay());
                            memberPoint.setSetType(pointSourceQuery.getSetType());
                            memberPoint.setTypeFlag(pointSourceQuery.getTypeFlag());
                            memberPoints.add(memberPoint);
                        }
                        //批量保存信息到会员积分表
                        boolean saveBatch = memberPointService.saveBatch(memberPoints);
                        return b && saveBatch ? ResponseEntity.ok("修改成功") : ResponseEntity.badRequest().body("修改失败");
                    } else {
                        return ResponseEntity.badRequest().body("参数传递有误");
                    }
                }
            } else {
                // 数据库是会员设置
                if (byIdSetType == 1) {
                    //会员设置改会员设置
                    if (pointSourceQuery.getSetType() == 1) {
                        PointSource build = PointSource.builder()
                                .updateTime(System.currentTimeMillis())
                                .setType(1)
                                .id(pointSourceQuery.getId())
                                .build();
                        assertCondition(pointSourceQuery, build);

                        List<MemberPoint> memberPoints = new ArrayList<>();

                        String pathWayCondition = byId.getPathWay();
                        Integer typeFlagCondition = byId.getTypeFlag();
                        QueryWrapper<MemberPoint> memberPointQueryWrapper = new QueryWrapper<>();

                        memberPointQueryWrapper.eq("path_way", pathWayCondition)
                                .eq("type_flag", typeFlagCondition);
                        List<MemberPoint> memberPointList = memberPointService.list(memberPointQueryWrapper);
                        List<MemberPointsQuery> memberPointsQueries = pointSourceQuery.getMemberPointsQueries();
                        //遍历参数中的会员id与积分 并设值用于更新会员积分表
                        for (MemberPoint memberPoint1 : memberPointList) {

                            for (MemberPointsQuery memberPointsQuery : memberPointsQueries) {
                                if (memberPoint1.getMemberLevelId().equals(memberPointsQuery.getMemberLevelId())) {
                                    MemberPoint memberPoint = new MemberPoint();
                                    memberPoint.setId(memberPoint1.getId());
                                    memberPoint.setMemberLevelId(memberPointsQuery.getMemberLevelId());
                                    memberPoint.setMemberScore(memberPointsQuery.getScore());
                                    memberPoint.setPathWay(pointSourceQuery.getPathWay());
                                    memberPoint.setSetType(pointSourceQuery.getSetType());
                                    memberPoint.setTypeFlag(pointSourceQuery.getTypeFlag());
                                    memberPoints.add(memberPoint);
                                }
                            }
                        }

                        //修改积分来源
                        boolean b = this.updateById(build);
                        //修改会员积分表信息
                        boolean b1 = memberPointService.updateBatchById(memberPoints);
                        return b && b1 ? ResponseEntity.ok("修改成功") : ResponseEntity.badRequest().body("修改失败");

                    } else {
                        //会员设置改统一设置
                        if (pointSourceQuery.getSetType() == 0) {
                            PointSource build = PointSource.builder()
                                    .id(pointSourceQuery.getId())
                                    .setType(0)
                                    .updateTime(System.currentTimeMillis())
                                    .score(pointSourceQuery.getScore())
                                    .build();
                            assertCondition(pointSourceQuery, build);
                            boolean b = this.updateById(build);
//                            PointSource byId1 = this.getById(pointSourceQuery.getId());
                            QueryWrapper<MemberPoint> memberPointQueryWrapper = new QueryWrapper<>();
                            memberPointQueryWrapper.eq("type_flag", byId.getTypeFlag())
                                    .eq("path_way", byId.getPathWay());
                            boolean remove = memberPointService.remove(memberPointQueryWrapper);
                            return b && remove ? ResponseEntity.ok("修改成功") : ResponseEntity.badRequest().body("修改失败");
                        }
                    }

                } else {
                    return ResponseEntity.badRequest().body("参数传递有误");
                }

            }
            return ResponseEntity.badRequest().body("参数传递有误");
        }
    }

    @Override
    public ResponseEntity disable(Long id) {
        PointSource byId = this.getById(id);
        if (byId.getStatus() == 1) {
            return ResponseEntity.badRequest().body("该积分来源已经被禁用");
        }
        PointSource pointSource = new PointSource();
        pointSource.setId(id);
        pointSource.setStatus(1);
        boolean b = this.updateById(pointSource);/*
        UpdateWrapper<PointSource> pointSourceUpdateWrapper = new UpdateWrapper<>();
        pointSourceUpdateWrapper.eq("status", 1);
        boolean update = this.update(pointSourceUpdateWrapper);*/
        return b ? ResponseEntity.ok("禁用成功") : ResponseEntity.badRequest().body("禁用失败");
    }

    @Override
    public ResponseEntity deleteOne(Long id) {
        boolean b = this.removeById(id);
        return b ? ResponseEntity.ok("删除成功") : ResponseEntity.badRequest().body("删除失败");
    }

    @Override
    public Double getScore(Long memberId, String pathway) {

        QueryWrapper<PointSource> pointSourceQueryWrapper = new QueryWrapper<>();
        pointSourceQueryWrapper.eq("path_way", pathway)
                .eq("status", 0);
        PointSource one = this.getOne(pointSourceQueryWrapper);
        if (one.getSetType() == 0) {
            if (one.getTypeFlag() == 1) {
                double v = 0 - one.getScore();
                return v;
            }
            return one.getScore();
        } else {
            if (one.getSetType() == 1) {
                QueryWrapper<MemberPoint> memberPointQueryWrapper = new QueryWrapper<>();
                memberPointQueryWrapper.eq("member_level_id", memberId)
                        .eq("path_way", pathway);
                MemberPoint memberPointServiceOne = memberPointService.getOne(memberPointQueryWrapper);
                if (one.getTypeFlag() == 1) {
                    return 0 - memberPointServiceOne.getMemberScore();
                }
                return memberPointServiceOne.getMemberScore();
            }
        }
        return null;
    }

    @Override
    public ResponseEntity enable(Long id) {
        PointSource byId = this.getById(id);
        if (byId.getStatus() == 0) {
            return ResponseEntity.badRequest().body("该积分来源处于启用状态");
        }
        PointSource pointSource = new PointSource();
        pointSource.setId(id);
        pointSource.setStatus(0);
        boolean b = this.updateById(pointSource);/*
        UpdateWrapper<PointSource> pointSourceUpdateWrapper = new UpdateWrapper<>();
        pointSourceUpdateWrapper.eq("status", 0);
        boolean update = this.update(pointSourceUpdateWrapper);*/
        return b ? ResponseEntity.ok("启用成功") : ResponseEntity.badRequest().body("启用失败");
    }

    private void assertCondition(PointSourceQuery pointSourceQuery, PointSource build) {
        if (pointSourceQuery.getPathWay() != null) {
            build.setPathWay(pointSourceQuery.getPathWay());
        }
        if (pointSourceQuery.getTypeFlag() != null) {
            build.setTypeFlag(pointSourceQuery.getTypeFlag());
        }
        if (pointSourceQuery.getDescription() != null) {
            build.setDescription(pointSourceQuery.getDescription());
        }
    }

}
