package cn.wise.sc.pay.business.points.service.impl;

import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.VirtualGoodsMapper;
import cn.wise.sc.pay.business.points.service.ICouponTypeService;
import cn.wise.sc.pay.business.points.service.ICouponsService;
import cn.wise.sc.pay.business.points.service.IVirtualGoodsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.CouponType;
import cn.wise.sc.pay.domain.point.bean.VirtualGoods;
import cn.wise.sc.pay.domain.point.query.VirtualGoodQuery;
import cn.wise.sc.pay.domain.point.vo.VirtualGoodVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 虚拟商品业务实现类
 *
 * @author jiawei
 */
@Service
public class VirtualGoodsServiceImpl extends ServiceImpl<VirtualGoodsMapper, VirtualGoods> implements IVirtualGoodsService {

    @Autowired
    ICouponTypeService couponTypeService;
    @Autowired
    ICouponsService couponsService;
    @Autowired
    VirtualGoodsMapper virtualGoodsMapper;
    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    /**
     * 添加一条虚拟商品记录
     */
    public BaseResponse add(VirtualGoodQuery virtualGoodQuery) {
        VirtualGoods virtualGoods = VirtualGoodQuery.query2Entity(virtualGoodQuery);
        virtualGoods.setCreateTime(System.currentTimeMillis());
        virtualGoods.setRemainNum(virtualGoods.getIssueNum());
        virtualGoods.setStatus(0);

        boolean save = this.save(virtualGoods);
        return save ? BaseResponse.ok(virtualGoods) : BaseResponse.error("添加数据失败", VirtualGoodsServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据券名称与券类型id条件分页
     */
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery, String typeName) {
        Page<VirtualGoodVo> virtualGoodVoPage = new Page<>();
        virtualGoodVoPage.setSize(pageQuery.getPageSize());
        virtualGoodVoPage.setCurrent(pageQuery.getPageNo());
        QueryWrapper queryWrapper = new QueryWrapper();
        //逻辑删除条件
        queryWrapper.eq("c.del_flag", 0);
        queryWrapper.eq("v.del_flag", 0);
        queryWrapper.eq("ct.del_flag", 0);
        queryWrapper.orderByDesc("v.create_time");
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("c.name", name);
        }
        List<Integer> typeIds = new ArrayList<>();
        if (!org.springframework.util.StringUtils.isEmpty(typeName)) {
            QueryWrapper<CouponType> typeQueryWrapper = new QueryWrapper<>();
            typeQueryWrapper.like("name", typeName);
            List<CouponType> couponTypes = couponTypeService.list(typeQueryWrapper);
            if (CollectionUtils.isEmpty(couponTypes)) {
                return BaseResponse.ok((List<VirtualGoodVo>) null, this.getClass());
            }
            Set<Integer> typeIdSet = couponTypes.stream().map(ct -> ct.getId()).collect(Collectors.toSet());
            typeIds.addAll(typeIdSet);
        }

        if (!CollectionUtils.isEmpty(typeIds)) {
            queryWrapper.in("ct.id", typeIds);
        }
        IPage<VirtualGoodVo> pageCondition = virtualGoodsMapper.getPageCondition(virtualGoodVoPage, queryWrapper);
        //拼接图片
        if (pageCondition != null && !CollectionUtils.isEmpty(pageCondition.getRecords())) {
            pageCondition.getRecords().stream().forEach(re -> re.setImage(fastDFSUtils.fullUrl(re.getImage())));
        }
        return BaseResponse.ok(pageCondition, VirtualGoodsServiceImpl.class);
    }

    @Override
    /**
     * 根据id删除 或者批量删除
     */
    public BaseResponse discard(List<Long> ids) {
        if (ids == null || ids.size() < 1) {
            return BaseResponse.error("请传入id值", VirtualGoodsServiceImpl.class.getName());
        }
        boolean remove = this.removeByIds(ids);
        return remove ? BaseResponse.ok() : BaseResponse.error("删除失败", VirtualGoodsServiceImpl.class.getName());
    }

    @Override
    /**
     * 修改数据信息，传入数据时请传入id值
     */
    public BaseResponse edit(VirtualGoodQuery virtualGoodQuery) {
        if (virtualGoodQuery.getId() == null) {
            return BaseResponse.error("请传入id值", VirtualGoodsServiceImpl.class.getName());
        }
        VirtualGoods virtualGoods = VirtualGoodQuery.query2Entity(virtualGoodQuery);
        virtualGoods.setUpdateTime(System.currentTimeMillis());
        boolean update = this.updateById(virtualGoods);
        return update ? BaseResponse.ok(virtualGoods) : BaseResponse.error("删除失败", VirtualGoodsServiceImpl.class.getName());
    }
}
