package cn.wise.sc.pay.domain.point.query;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * 积分来源设置参数对象
 */
@Data
@Builder
@ApiModel
@AllArgsConstructor
@NoArgsConstructor
public class PointSourceV1Query {
    /**
     * 主键id
     */
    @ApiModelProperty(value = "主键id", notes = "新增记录时不传id")
    private Integer id;
    /**
     * 途径  0:发布文章,  1:线上消费,  2:签到奖励,  3:优质点评
     */
    @ApiModelProperty(value = "途径  0:发布文章,  1:线上消费,  2:签到奖励,  3:优质点评")
    private Integer way;
    /**
     * 设置类型,0统一设置，1按会员等级设置，2不累计设置，3按天累计设置，4点评设置
     */
    @ApiModelProperty(value = "设置类型,0统一设置，1按会员等级设置，2不累计设置，3按天累计设置，4点评设置")
    private Integer setType;
    /**
     * 统一设置积分值
     */
    @ApiModelProperty(value = "统一设置积分值")
    private Integer unionScore;
    /**
     * 会员积分值
     */
    @ApiModelProperty(value = "会员积分存储字段", notes = "数组类型")
    private List<MemberScoreQuery> memberScores;
    /**
     * 是否禁用 0 未禁用，1 禁用
     */
    @ApiModelProperty(value = "是否禁用 0 未禁用，1 禁用")
    private Integer status;
    /**
     * 按天累计积分值
     */
    @ApiModelProperty(value = "按天累计积分值", notes = "数组类型，取值0、1、2，分别对应：基础积分、每天累计、清0天数")
    private List<CumulativeScoreQuery> cumulativeScores;
    /**
     * 优质点评积分值
     */
    @ApiModelProperty(value = "优质点评存储字段")
    private WordComment wordComment;
    /**
     * 不累计设置积分值
     */
    @ApiModelProperty(value = "不累计设置积分值")
    private Integer nonCumulativeScore;
}
