package cn.wise.sc.pay.domain.point.util;

import cn.wise.sc.pay.domain.point.bean.PointSourceV1;
import cn.wise.sc.pay.domain.point.enums.PointSourceEnum;
import cn.wise.sc.pay.domain.point.enums.SignRewardEnum;
import cn.wise.sc.pay.domain.point.query.CumulativeScoreQuery;
import cn.wise.sc.pay.domain.point.query.MemberScoreQuery;
import cn.wise.sc.pay.domain.point.query.PointSourceV1Query;
import cn.wise.sc.pay.domain.point.query.WordComment;
import cn.wise.sc.pay.domain.point.vo.CumulativeScoreVo;
import cn.wise.sc.pay.domain.point.vo.PointSourceV1Vo;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 积分来源工具类
 */
public class PointSourceV1Util {
    static ObjectMapper objectMapper = new ObjectMapper();

    //解析参数
    public static PointSourceV1 parseQueryParameter(PointSourceV1Query pointSourceV1Query) {
        if (pointSourceV1Query == null) {
            return null;
        }
        PointSourceV1 pointSourceBuilder = null;
        try {
            pointSourceBuilder = PointSourceV1.builder()
                    .id(pointSourceV1Query.getId())
                    .memberScores(pointSourceV1Query.getMemberScores() == null ? null : objectMapper.writeValueAsString(pointSourceV1Query.getMemberScores()))
                    .nonCumulativeScore(pointSourceV1Query.getNonCumulativeScore())
                    .setType(pointSourceV1Query.getSetType())
                    .unionScore(pointSourceV1Query.getUnionScore())
                    .wordScores(pointSourceV1Query.getWordComment() == null ? null : objectMapper.writeValueAsString(pointSourceV1Query.getWordComment()))
                    .way(getWay(pointSourceV1Query.getWay()))
                    .cumulativeScores(pointSourceV1Query.getCumulativeScores() == null ? null : objectMapper.writeValueAsString(pointSourceV1Query.getCumulativeScores()))
                    .build();
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        return pointSourceBuilder;
    }

    //途径与标识相互转换
    public static String getWay(int way) {
        switch (way) {
            case 0:
                return PointSourceEnum.ARTICLE.getCode();
            case 1:
                return PointSourceEnum.ONLINE.getCode();
            case 2:
                return PointSourceEnum.SIGN.getCode();
            case 3:
                return PointSourceEnum.COMMON.getCode();
            default:
                return null;
        }
    }

    //途径与标识相互转换
    public static Integer getWayInt(String way) {
        if (way == null) {
            return null;
        }
        Integer result = null;
        PointSourceEnum[] values = PointSourceEnum.values();
        for (PointSourceEnum value : values) {
            if (value.getCode().equals(way)) {
                result = value.getFlag();
                return result;
            }
        }
        return result;
    }

    public static PointSourceV1Vo parseBean2Vo(PointSourceV1 pointSourceV1) {
        if (pointSourceV1 == null) {
            return null;
        }
        PointSourceV1Vo pointSourceBuild = null;
        try {
            pointSourceBuild = PointSourceV1Vo.builder()
                    .createTime(pointSourceV1.getCreateTime())
                    .cumulativeScores(StringUtils.isEmpty(pointSourceV1.getCumulativeScores()) ? null : objectMapper.readValue(pointSourceV1.getCumulativeScores(), objectMapper.getTypeFactory().constructParametricType(ArrayList.class, CumulativeScoreQuery.class)))
                    .id(pointSourceV1.getId())
                    //                .memberScore()
                    .nonCumulativeScore(pointSourceV1.getNonCumulativeScore())
                    .setType(pointSourceV1.getSetType())
                    .status(pointSourceV1.getStatus())
                    .updateTime(pointSourceV1.getUpdateTime())
                    .unionScore(pointSourceV1.getUnionScore())
                    .way(pointSourceV1.getWay())
                    .wordComment(StringUtils.isEmpty(pointSourceV1.getWordScores()) ? null : objectMapper.readValue(pointSourceV1.getWordScores(), WordComment.class))
                    .memberScores(
                            StringUtils.isEmpty(pointSourceV1.getMemberScores()) ? null : JSON.parseArray(pointSourceV1.getMemberScores(), MemberScoreQuery.class))
                    .build();
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        return pointSourceBuild;
    }

    public static String cumulativeInt(Integer cumulative) {
        String val = null;
        for (SignRewardEnum value : SignRewardEnum.values()) {
            if (cumulative.equals(value.getFlag())) {
                val = value.getCode();
            }
        }
        return val;
    }

    public static List<CumulativeScoreQuery> defaultCumulativeScore() {
        SignRewardEnum[] values = SignRewardEnum.values();
        List<CumulativeScoreQuery> cumulativeScoreQueryList = new ArrayList<>();
        for (SignRewardEnum value : values) {
            CumulativeScoreQuery build = CumulativeScoreQuery.builder()
                    .cumulativeFlag(value.getFlag())
                    .cumulativeScore(0)
                    .build();
            cumulativeScoreQueryList.add(build);
        }
        return cumulativeScoreQueryList;
    }

    public static List<CumulativeScoreVo> defaultCumulativeScoreVo() {
        SignRewardEnum[] values = SignRewardEnum.values();
        List<CumulativeScoreVo> cumulativeScoreVoList = new ArrayList<>();
        for (SignRewardEnum value : values) {
            CumulativeScoreVo build = CumulativeScoreVo.builder()
                    .cumulativeFlag(value.getFlag())
                    .cumulativeName(value.getCode())
                    .cumulativeScore(0)
                    .build();
            cumulativeScoreVoList.add(build);
        }
        return cumulativeScoreVoList;
    }
}
