package cn.wise.sc.pay.task.timer.order.handler;

import cn.wise.sc.pay.api.order.service.IOrdersAutoSettingService;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.OrdersAutoSetting;
import cn.wise.sc.pay.domain.order.model.OrdersDBColumn;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @description: 订单自动取消定时任务
 * @author: qh
 * @create: 2020-03-31 17:22
 **/
@Component
public class OrderAutoCancelHandler  {

    final
    BaseOrderSettingHandler baseOrderSettingHandler;

    final
    IOrdersAutoSettingService iOrdersAutoSettingService;

    /**
     * 一次性处理订单的大小
     */
    private static final int BATCH_COUNT = 200;


    /**
     * 批量处理处理订单取消 允许最大同时处理 50*200 其余的丢弃
     */
    final ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(20, 50, 2, TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(BATCH_COUNT), r -> new Thread("thread -- OrderAutoCancel"), new ThreadPoolExecutor.DiscardPolicy());

    public OrderAutoCancelHandler(BaseOrderSettingHandler baseOrderSettingHandler,
                                  IOrdersAutoSettingService iOrdersAutoSettingService) {
        this.baseOrderSettingHandler = baseOrderSettingHandler;
        this.iOrdersAutoSettingService = iOrdersAutoSettingService;
    }


    @XxlJob("OrderAutoCancelHandler")
    public ReturnT<String> execute(String s) {
        //获取订单自动取消时间
        OrdersAutoSetting ordersAutoSetting = baseOrderSettingHandler.getOrderAutoSetting();
        if (ordersAutoSetting == null) {
            return ReturnT.FAIL;
        }
        //取订单状态为未支付的  一次性取200条
        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        Map<String, Object> eq = new HashMap<>(2);
        eq.put(OrdersDBColumn.PAY_STATUS, PayStatus.PAY_ON);
        eq.put("paas", "ESHOP");
        pageWrapperQuery.setEq(eq);

        return baseOrderSettingHandler.baseExecute(threadPoolExecutor, pageWrapperQuery,
                ordersAutoSetting.getAutoCancelOrderTime(), OrderStatus.CANCELLED);

    }


}
