package cn.wise.sc.pay.task.timer.order.handler;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.api.order.service.IOrderCallExService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.order.service.ISafeguardService;
import cn.wise.sc.pay.api.order.service.IZHPaymentService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.domain.order.query.SafeguardDetailQuery;
import cn.wise.sc.pay.task.timer.order.model.RefundRtsQueryParam;
import cn.wise.sc.pay.task.timer.order.model.RefundRtsState;
import com.alibaba.fastjson.JSON;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 订单退款回调失败 定时处理器
 * @author: qh
 * @create: 2020-01-14 14:43
 **/
@Component
public class OrderrRefundCallExHandler {

    final
    IOrderCallExService orderCallExService;
    final
    IZHPaymentService izhPaymentService;
    final
    IOrdersService orderService;
    final
    ISafeguardService iSafeguardService;

    public OrderrRefundCallExHandler(IOrderCallExService orderCallExService,
                                     IZHPaymentService izhPaymentService,
                                     IOrdersService orderService,
                                     ISafeguardService iSafeguardService) {
        this.orderCallExService = orderCallExService;
        this.izhPaymentService = izhPaymentService;
        this.orderService = orderService;
        this.iSafeguardService = iSafeguardService;
    }

    @XxlJob("OrderRefundCallExHandler")
    public ReturnT<String> execute(String s) throws Exception {

        BaseResponse orderResponse = orderCallExService.getList(1);
        if (orderResponse.getCode() == 400) {
            orderResponse = orderCallExService.getList(1);
        }
        List<HashMap> hashMaps = orderResponse.getList();
        //HashMap转Bean
        List<OrderCallException> refundExceptionList = hashMaps.stream()
                .map(arg -> BeanUtil.mapToBean(arg, OrderCallException.class, true))
                .collect(Collectors.toList());

        /*
         * 轮询  退款结果返回异常的订单，发起主动查询
         *       预退款完成->修改失败次数+1  次数达到20次 则退款失败
         *       失败->修改异常状态为已关闭,修改订单状态为取消。
         *       退款成功->修改订单状态 为REFUND,关闭退款异常
         * */
        List<OrderDetailQuery> editOrders = new ArrayList<>();
        List<OrderCallException> editOrderEx = new ArrayList<>(refundExceptionList.size());
        for (OrderCallException orderCallEx : refundExceptionList) {
            //找原始订单号  orderCallEx的orderNo存的是统一下单号
            Map condition = new HashMap(1);
            condition.put("unified_order_no", orderCallEx.getOrderNo());
            Orders orders = orderService.getOneOrder(JSON.toJSONString(condition)).getData();
            if (orders == null){
                continue;
            }
            BaseResponse payResponse = izhPaymentService.payRefundQuery(orderCallEx.getRefundNo(), ""
                    , orders.getSellerId() + "");
            if (payResponse.getCode() == 200) {
                RefundRtsQueryParam payRtsQueryParam = JSON.parseObject(payResponse.getData().toString(), RefundRtsQueryParam.class);

                if (orders == null) {
                    return ReturnT.FAIL;
                }
                OrderCallException editOrderCallEx = new OrderCallException();
                BeanUtil.copyProperties(orderCallEx, editOrderCallEx);

                if (RefundRtsState.P.name().equals(payRtsQueryParam.getTradeState())) {
                    editOrderCallEx.setFailCount(orderCallEx.getFailCount() + 1);
                } else if (RefundRtsState.S.name().equals(payRtsQueryParam.getTradeState())) {
                    //修改原始订单field
                    OrderDetailQuery editOrder = new OrderDetailQuery();
                    editOrder.setOrderNo(orders.getUnifiedOrderNo());
                    editOrder.setCashier(OrderStatus.REFUND.name());
                    editOrder.setStatus(OrderStatus.REFUND.name());
                    editOrders.add(editOrder);
                    editOrderCallEx.setStatus(1);
                    //编辑退款单 为退款成功
                    SafeguardDetailQuery safeguardDetailQuery = new SafeguardDetailQuery();
                    safeguardDetailQuery.setRefundNo(orderCallEx.getRefundNo());
                    safeguardDetailQuery.setStatus(8);
                    iSafeguardService.edit(safeguardDetailQuery);
                } else {//状态为F  => 失败
                    //优先修改订单状态
                    OrderDetailQuery query = new OrderDetailQuery();
                    query.setOrderNo(orders.getOrderNo());
                    query.setCashier("1");
                    editOrders.add(query);

                    //退款失败是修改退款状态为失败
                    editOrderCallEx.setFailCount(1);

                    SafeguardDetailQuery safeguardDetailQuery = new SafeguardDetailQuery();
                    safeguardDetailQuery.setRefundNo(orderCallEx.getRefundNo());
                    safeguardDetailQuery.setStatus(7);
                    iSafeguardService.edit(safeguardDetailQuery);
                }
                editOrderEx.add(editOrderCallEx);
            } else {
                //退款失败是修改退款状态为失败
                OrderCallException editOrderCallEx = new OrderCallException();
                BeanUtil.copyProperties(orderCallEx, editOrderCallEx);
                //退款失败是修改退款状态为失败
                editOrderCallEx.setFailCount(orderCallEx.getFailCount() + 1);

                editOrderEx.add(editOrderCallEx);

                SafeguardDetailQuery safeguardDetailQuery = new SafeguardDetailQuery();
                safeguardDetailQuery.setRefundNo(orderCallEx.getRefundNo());
                safeguardDetailQuery.setStatus(7);
                iSafeguardService.edit(safeguardDetailQuery);
            }
        }

        if (editOrders.size() != 0) {
            BaseResponse baseResponse = orderService.updateBatch(editOrders);
            if (baseResponse.getCode() == 200) {
                if (editOrderEx.size() != 0) {
                    orderCallExService.editBatchById(editOrderEx);
                }
                return ReturnT.SUCCESS;
            } else {
                return ReturnT.FAIL;
            }
        }
        return ReturnT.SUCCESS;
    }
}
